@@ -16,43 +16,10 @@ import honeybee_standards
 
 import os
 import platform
+import subprocess
 import json
 import pkgutil
 
-# Matrix matching OpenStudio version to EnergyPlus version
-# https://github.com/NREL/OpenStudio/wiki/OpenStudio-Version-Compatibility-Matrix
-OPENSTUDIO_VERSIONS = {
-    (3, 0, 0): (9, 3, 0),
-    (2, 9, 1): (9, 2, 0),
-    (2, 9, 0): (9, 2, 0),
-    (2, 8, 1): (9, 1, 0),
-    (2, 8, 0): (9, 1, 0),
-    (2, 7, 1): (9, 0, 1),
-    (2, 7, 0): (9, 0, 1),
-    (2, 6, 1): (8, 9, 0),
-    (2, 6, 0): (8, 9, 0),
-    (2, 5, 2): (8, 9, 0),
-    (2, 5, 1): (8, 9, 0),
-    (2, 5, 0): (8, 9, 0),
-    (2, 4, 3): (8, 9, 0),
-    (2, 4, 1): (8, 8, 0),
-    (2, 4, 0): (8, 8, 0),
-    (2, 3, 1): (8, 8, 0),
-    (2, 3, 0): (8, 8, 0),
-    (2, 2, 2): (8, 8, 0),
-    (2, 2, 1): (8, 7, 0),
-    (2, 2, 0): (8, 7, 0),
-    (2, 1, 2): (8, 7, 0),
-    (2, 1, 1): (8, 7, 0),
-    (2, 1, 0): (8, 7, 0),
-    (2, 0, 5): (8, 7, 0),
-    (2, 0, 4): (8, 6, 0),
-    (2, 0, 3): (8, 6, 0),
-    (2, 0, 2): (8, 6, 0),
-    (2, 0, 1): (8, 6, 0),
-    (2, 0, 0): (8, 6, 0)
-}
-
 
 class Folders(object):
     """Honeybee_energy folders.
@@ -69,10 +36,24 @@ class Folders(object):
         * openstudio_path
         * openstudio_exe
         * openstudio_version
+        * openstudio_version_str
+        * openstudio_csharp_path
+        * openstudio_lib_path
         * energyplus_path
         * energyplus_exe
         * energyplus_version
+        * energyplus_version_str
+        * energyplus_idd_path
+        * lbt_measures_path
+        * openstudio_results_measure_path
+        * view_data_measure_path
+        * inject_idf_measure_path
         * honeybee_openstudio_gem_path
+        * honeybee_adapter_path
+        * ironbug_path
+        * ironbug_exe
+        * ironbug_version
+        * ironbug_version_str
         * standards_data_folder
         * construction_lib
         * constructionset_lib
@@ -107,11 +88,23 @@ class Folders(object):
         if path:  # check that the OpenStudio executable exists in the path
             assert os.path.isfile(os_exe_file), \
                 '{} is not a valid path to an openstudio installation.'.format(path)
+            # see if the CSharp folder is installed
+            csharp_path = os.path.join(os.path.dirname(path), 'CSharp', 'openstudio')
+            self._openstudio_csharp_path = csharp_path \
+                if os.path.isdir(csharp_path) else None
+            # see if the lib path is installed
+            lib_path = os.path.join(os.path.dirname(path), 'lib')
+            self._openstudio_lib_path = lib_path \
+                if os.path.isdir(lib_path) else None
+        else:
+            self._openstudio_csharp_path = None
+            self._openstudio_lib_path = None
 
         # set the openstudio_path
         self._openstudio_path = path
         self._openstudio_exe = os_exe_file
-        self._openstudio_version = self._os_version_from_path(path)
+        self._openstudio_version = None
+        self._openstudio_version_str = None
         if path and not self.mute:
             print("Path to OpenStudio is set to: %s" % path)
 
@@ -122,13 +115,40 @@ class Folders(object):
 
     @property
     def openstudio_version(self):
-        """Get a tuple for the version of openstudio (eg. (2, 9, 1)).
+        """Get a tuple for the version of openstudio (eg. (3, 0, 1)).
 
         This will be None if the version could not be sensed or if no OpenStudio
         installation was found.
         """
+        if self._openstudio_exe and self._openstudio_version_str is None:
+            self._openstudio_version_from_cli()
         return self._openstudio_version
 
+    @property
+    def openstudio_version_str(self):
+        """Get text for the full version of openstudio (eg. "3.0.1+09b7c8a554").
+
+        This will be None if the version could not be sensed or if no OpenStudio
+        installation was found.
+        """
+        if self._openstudio_exe and self._openstudio_version_str is None:
+            self._openstudio_version_from_cli()
+        return self._openstudio_version_str
+
+    @property
+    def openstudio_csharp_path(self):
+        """Get the path to the OpenStudio CSharp folder if it exists."""
+        if self._openstudio_csharp_path is None and self._ironbug_path is not None:
+            os_dll = os.path.join(self._ironbug_path, 'OpenStudio.dll')
+            if os.path.isfile(os_dll):
+                return self._ironbug_path
+        return self._openstudio_csharp_path
+
+    @property
+    def openstudio_lib_path(self):
+        """Get the path to the OpenStudio lib folder if it exists."""
+        return self._openstudio_lib_path
+
     @property
     def energyplus_path(self):
         """Get or set the path to EnergyPlus installation folder."""
@@ -136,7 +156,6 @@ class Folders(object):
 
     @energyplus_path.setter
     def energyplus_path(self, path):
-        self._energyplus_version = None
         if not path:  # check the default installation location
             path = self._find_energyplus_folder()
         exe_name = 'energyplus.exe' if os.name == 'nt' else 'energyplus'
@@ -149,6 +168,9 @@ class Folders(object):
         # set the energyplus_path
         self._energyplus_path = path
         self._energyplus_exe = ep_exe_file
+        self._energyplus_version = None
+        self._energyplus_version_str = None
+        self._energyplus_idd_path = None
         if path and not self.mute:
             print("Path to EnergyPlus is set to: %s" % self._energyplus_path)
 
@@ -159,13 +181,86 @@ class Folders(object):
 
     @property
     def energyplus_version(self):
-        """Get a tuple for the version of energyplus (eg. (9, 2, 0)).
+        """Get a tuple for the version of energyplus (eg. (9, 3, 0)).
 
         This will be None if the version could not be sensed or if no EnergyPlus
         installation was found.
         """
+        if self._energyplus_exe and self._energyplus_version_str is None:
+            self._energyplus_version_from_cli()
         return self._energyplus_version
 
+    @property
+    def energyplus_version_str(self):
+        """Get text for the full version of energyplus (eg. "9.3.0-baff08990c").
+
+        This will be None if the version could not be sensed or if no EnergyPlus
+        installation was found.
+        """
+        if self._energyplus_exe and self._energyplus_version_str is None:
+            self._energyplus_version_from_cli()
+        return self._energyplus_version_str
+
+    @property
+    def energyplus_idd_path(self):
+        if self._energyplus_exe and self._energyplus_idd_path is None:
+            idd_path = os.path.join(self.energyplus_path, 'Energy+.idd')
+            if os.path.isfile(idd_path):
+                self._energyplus_idd_path = idd_path
+        return self._energyplus_idd_path
+
+    @property
+    def lbt_measures_path(self):
+        """Get or set the path to the the measures that ship with with Ladybug Tools.
+
+        This folder must have the following sub-folders in order to be valid:
+
+        * openstudio_results - Measure to display a clean energy use HTML report.
+        * view_data - Measure to color geometry with simulation results.
+        """
+        return self._lbt_measures_path
+
+    @lbt_measures_path.setter
+    def lbt_measures_path(self, path):
+        if not path:  # check the default locations of the lbt_measures
+            path = self._find_lbt_measures_path()
+
+        # check that the library's sub-folders exist
+        self._openstudio_results_measure_path = None
+        self._view_data_measure_path = None
+        self._inject_idf_measure_path = None
+        if path:
+            result_mea = os.path.join(path, 'openstudio_results')
+            view_mea = os.path.join(path, 'view_data')
+            idf_mea = os.path.join(path, 'inject_idf')
+            if os.path.isdir(result_mea):
+                self._openstudio_results_measure_path = result_mea
+            if os.path.isdir(view_mea):
+                self._view_data_measure_path = view_mea
+            if os.path.isdir(idf_mea):
+                self._inject_idf_measure_path = idf_mea
+
+        # set the lbt_measures_path
+        self._lbt_measures_path = path
+        if path and not self.mute:
+            print('Path to the lbt_measures is set to: '
+                  '{}'.format(self._lbt_measures_path))
+
+    @property
+    def openstudio_results_measure_path(self):
+        """Get the path to the measure that displays a clean energy use HTML report."""
+        return self._openstudio_results_measure_path
+
+    @property
+    def view_data_measure_path(self):
+        """Get the path to the measure that colors geometry with simulation results."""
+        return self._view_data_measure_path
+
+    @property
+    def inject_idf_measure_path(self):
+        """Get the path to the measure that injects IDF text."""
+        return self._inject_idf_measure_path
+
     @property
     def honeybee_openstudio_gem_path(self):
         """Get or set the path to the honeybee_openstudio_gem.
@@ -174,9 +269,9 @@ class Folders(object):
         JSON schema and OpenStudio Model schema (OSM).
         This folder must have the following sub-folders in order to be valid:
 
-        * from_honeybee - Ruby library with modules for model translation to OpenStudio.
+        * honeybee - Ruby library with modules for model translation to OpenStudio.
         * measures - folder with the actual measures that run the translation.
-        * files - folder containing the openapi schemas
+        * files - folder containing the adapter and other supporting files.
         """
         return self._honeybee_openstudio_gem_path
 
@@ -186,11 +281,14 @@ class Folders(object):
             path = self._find_honeybee_openstudio_gem_path()
 
         # check that the library's sub-folders exist
+        self._honeybee_adapter_path = None
         if path:
-            assert os.path.isdir(os.path.join(path, 'from_honeybee')), \
-                '{} lacks a "from_honeybee" folder.'.format(path)
             assert os.path.isdir(os.path.join(path, 'measures')), \
                 '{} lacks a "measures" folder.'.format(path)
+            assert os.path.isdir(os.path.join(path, 'files')), \
+                '{} lacks a "files" folder.'.format(path)
+            adapter = os.path.join(path, 'files', 'honeybee_adapter.rb')
+            self._honeybee_adapter_path = adapter if os.path.isfile(adapter) else None
 
         # set the honeybee_openstudio_gem_path
         self._honeybee_openstudio_gem_path = path
@@ -198,6 +296,66 @@ class Folders(object):
             print('Path to the honeybee_openstudio_gem is set to: '
                   '{}'.format(self._honeybee_openstudio_gem_path))
 
+    @property
+    def honeybee_adapter_path(self):
+        """Get the path to the honeybee adapter.
+
+        This adapter file is used to report the EnergyPlus simulation progress
+        when running simulations using the OpenStudio CLI.
+        """
+        return self._honeybee_adapter_path
+
+    @property
+    def ironbug_path(self):
+        """Get or set the path to an Ironbug installation folder."""
+        return self._ironbug_path
+
+    @ironbug_path.setter
+    def ironbug_path(self, path):
+        if not path:  # check the default installation location
+            path = self._find_ironbug_path_folder()
+        exe_name = 'Ironbug.Console.exe' if os.name == 'nt' else 'Ironbug.Console'
+        ib_exe_file = os.path.join(path, exe_name) if path is not None else None
+
+        if path:  # check that the Ironbug executable exists in the installation
+            assert os.path.isfile(ib_exe_file), \
+                '{} is not a valid path to an Ironbug installation.'.format(path)
+
+        # set the ironbug_path
+        self._ironbug_path = path
+        self._ironbug_exe = ib_exe_file
+        self._ironbug_version = None
+        self._ironbug_version_str = None
+        if path and not self.mute:
+            print("Path to IronBug is set to: %s" % self._ironbug_path)
+
+    @property
+    def ironbug_exe(self):
+        """Get the path to the executable Ironbug console file."""
+        return self._ironbug_exe
+
+    @property
+    def ironbug_version(self):
+        """Get a tuple for the version of Ironbug (eg. (1, 4, 3, 0)).
+
+        This will be None if the version could not be sensed or if no Ironbug
+        installation was found.
+        """
+        if self._ironbug_exe and self._ironbug_version_str is None:
+            self._ironbug_version_from_cli()
+        return self._ironbug_version
+
+    @property
+    def ironbug_version_str(self):
+        """Get text for the full version of Ironbug (eg. "v1.4.3.0 (Jan 17, 2023)").
+
+        This will be None if the version could not be sensed or if no Ironbug
+        installation was found.
+        """
+        if self._ironbug_exe and self._ironbug_version_str is None:
+            self._ironbug_version_from_cli()
+        return self._ironbug_version_str
+
     @property
     def standards_data_folder(self):
         """Get or set the path to the library of standards loaded to honeybee_energy.lib.
@@ -218,8 +376,7 @@ class Folders(object):
 
         # gather all of the sub folders underneath the master folder
         self._construction_lib, self._constructionset_lib, self._schedule_lib, \
-            self._programtype_lib, self._defaults_file = \
-            self._check_standards_folder(path, True)
+            self._programtype_lib = self._check_standards_folder(path)
 
         # set the standards_data_folder
         self._standards_data_folder = path
@@ -236,9 +393,9 @@ class Folders(object):
         * constructions - folder with honeybee JSON files for materials + constructions.
             It should have the following 4 JSON files:
             opaque_material, opaque_construction, window_material, window_construction.
-        * constructionsets - folder with honeybee JSON files of abridged ConstructionSets.
+        * constructionsets - folder with honeybee JSON files of ConstructionSets.
         * schedules - folder with honeybee JSON files for schedules.
-        * programtypes - folder with honeybee JSON files of abridged ProgramTypes.
+        * programtypes - folder with honeybee JSON files of ProgramTypes.
         """
         return tuple(self._standards_extension_folders)
 
@@ -281,6 +438,16 @@ class Folders(object):
         """Get the path to the JSON file where honeybee's defaults are loaded from."""
         return self._defaults_file
 
+    @defaults_file.setter
+    def defaults_file(self, path):
+        if not path:  # check the default location
+            path = self._find_defaults_file()
+        assert os.path.isfile(path), \
+            '{} is not a valid path to an defaults JSON file.'.format(path)
+        self._defaults_file = path
+        if not self.mute:
+            print("Path to defaults file is set to: %s" % self._defaults_file)
+
     @property
     def config_file(self):
         """Get or set the path to the config.json file from which folders are loaded.
@@ -312,9 +479,12 @@ class Folders(object):
         default_path = {
             "energyplus_path": r'',
             "openstudio_path": r'',
+            "lbt_measures_path": r'',
             "honeybee_openstudio_gem_path": r'',
+            "ironbug_path": r'',
             "standards_data_folder": r'',
-            "standards_extension_folders": []
+            "standards_extension_folders": [],
+            "defaults_file": r''
         }
 
         with open(file_path, 'r') as cfg:
@@ -334,22 +504,43 @@ class Folders(object):
         self.openstudio_path = default_path["openstudio_path"]
         self.energyplus_path = default_path["energyplus_path"]
 
-        # set the paths for the honeybee_openstudio_gem
+        # set the paths for lbt_measures and the honeybee_openstudio_gem
+        self.lbt_measures_path = default_path["lbt_measures_path"]
         self.honeybee_openstudio_gem_path = default_path["honeybee_openstudio_gem_path"]
 
-        # set path for the standards_data_folder
+        # set the paths for ironbug
+        self.ironbug_path = default_path["ironbug_path"]
+
+        # set path for the standards_data_folder and defaults_file
         self.standards_data_folder = default_path["standards_data_folder"]
+        self.defaults_file = default_path["defaults_file"]
 
         # set path for the standards_extension_folders
         self.standards_extension_folders = default_path["standards_extension_folders"]
 
+    def _find_lbt_measures_path(self):
+        """Find the lbt_measures_path in its default location.
+
+        The ladybug_tools/resources/measures folder will be checked for the
+        expected directories of measures.
+        """
+        # first, check the resources/measures folder in the ladybug_tools folder
+        lb_install = lb_config.folders.ladybug_tools_folder
+        if os.path.isdir(lb_install):
+            measure_path = os.path.join(lb_install, 'resources', 'measures')
+            if os.path.isdir(os.path.join(measure_path, 'openstudio_results')):
+                return measure_path
+        return None  # No lbt_measures is installed
+
     def _find_honeybee_openstudio_gem_path(self):
         """Find the honeybee_openstudio_gem_path in its default location.
 
-        First, the OpenStudio installation will be checked to see if there is a
-        compatible version of the measure installed for that version of OpenStudio.
-        If nothing is found there, the root of the ladybug_tools folder will be
-        checked for an honeybee_openstudio_gem directory.
+        First, the ladybug_tools/resources/measures folder will be checked for a
+        honeybee_openstudio_gem directory.
+
+        If nothing is found there, the OpenStudio installation will be checked
+        to see if there is a compatible version of the measure installed that
+        is specific for that version of OpenStudio.
         """
         # first, check the resources/measures folder in the ladybug_tools folder
         lb_install = lb_config.folders.ladybug_tools_folder
@@ -368,6 +559,20 @@ class Folders(object):
 
         return None  # No energy model measure is installed
 
+    def _find_ironbug_path_folder(self):
+        """Find the ironbug_path in its default location.
+
+        The ladybug_tools/grasshopper/ironbug folder will be checked for the
+        expected directories.
+        """
+        # first, check the grasshopper/ironbug folder in the ladybug_tools folder
+        lb_install = lb_config.folders.ladybug_tools_folder
+        if os.path.isdir(lb_install):
+            ib_path = os.path.join(lb_install, 'grasshopper', 'ironbug')
+            if os.path.isdir(ib_path):
+                return ib_path
+        return None  # No ironbug is installed
+
     def _find_energyplus_folder(self):
         """Find the most recent EnergyPlus installation in its default location.
 
@@ -389,56 +594,80 @@ class Folders(object):
                 return 0
 
         # first check for the EnergyPlus that comes with OpenStudio
-        ep_path = None
+        ep_path, ep_folders = None, []
         if self.openstudio_path is not None and os.path.isdir(os.path.join(
                 os.path.split(self.openstudio_path)[0], 'EnergyPlus')):
             ep_path = os.path.join(os.path.split(self.openstudio_path)[0], 'EnergyPlus')
-            if self.openstudio_version:
-                try:
-                    self._energyplus_version = OPENSTUDIO_VERSIONS[self.openstudio_version]
-                except KeyError:
-                    pass  # Newer version and matrix hasn't been updated
 
         # then check the default location where standalone EnergyPlus is installed
         elif os.name == 'nt':  # search the C:/ drive on Windows
-            ep_folders = ['C:\\{}'.format(f) for f in os.listdir('C:\\')
-                          if (f.lower().startswith('energyplus') and
-                              os.path.isdir('C:\\{}'.format(f)))]
+            for f in os.listdir('C:\\'):
+                f_path = 'C:\\{}'.format(f)
+                if f.lower().startswith('energyplus') and os.path.isdir(f_path):
+                    ep_folders.append(f_path)
         elif platform.system() == 'Darwin':  # search the Applications folder on Mac
-            ep_folders = ['/Applications/{}'.format(f) for f in os.listdir('/Applications/')
-                          if (f.lower().startswith('energyplus') and
-                              os.path.isdir('/Applications/{}'.format(f)))]
+            for f in os.listdir('/Applications/'):
+                f_path = '/Applications/{}'.format(f)
+                if f.lower().startswith('energyplus') and os.path.isdir(f_path):
+                    ep_folders.append(f_path)
         elif platform.system() == 'Linux':  # search the usr/local folder
-            ep_folders = ['/usr/local/{}'.format(f) for f in os.listdir('/usr/local/')
-                          if (f.lower().startswith('energyplus') and
-                              os.path.isdir('/usr/local/{}'.format(f)))]
-        else:  # unknown operating system
-            ep_folders = None
+            for f in os.listdir('/usr/local/'):
+                f_path = '/usr/local/{}'.format(f)
+                if f.lower().startswith('energyplus') and os.path.isdir(f_path):
+                    ep_folders.append(f_path)
 
         if not ep_path and not ep_folders:  # No EnergyPlus installations were found
             return None
         elif not ep_path:  # get the most recent version of energyplus that was found
             ep_path = sorted(ep_folders, key=getversion, reverse=True)[0]
-            self._energyplus_version = self._ep_version_from_path(ep_path)
         return ep_path
 
-    @staticmethod
-    def _ep_version_from_path(ep_path):
-        """Extract a tuple for the version of EnergyPlus from the path.
+    def _openstudio_version_from_cli(self):
+        """Set this object's OpenStudio version by making a call to OpenStudio CLI."""
+        cmds = [self.openstudio_exe, 'openstudio_version']
+        use_shell = True if os.name == 'nt' else False
+        process = subprocess.Popen(cmds, stdout=subprocess.PIPE, shell=use_shell)
+        stdout = process.communicate()
+        try:
+            base_str = str(stdout[0]).replace("b'", '').replace(r"\r\n'", '')
+            self._openstudio_version_str = base_str
+            ver_nums = self._openstudio_version_str.split('+')[0].split('.')
+            ver_nums[-1] = ver_nums[-1].split('-')[0] \
+                if '-' in ver_nums[-1] else ver_nums[-1]
+            self._openstudio_version = tuple(int(i) for i in ver_nums)
+        except Exception:
+            pass  # failed to parse the version into integers
 
-        Args:
-            ep_path: File directory to EnergyPlus.
+    def _energyplus_version_from_cli(self):
+        """Set this object's EnergyPlus version by making a call to EnergyPlus CLI."""
+        cmds = [self.energyplus_exe, '--version']
+        use_shell = True if os.name == 'nt' else False
+        process = subprocess.Popen(cmds, stdout=subprocess.PIPE, shell=use_shell)
+        stdout = process.communicate()
+        try:
+            base_str = str(stdout[0]).replace("b'", '').replace(r"\r\n'", '')
+            self._energyplus_version_str = base_str.split(',')[1].split(' ')[-1]
+            ver_nums = self._energyplus_version_str.split('-')[0].split('.')
+            self._energyplus_version = tuple(int(i) for i in ver_nums)
+        except Exception:
+            pass  # failed to parse the version into integers
 
-        Returns:
-            Tuple of integers for the version in case of success.
-            None, None in case of failure.
-        """
-        if not ep_path:
-            return None
-        ver = ''.join(s for s in ep_path if (s.isdigit() or s == '-'))
-        if ver:  # version was found in the file path name
-            return tuple(int(d) for d in ver.split('-'))
-        return None
+    def _ironbug_version_from_cli(self):
+        """Set this object's Ironbug version by making a call to Ironbug CLI."""
+        cmds = [self.ironbug_exe, '--version']
+        use_shell = True if os.name == 'nt' else False
+        process = subprocess.Popen(cmds, stdout=subprocess.PIPE, shell=use_shell)
+        stdout = process.communicate()
+        base_str = str(stdout[0]).replace("b'", '')
+        base_str = base_str.replace("\r\n", '__').replace(r"\r\n", '__') \
+            if os.name == 'nt' else base_str.replace("\n", '__').replace(r"\n", '__')
+        try:
+            base_str = base_str.split('__')[-2]
+            self._ironbug_version_str = base_str
+            ver_nums = base_str.split(' ')[0].replace('v', '').split('.')
+            self._ironbug_version = tuple(int(i) for i in ver_nums)
+        except Exception:
+            pass  # failed to parse the version into integers
 
     @staticmethod
     def _find_openstudio_folder():
@@ -461,25 +690,29 @@ class Folders(object):
         lb_install = lb_config.folders.ladybug_tools_folder
         os_folders = []
         if os.path.isdir(lb_install):
-            os_folders = [os.path.join(lb_install, f) for f in os.listdir(lb_install)
-                          if (f.lower().startswith('openstudio') and
-                              os.path.isdir(os.path.join(lb_install, f)))]
+            for f in os.listdir(lb_install):
+                f_path = os.path.join(lb_install, f)
+                if f.lower().startswith('openstudio') and os.path.isdir(f_path):
+                    os_folders.append(f_path)
 
         # then check the default installation folders
         if len(os_folders) != 0 and os.path.isdir(os.path.join(os_folders[0], 'bin')):
             pass  # we found a version of openstudio in the ladybug_tools folder
         elif os.name == 'nt':  # search the C:/ drive on Windows
-            os_folders = ['C:\\{}'.format(f) for f in os.listdir('C:\\')
-                          if (f.lower().startswith('openstudio') and
-                              os.path.isdir('C:\\{}'.format(f)))]
+            for f in os.listdir('C:\\'):
+                f_path = 'C:\\{}'.format(f)
+                if f.lower().startswith('openstudio') and os.path.isdir(f_path):
+                    os_folders.append(f_path)
         elif platform.system() == 'Darwin':  # search the Applications folder on Mac
-            os_folders = ['/Applications/{}'.format(f) for f in os.listdir('/Applications/')
-                          if (f.lower().startswith('openstudio') and
-                              os.path.isdir('/Applications/{}'.format(f)))]
+            for f in os.listdir('/Applications/'):
+                f_path = '/Applications/{}'.format(f)
+                if f.lower().startswith('openstudio') and os.path.isdir(f_path):
+                    os_folders.append(f_path)
         elif platform.system() == 'Linux':  # search the usr/local folder
-            os_folders = ['/usr/local/{}'.format(f) for f in os.listdir('/usr/local/')
-                          if (f.lower().startswith('openstudio') and
-                              os.path.isdir('/usr/local/{}'.format(f)))]
+            for f in os.listdir('/usr/local/'):
+                f_path = '/usr/local/{}'.format(f)
+                if f.lower().startswith('openstudio') and os.path.isdir(f_path):
+                    os_folders.append(f_path)
         else:  # unknown operating system
             os_folders = None
 
@@ -491,43 +724,34 @@ class Folders(object):
 
         return os.path.join(os_path, 'bin')
 
-    @staticmethod
-    def _os_version_from_path(os_path):
-        """Extract a tuple for the version of OpenStudio from the path.
-
-        Args:
-            os_path: File directory to OpenStudio.
-
-        Returns:
-            Tuple of integers for the version in case of success.
-            None, None in case of failure.
-        """
-        if not os_path:
-            return None
-        try:
-            ver = ''.join(s for s in os_path if (s.isdigit() or s == '.'))
-            if ver:  # version was found in the file path name
-                return tuple(int(d) for d in ver.split('.'))
-        except ValueError:
-            pass  # folder starting with 'openstudio' and no version
-        return None
-
     @staticmethod
     def _find_standards_data_folder():
         """Find the user template library in its default location.
 
-        The ladybug_tools/resources/standards/honeybee_standards folder will be
-        checked first, which can contain libraries that are not overwritten
-        with the update of the honeybee_energy package. If no such folder is found,
-        this method defaults to the lib/library/ folder within this package.
+        The %AppData%/ladybug_tools/standards folder will be checked first, which
+        can contain libraries that are not overwritten with the update of the
+        honeybee_energy package. If this is not found, the ladybug_tools/resources/
+        standards/honeybee_standards folder will be checked next,  If no such folder
+        is found, this method defaults to the lib/library/ folder within this package.
         """
-        # first check the ladybug_tools installation folder were permanent lib is
+        # first check if there's a user-defined folder in AppData
+        app_folder = os.getenv('APPDATA')
+        if app_folder is not None:
+            lib_folder = os.path.join(app_folder, 'ladybug_tools', 'standards')
+            if os.path.isdir(lib_folder):
+                return lib_folder
+
+        # then check the ladybug_tools installation folder were permanent lib is
         lb_install = lb_config.folders.ladybug_tools_folder
         if os.path.isdir(lb_install):
             lib_folder = os.path.join(
                 lb_install, 'resources', 'standards', 'honeybee_standards')
             if os.path.isdir(lib_folder):
-                return lib_folder
+                try:
+                    Folders._check_standards_folder(lib_folder)
+                    return lib_folder
+                except AssertionError:  # the folder is not valid
+                    pass
 
         # default to the library folder that installs with this Python package
         return os.path.join(os.path.dirname(honeybee_standards.__file__))
@@ -565,17 +789,16 @@ class Folders(object):
         return standards_extensions
 
     @staticmethod
-    def _check_standards_folder(path, check_defaults=False):
+    def _check_standards_folder(path):
         """Check that a standards data sub-folders exist."""
         if not path:  # first check that a path exists
-            return [None] * 5
+            return [None] * 4
 
         # gather all of the sub folders underneath the master folder
         _construction_lib = os.path.join(path, 'constructions')
         _constructionset_lib = os.path.join(path, 'constructionsets')
         _schedule_lib = os.path.join(path, 'schedules')
         _programtype_lib = os.path.join(path, 'programtypes')
-        _energy_default = os.path.join(path, 'energy_default.json')
 
         assert os.path.isdir(_construction_lib), \
             '{} lacks a "constructions" folder.'.format(path)
@@ -586,12 +809,23 @@ class Folders(object):
         assert os.path.isdir(_programtype_lib), \
             '{} lacks a "programtypes" folder.'.format(path)
 
-        if check_defaults:
-            assert os.path.isfile(_energy_default), \
-                '{} lacks a "energy_default.json."'.format(path)
+        return _construction_lib, _constructionset_lib, _schedule_lib, _programtype_lib
 
-        return _construction_lib, _constructionset_lib, _schedule_lib, \
-            _programtype_lib, _energy_default
+    @staticmethod
+    def _find_defaults_file():
+        """Find the energy default JSON in its default locations."""
+        # first check the ladybug_tools installation folder were permanent lib is
+        lb_install = lb_config.folders.ladybug_tools_folder
+        if os.path.isdir(lb_install):
+            def_file = os.path.join(
+                lb_install, 'resources', 'standards', 'honeybee_standards',
+                'energy_default.json')
+            if os.path.isfile(def_file):
+                return def_file
+
+        # default to the library folder that installs with this Python package
+        return os.path.join(
+            os.path.dirname(honeybee_standards.__file__), 'energy_default.json')
 
 
 """Object possesing all key folders within the configuration."""
