@@ -2,16 +2,17 @@
 """Opaque Construction."""
 from __future__ import division
 
-from ._base import _ConstructionBase
-from ..material.dictutil import dict_to_material
-from ..material._base import _EnergyMaterialOpaqueBase
-from ..material.opaque import EnergyMaterial, EnergyMaterialNoMass
-from ..reader import parse_idf_string
+import re
 
 from honeybee._lockable import lockable
 
-import re
-import os
+from ._base import _ConstructionBase
+from ..material.dictutil import dict_to_material
+from ..material._base import _EnergyMaterialOpaqueBase
+from ..material.opaque import EnergyMaterial, EnergyMaterialNoMass, \
+    EnergyMaterialVegetation
+from ..reader import parse_idf_string, clean_idf_file_contents
+from ..properties.extension import OpaqueConstructionProperties
 
 
 @lockable
@@ -36,6 +37,9 @@ class OpaqueConstruction(_ConstructionBase):
         * u_factor
         * r_factor
         * is_symmetric
+        * has_frame
+        * has_shade
+        * is_dynamic
         * inside_emissivity
         * inside_solar_reflectance
         * inside_visible_reflectance
@@ -44,9 +48,54 @@ class OpaqueConstruction(_ConstructionBase):
         * outside_visible_reflectance
         * mass_area_density
         * area_heat_capacity
+        * thickness
+        * inside_material
+        * outside_material
+        * user_data
+        * properties
     """
     __slots__ = ()
 
+    def __init__(self, identifier, materials):
+        """Initialize opaque construction."""
+        _ConstructionBase.__init__(self, identifier, materials)
+        self._properties = OpaqueConstructionProperties(self)
+
+    @classmethod
+    def from_simple_parameters(
+        cls, identifier, r_value, roughness='MediumRough',
+        thermal_absorptance=0.9, solar_absorptance=0.7, visible_absorptance=None
+    ):
+        """Create a no-mass OpaqueConstruction from a specification of simple parameters.
+
+        The result will have a single EnergyMaterialNoMass layer that is derived
+        from the input parameters.
+
+        Args:
+            identifier: Text string for a unique construction ID.
+                Must be <90 characters and not contain any EnergyPlus special
+                characters. This will be used to identify the object across a model
+                and in the exported IDF.
+            r_value: Number for the R-value of the construction [m2-K/W].
+            roughness: Text describing the relative roughness of the construction.
+                Must be one of the following: 'VeryRough', 'Rough', 'MediumRough',
+                'MediumSmooth', 'Smooth', 'VerySmooth'. Default: 'MediumRough'.
+            thermal_absorptance: A number between 0 and 1 for the fraction of
+                incident long wavelength radiation that is absorbed by the construction.
+                Default: 0.9.
+            solar_absorptance: A number between 0 and 1 for the fraction of incident
+                solar radiation absorbed by the construction. Default: 0.7.
+            visible_absorptance: A number between 0 and 1 for the fraction of incident
+                visible wavelength radiation absorbed by the construction.
+                Default value is None, which will use the same value as the
+                solar_absorptance.
+        """
+        mat_layer = EnergyMaterialNoMass(
+            '{} Material'.format(identifier), r_value, roughness,
+            thermal_absorptance, solar_absorptance, visible_absorptance
+        )
+        return cls(identifier, (mat_layer,))
+
     @property
     def materials(self):
         """Get or set the list of materials in the construction (outside to inside).
@@ -63,31 +112,34 @@ class OpaqueConstruction(_ConstructionBase):
         except TypeError:
             raise TypeError('Expected list or tuple for construction materials. '
                             'Got {}'.format(type(mats)))
-        for mat in mats:
+        for i, mat in enumerate(mats):
             assert isinstance(mat, _EnergyMaterialOpaqueBase), 'Expected opaque energy' \
                 ' material for construction. Got {}.'.format(type(mat))
+            if isinstance(mat, EnergyMaterialVegetation):
+                assert i == 0, 'Vegetation material layer must bet the first ' \
+                    '(exterior) layer in the construction.'
         assert len(mats) > 0, 'Construction must possess at least one material.'
         assert len(mats) <= 10, 'Opaque Construction cannot have more than 10 materials.'
         self._materials = mats
 
     @property
     def inside_solar_reflectance(self):
-        """"The solar reflectance of the inside face of the construction."""
+        """The solar reflectance of the inside face of the construction."""
         return 1 - self.materials[-1].solar_absorptance
 
     @property
     def inside_visible_reflectance(self):
-        """"The visible reflectance of the inside face of the construction."""
+        """The visible reflectance of the inside face of the construction."""
         return 1 - self.materials[-1].visible_absorptance
 
     @property
     def outside_solar_reflectance(self):
-        """"The solar reflectance of the outside face of the construction."""
+        """The solar reflectance of the outside face of the construction."""
         return 1 - self.materials[0].solar_absorptance
 
     @property
     def outside_visible_reflectance(self):
-        """"The visible reflectance of the outside face of the construction."""
+        """The visible reflectance of the outside face of the construction."""
         return 1 - self.materials[0].visible_absorptance
 
     @property
@@ -97,7 +149,7 @@ class OpaqueConstruction(_ConstructionBase):
 
     @property
     def area_heat_capacity(self):
-        """The heat capacity per unit area of the construction [kg/K-m2]."""
+        """The heat capacity per unit area of the construction [J/K-m2]."""
         return sum(tuple(mat.area_heat_capacity for mat in self.materials))
 
     @property
@@ -105,29 +157,48 @@ class OpaqueConstruction(_ConstructionBase):
         """Thickness of the construction [m]."""
         thickness = 0
         for mat in self.materials:
-            if isinstance(mat, EnergyMaterial):
-                thickness += mat.thickness
+            thickness += mat.thickness
         return thickness
 
+    @property
+    def inside_material(self):
+        """The the inside material layer of the construction.
+
+        Useful for checking that an asymmetric construction is correctly assigned.
+        """
+        return self.materials[-1]
+
+    @property
+    def outside_material(self):
+        """The the outside material layer of the construction.
+
+        Useful for checking that an asymmetric construction is correctly assigned.
+        """
+        return self.materials[0]
+
     def temperature_profile(self, outside_temperature=-18, inside_temperature=21,
-                            outside_wind_speed=6.7, height=1.0, angle=90.0,
-                            pressure=101325):
+                            outside_wind_speed=6.7, solar_irradiance=0,
+                            height=1.0, angle=90.0, pressure=101325):
         """Get a list of temperatures at each material boundary across the construction.
 
         Args:
-            outside_temperature: The temperature on the outside of the construction [C].
-                Default is -18, which is consistent with NFRC 100-2010.
-            inside_temperature: The temperature on the inside of the construction [C].
-                Default is 21, which is consistent with NFRC 100-2010.
+            outside_temperature: The temperature on the outside of the
+                construction [C]. (Default: -18, consistent with NFRC 100-2010).
+            inside_temperature: The temperature on the inside of the
+                construction [C]. (Default: 21, consistent with NFRC 100-2010).
             wind_speed: The average outdoor wind speed [m/s]. This affects outdoor
-                convective heat transfer coefficient. Default is 6.7 m/s.
-            height: An optional height for the surface in meters. Default is 1.0 m.
+                convective heat transfer coefficient. (Default: 6.7 m/s).
+            solar_irradiance: An optional value for solar irradiance that is incident
+                on the front (exterior) of the construction [W/m2]. (Default: 0 W/m2).
+            height: An optional height for the surface in meters. (Default: 1.0 m).
             angle: An angle in degrees between 0 and 180.
-            * 0 = A horizontal surface with the outside boundary on the bottom.
-            * 90 = A vertical surface
-            * 180 = A horizontal surface with the outside boundary on the top.
-            pressure: The average pressure of in Pa.
-                Default is 101325 Pa for standard pressure at sea level.
+
+                * 0 = A horizontal surface with the outside boundary on the top.
+                * 90 = A vertical surface
+                * 180 = A horizontal surface with the outside boundary on the bottom.
+
+            pressure: The average pressure of in Pa. (Default: 101325 Pa for
+                standard pressure at sea level).
 
         Returns:
             A tuple with two elements
@@ -144,8 +215,16 @@ class OpaqueConstruction(_ConstructionBase):
                 The sum of this list is the R-factor for this construction given
                 the input parameters.
         """
+        # reverse the angle if the outside temperature is greater than the inside one
         if angle != 90 and outside_temperature > inside_temperature:
             angle = abs(180 - angle)
+
+        # compute delta temperature from solar irradiance if applicable
+        heat_gen = None
+        if solar_irradiance != 0:
+            heat_gen = self.materials[0].solar_absorptance * solar_irradiance
+
+        # use the r-values to get the temperature profile
         in_r_init = 1 / self.in_h_simple()
         r_values = [1 / self.out_h(outside_wind_speed, outside_temperature + 273.15)] + \
             [mat.r_value for mat in self.materials] + [in_r_init]
@@ -154,7 +233,7 @@ class OpaqueConstruction(_ConstructionBase):
         r_values[-1] = 1 / self.in_h(inside_temperature - (in_delta_t / 2) + 273.15,
                                      in_delta_t, height, angle, pressure)
         temperatures = self._temperature_profile_from_r_values(
-            r_values, outside_temperature, inside_temperature)
+            r_values, outside_temperature, inside_temperature, heat_gen)
         return temperatures, r_values
 
     @classmethod
@@ -169,7 +248,7 @@ class OpaqueConstruction(_ConstructionBase):
         materials_dict = cls._idf_materials_dictionary(ep_mat_strings)
         ep_strs = parse_idf_string(idf_string)
         try:
-            materials = [materials_dict[mat] for mat in ep_strs[1:]]
+            materials = [materials_dict[mat.upper()] for mat in ep_strs[1:]]
         except KeyError as e:
             raise ValueError('Failed to find {} in the input ep_mat_strings.'.format(e))
         return cls(ep_strs[0], materials)
@@ -190,19 +269,20 @@ class OpaqueConstruction(_ConstructionBase):
             "type": 'OpaqueConstruction',
             "identifier": 'Generic Brick Wall R-10',
             "display_name": 'Brick Wall',
-            "layers": [],  # list of material identifiers (from outside to inside)
-            "materials": []  # list of unique material objects
+            "materials": []  # list of unique material objects (from outside to inside)
             }
         """
         assert data['type'] == 'OpaqueConstruction', \
             'Expected OpaqueConstruction. Got {}.'.format(data['type'])
-        materials = {}
-        for mat in data['materials']:
-            materials[mat['identifier']] = dict_to_material(mat)
-        mat_layers = [materials[mat_id] for mat_id in data['layers']]
+        mat_layers = cls._old_schema_materials(data) if 'layers' in data else \
+            [dict_to_material(mat) for mat in data['materials']]
         new_obj = cls(data['identifier'], mat_layers)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
         return new_obj
 
     @classmethod
@@ -220,15 +300,24 @@ class OpaqueConstruction(_ConstructionBase):
             "type": 'OpaqueConstructionAbridged',
             "identifier": 'Generic Brick Wall R-10',
             "display_name": 'Brick Wall',
-            "layers": [],  # list of material identifiers (from outside to inside)
+            "materials": [],  # list of material identifiers (from outside to inside)
             }
         """
         assert data['type'] == 'OpaqueConstructionAbridged', \
             'Expected OpaqueConstructionAbridged. Got {}.'.format(data['type'])
-        mat_layers = [materials[mat_id] for mat_id in data['layers']]
+        # handle old schema definition before May 2021 (used layers instead of materials)
+        mat_key = 'layers' if 'layers' in data else 'materials'
+        try:
+            mat_layers = [materials[mat_id] for mat_id in data[mat_key]]
+        except KeyError as e:
+            raise ValueError('Failed to find {} in materials.'.format(e))
         new_obj = cls(data['identifier'], mat_layers)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
         return new_obj
 
     def to_idf(self):
@@ -244,19 +333,19 @@ class OpaqueConstruction(_ConstructionBase):
         return self._generate_idf_string('opaque', self.identifier, self.materials)
 
     def to_radiance_solar_interior(self, specularity=0.0):
-        """Honeybee Radiance material with the interior solar reflectance."""
+        """Honeybee Radiance modifier with the interior solar reflectance."""
         return self.materials[-1].to_radiance_solar(specularity)
 
     def to_radiance_visible_interior(self, specularity=0.0):
-        """Honeybee Radiance material with the interior visible reflectance."""
+        """Honeybee Radiance modifier with the interior visible reflectance."""
         return self.materials[-1].to_radiance_visible(specularity)
 
     def to_radiance_solar_exterior(self, specularity=0.0):
-        """Honeybee Radiance material with the exterior solar reflectance."""
+        """Honeybee Radiance modifier with the exterior solar reflectance."""
         return self.materials[0].to_radiance_solar(specularity)
 
     def to_radiance_visible_exterior(self, specularity=0.0):
-        """Honeybee Radiance material with the exterior visible reflectance."""
+        """Honeybee Radiance modifier with the exterior visible reflectance."""
         return self.materials[0].to_radiance_visible(specularity)
 
     def to_dict(self, abridged=False):
@@ -270,11 +359,15 @@ class OpaqueConstruction(_ConstructionBase):
         base = {'type': 'OpaqueConstruction'} if not \
             abridged else {'type': 'OpaqueConstructionAbridged'}
         base['identifier'] = self.identifier
-        base['layers'] = self.layers
-        if not abridged:
-            base['materials'] = [m.to_dict() for m in self.unique_materials]
+        base['materials'] = self.layers if abridged else \
+            [m.to_dict() for m in self.materials]
         if self._display_name is not None:
             base['display_name'] = self.display_name
+        if self._user_data is not None:
+            base['user_data'] = self.user_data
+        prop_dict = self.properties.to_dict()
+        if prop_dict is not None:
+            base['properties'] = prop_dict
         return base
 
     @staticmethod
@@ -294,16 +387,16 @@ class OpaqueConstruction(_ConstructionBase):
             -   materials: A list of all opaque materials in the IDF file as
                 honeybee_energy EnergyMaterial objects.
         """
-        # check the file
-        assert os.path.isfile(idf_file), 'Cannot find an idf file at {}'.format(idf_file)
-        with open(idf_file, 'r') as ep_file:
-            file_contents = ep_file.read()
+        # read the file and remove lines of comments
+        file_contents = clean_idf_file_contents(idf_file)
         # extract all of the opaque material objects
         mat_pattern1 = re.compile(r"(?i)(Material,[\s\S]*?;)")
         mat_pattern2 = re.compile(r"(?i)(Material:NoMass,[\s\S]*?;)")
         mat_pattern3 = re.compile(r"(?i)(Material:AirGap,[\s\S]*?;)")
+        mat_pattern4 = re.compile(r"(?i)(Material:RoofVegetation,[\s\S]*?;)")
         material_str = mat_pattern1.findall(file_contents) + \
-            mat_pattern2.findall(file_contents) + mat_pattern3.findall(file_contents)
+            mat_pattern2.findall(file_contents) + mat_pattern3.findall(file_contents) + \
+            mat_pattern4.findall(file_contents)
         materials_dict = OpaqueConstruction._idf_materials_dictionary(material_str)
         materials = list(materials_dict.values())
         # extract all of the construction objects
@@ -313,10 +406,10 @@ class OpaqueConstruction(_ConstructionBase):
         constructions = []
         for constr in constr_props:
             try:
-                constr_mats = [materials_dict[mat] for mat in constr[1:]]
+                constr_mats = [materials_dict[mat.upper()] for mat in constr[1:]]
                 constructions.append(OpaqueConstruction(constr[0], constr_mats))
-            except KeyError:
-                pass  # the construction is a window construction
+            except (KeyError, AssertionError):
+                pass  # the construction is probably a window construction
         return constructions, materials
 
     @staticmethod
@@ -325,14 +418,36 @@ class OpaqueConstruction(_ConstructionBase):
         materials_dict = {}
         for mat_str in ep_mat_strings:
             mat_str = mat_str.strip()
-            if mat_str.startswith('Material:NoMass,'):
-                mat_obj = EnergyMaterialNoMass.from_idf(mat_str)
-                materials_dict[mat_obj.identifier] = mat_obj
-            elif mat_str.startswith('Material,'):
+            if mat_str.startswith('Material,'):
                 mat_obj = EnergyMaterial.from_idf(mat_str)
-                materials_dict[mat_obj.identifier] = mat_obj
+                materials_dict[mat_obj.identifier.upper()] = mat_obj
+            elif mat_str.startswith('Material:NoMass,'):
+                mat_obj = EnergyMaterialNoMass.from_idf(mat_str)
+                materials_dict[mat_obj.identifier.upper()] = mat_obj
+            elif mat_str.startswith('Material:AirGap,'):
+                mat_obj = EnergyMaterialNoMass.from_idf_air_gap(mat_str)
+                materials_dict[mat_obj.identifier.upper()] = mat_obj
+            elif mat_str.startswith('Material:RoofVegetation,'):
+                mat_obj = EnergyMaterialVegetation.from_idf(mat_str)
+                materials_dict[mat_obj.identifier.upper()] = mat_obj
         return materials_dict
 
+    @staticmethod
+    def _old_schema_materials(data):
+        """Get material objects from an old schema definition of OpaqueConstruction.
+
+        The schema is from before May 2021 and this method should eventually be removed.
+        """
+        materials = {}
+        for mat in data['materials']:
+            materials[mat['identifier']] = dict_to_material(mat)
+        try:
+            mat_layers = [materials[mat_id] for mat_id in data['layers']]
+        except KeyError as e:
+            raise ValueError(
+                'Failed to find {} in opaque construction materials.'.format(e))
+        return mat_layers
+
     def __repr__(self):
         """Represent opaque energy construction."""
         return self._generate_idf_string('opaque', self.identifier, self.materials)
