@@ -11,6 +11,8 @@ from ..schedule.dictutil import dict_to_schedule
 from ..schedule.ruleset import ScheduleRuleset
 from ..schedule.fixedinterval import ScheduleFixedInterval
 from ..writer import generate_idf_string
+from ..properties.extension import WindowConstructionShadeProperties
+
 
 from honeybee._lockable import lockable
 from honeybee.typing import valid_ep_string
@@ -36,34 +38,34 @@ class WindowConstructionShade(object):
         shade_location: Text to indicate where in the window assembly the shade_material
             is located. (Default: "Interior"). Choose from the following 3 options:
 
-                * Interior
-                * Between
-                * Exterior
+            * Interior
+            * Between
+            * Exterior
 
             Note that the WindowConstruction must have at least one gas gap to use
             the "Between" option. Also note that, for a WindowConstruction with more
-            than one gas gap, the "Between" option defalts to using the inner gap
+            than one gas gap, the "Between" option defaults to using the inner gap
             as this is the only option that EnergyPlus supports.
         control_type: Text to indicate how the shading device is controlled, which
              determines when the shading is “on” or “off.” (Default: "AlwaysOn").
              Choose from the options below (units for the values of the corresponding
              setpoint are noted in parentheses next to each option):
 
-                * AlwaysOn
-                * OnIfHighSolarOnWindow (W/m2)
-                * OnIfHighHorizontalSolar (W/m2)
-                * OnIfHighOutdoorAirTemperature (C)
-                * OnIfHighZoneAirTemperature (C)
-                * OnIfHighZoneCooling (W)
-                * OnNightIfLowOutdoorTempAndOffDay (C)
-                * OnNightIfLowInsideTempAndOffDay (C)
-                * OnNightIfHeatingAndOffDay (W)
+            * AlwaysOn
+            * OnIfHighSolarOnWindow (W/m2)
+            * OnIfHighHorizontalSolar (W/m2)
+            * OnIfHighOutdoorAirTemperature (C)
+            * OnIfHighZoneAirTemperature (C)
+            * OnIfHighZoneCooling (W)
+            * OnNightIfLowOutdoorTempAndOffDay (C)
+            * OnNightIfLowInsideTempAndOffDay (C)
+            * OnNightIfHeatingAndOffDay (W)
 
         setpoint: A number that corresponds to the specified control_type. This can
             be a value in (W/m2), (C) or (W) depending upon the control type.
         schedule: An optional ScheduleRuleset or ScheduleFixedInterval to be applied
             on top of the control_type. If None, the control_type will govern all
-            behavior of the construction.
+            behavior of the construction. (Default: None).
 
     Properties:
         * identifier
@@ -77,29 +79,48 @@ class WindowConstructionShade(object):
         * materials
         * layers
         * unique_materials
+        * frame
         * r_value
         * u_value
         * u_factor
         * r_factor
         * is_symmetric
         * is_switchable_glazing
+        * has_frame
         * has_shade
+        * is_dynamic
         * inside_emissivity
         * outside_emissivity
+        * solar_transmittance
+        * visible_transmittance
+        * shgc
         * thickness
         * glazing_count
         * gap_count
+        * is_groupable
+        * is_zone_groupable
+        * inside_material
+        * outside_material
+        * user_data
+        * properties
     """
 
     __slots__ = ('_identifier', '_display_name', '_window_construction',
                  '_shade_material', '_shade_location', '_control_type',
-                 '_setpoint', '_schedule', '_between_gap', '_locked')
+                 '_setpoint', '_schedule', '_between_gap', '_locked', '_user_data',
+                 "_properties", )
     SHADE_LOCATIONS = ('Interior', 'Between', 'Exterior')
     CONTROL_TYPES = (
         'AlwaysOn', 'OnIfHighSolarOnWindow', 'OnIfHighHorizontalSolar',
         'OnIfHighOutdoorAirTemperature', 'OnIfHighZoneAirTemperature',
         'OnIfHighZoneCooling', 'OnNightIfLowOutdoorTempAndOffDay',
         'OnNightIfLowInsideTempAndOffDay', 'OnNightIfHeatingAndOffDay')
+    GROUPABLE_TYPES = (
+        'AlwaysOn', 'OnIfHighHorizontalSolar', 'OnIfHighOutdoorAirTemperature',
+        'OnNightIfLowOutdoorTempAndOffDay')
+    ROOM_GROUPABLE_TYPES = (
+        'OnIfHighZoneAirTemperature', 'OnIfHighZoneCooling',
+        'OnNightIfLowInsideTempAndOffDay', 'OnNightIfHeatingAndOffDay')
 
     def __init__(self, identifier, window_construction, shade_material,
                  shade_location='Interior', control_type='AlwaysOn',
@@ -109,7 +130,7 @@ class WindowConstructionShade(object):
         self.identifier = identifier
         self._display_name = None
         self._between_gap = None  # will be used if 'Between' option is used
-
+        self._user_data = None
         # check that the window construction, shade, and shade location are compatible
         assert isinstance(window_construction, WindowConstruction), \
             'Expected WindowConstruction for WindowConstructionShade. ' \
@@ -120,7 +141,7 @@ class WindowConstructionShade(object):
             'Got {}.'.format(type(shade_material))
         assert shade_location in self.SHADE_LOCATIONS, \
             'Invalid input "{}" for shade location.  Must be one ' \
-            'of the following:\n'.format(shade_location, self.SHADE_LOCATIONS)
+            'of the following:\n{}'.format(shade_location, self.SHADE_LOCATIONS)
         if isinstance(shade_material, EnergyWindowMaterialGlazing):
             ext_pane = window_construction[0]
             assert not isinstance(ext_pane, EnergyWindowMaterialSimpleGlazSys), \
@@ -134,7 +155,8 @@ class WindowConstructionShade(object):
             if isinstance(shade_material, EnergyWindowMaterialBlind):
                 assert shade_material.slat_width < int_gap.thickness, \
                     'Blind slat_width must be less than the width of the gap in which ' \
-                    'it sits. {} > {}.'.format(shade_material.slat_width, int_gap.thickness)
+                    'it sits. {} > {}.'.format(
+                        shade_material.slat_width, int_gap.thickness)
             shd_thick = 0 if isinstance(shade_material, EnergyWindowMaterialBlind) \
                 else shade_material.thickness
             gap_thick = (int_gap.thickness - shd_thick) / 2
@@ -143,7 +165,7 @@ class WindowConstructionShade(object):
             # create the split gap material to be used on either side of the shade
             between_int_gap = int_gap.duplicate()
             between_int_gap.identifier = \
-                '{}_Split{}'.format(int_gap.identifier, round(gap_thick, 4))
+                '{}_Split{}'.format(int_gap.identifier, round(gap_thick, 3))
             between_int_gap.thickness = gap_thick
             self._between_gap = between_int_gap
         window_construction.lock()  # lock to avoid illegal shade/material combinations
@@ -154,10 +176,11 @@ class WindowConstructionShade(object):
         # assign the control type, setpoint and schedule
         assert control_type in self.CONTROL_TYPES, \
             'Invalid input "{}" for shading control type.' \
-            ' Must be one of the following:\n'.format(control_type, self.CONTROL_TYPES)
+            ' Must be one of the following:\n{}'.format(control_type, self.CONTROL_TYPES)
         self._control_type = control_type
         self.setpoint = setpoint
         self.schedule = schedule
+        self._properties = WindowConstructionShadeProperties(self)
 
     @property
     def identifier(self):
@@ -180,10 +203,12 @@ class WindowConstructionShade(object):
 
     @display_name.setter
     def display_name(self, value):
-        try:
-            self._display_name = str(value)
-        except UnicodeEncodeError:  # Python 2 machine lacking the character set
-            self._display_name = value  # keep it as unicode
+        if value is not None:
+            try:
+                value = str(value)
+            except UnicodeEncodeError:  # Python 2 machine lacking the character set
+                pass  # keep it as unicode
+        self._display_name = value
 
     @property
     def window_construction(self):
@@ -200,7 +225,7 @@ class WindowConstructionShade(object):
         """Get text to indicate where in the construction the shade_material is located.
 
         This will be either "Interior", "Between" or "Exterior". Note that, for a
-        WindowConstruction with more than one gas gap, the "Between" option defalts
+        WindowConstruction with more than one gas gap, the "Between" option defaults
         to using the inner gap as this is the only option that EnergyPlus supports.
         """
         return self._shade_location
@@ -227,7 +252,7 @@ class WindowConstructionShade(object):
     def control_type(self, value):
         assert value in self.CONTROL_TYPES, \
             'Invalid input "{}" for shading control type.' \
-            ' Must be one of the following:\n'.format(value, self.CONTROL_TYPES)
+            ' Must be one of the following:\n{}'.format(value, self.CONTROL_TYPES)
         if value != 'AlwaysOn':
             assert self._setpoint is not None, 'Control setpoint must not ' \
                 'be None to use "{}" control type.'.format(value)
@@ -303,7 +328,7 @@ class WindowConstructionShade(object):
 
     @property
     def layers(self):
-        """A list of material identifiers in the construction (outside to inside).
+        """Get a list of material identifiers in the construction (outside to inside).
 
         This will include the shade material layer in its correct position.
         """
@@ -311,15 +336,21 @@ class WindowConstructionShade(object):
 
     @property
     def unique_materials(self):
-        """A set of only unique material objects in the construction.
+        """Get a list of only unique material objects in the construction.
 
         This will include the shade material layer. It will include both types of glass
-        layers if the consruction is a switchable glazing.
+        layers if the construction is a switchable glazing.
         """
         if self.is_switchable_glazing:
-            return list(set(self._window_construction.materials + (self.shade_material,)))
+            return list(set(
+                self._window_construction.materials + (self.shade_material,)))
         return list(set(self.materials))
 
+    @property
+    def frame(self):
+        """Get a window frame for the frame material surrounding the construction."""
+        return self._window_construction.frame
+
     @property
     def r_value(self):
         """R-value of the bare window construction [m2-K/W] (excluding air films).
@@ -338,7 +369,7 @@ class WindowConstructionShade(object):
 
     @property
     def r_factor(self):
-        """Bare window construction R-factor [m2-K/W] (with standard air film resistances).
+        """Bare window construction R-factor [m2-K/W] (with standard air resistances).
 
         Note that this excludes all effects of the shade layer.
         Formulas for film coefficients come from EN673 / ISO10292.
@@ -347,16 +378,40 @@ class WindowConstructionShade(object):
 
     @property
     def u_factor(self):
-        """Bare window construction U-factor [W/m2-K] (with standard air film resistances).
+        """Bare window construction U-factor [W/m2-K] (with standard air resistances).
 
         Note that this excludes all effects of the shade layer.
         Formulas for film coefficients come from EN673 / ISO10292.
         """
         return self._window_construction.u_factor
 
+    @property
+    def solar_transmittance(self):
+        """The solar transmittance of the bare window construction at normal incidence.
+
+        Note that this excludes all effects of the shade layer.
+        """
+        return self._window_construction.solar_transmittance
+
+    @property
+    def visible_transmittance(self):
+        """The visible transmittance of the bare window construction at normal incidence.
+
+        Note that this excludes all effects of the shade layer.
+        """
+        return self._window_construction.visible_transmittance
+
+    @property
+    def shgc(self):
+        """The solar heat gain coefficient (SHGC) of the bare window construction.
+
+        Note that this excludes all effects of the shade layer.
+        """
+        return self._window_construction.shgc
+
     @property
     def is_symmetric(self):
-        """Get a boolean to note whether the construction layers are symmetric.
+        """Get a boolean for whether the construction layers are symmetric.
 
         Symmetric means that the materials in reversed order are equal to those
         in the current order (eg. 'Glass', 'Air Gap', 'Glass'). This is particularly
@@ -370,6 +425,11 @@ class WindowConstructionShade(object):
                 return False
         return True
 
+    @property
+    def has_frame(self):
+        """Get a boolean noting whether the construction has a frame assigned to it."""
+        return self._window_construction.has_frame
+
     @property
     def has_shade(self):
         """Get a boolean noting whether dynamic materials are in the construction.
@@ -378,6 +438,14 @@ class WindowConstructionShade(object):
         """
         return True
 
+    @property
+    def is_dynamic(self):
+        """Get a boolean noting whether the construction is dynamic.
+
+        This will always be True for this class.
+        """
+        return True
+
     @property
     def is_switchable_glazing(self):
         """Get a boolean to note whether the construction is switchable glazing.
@@ -387,6 +455,24 @@ class WindowConstructionShade(object):
         """
         return isinstance(self.shade_material, EnergyWindowMaterialGlazing)
 
+    @property
+    def switched_glass_material(self):
+        """Get material replaced by shade glass when construction is switchable glazing.
+
+        This can be used to compare the properties of the glass layer replaced by
+        the shade glass. Will be None if the construction is not a switchable glazing.
+        """
+        if not self.is_switchable_glazing:
+            return None
+        base_mats = self._window_construction.materials
+        if self._shade_location == 'Interior':
+            return self.base_mats[-1]
+        elif self._shade_location == 'Exterior' or \
+                self._window_construction.gap_count == 0:
+            return base_mats[0]
+        else:  # middle glass pane
+            return base_mats[-3]
+
     @property
     def inside_emissivity(self):
         """"The emissivity of the inside face of the construction.
@@ -430,12 +516,38 @@ class WindowConstructionShade(object):
 
     @property
     def gap_count(self):
-        """The number of gas gaps contained within the construction."""
+        """Get the number of gas gaps contained within the construction."""
         count = self._window_construction.gap_count
         if self.shade_location == 'Between' and not self.is_switchable_glazing:
             count += 1
         return count
 
+    @property
+    def is_groupable(self):
+        """Get a boolean for whether controls allow the construction to be grouped."""
+        return self.control_type in self.GROUPABLE_TYPES
+
+    @property
+    def is_room_groupable(self):
+        """Get a boolean for whether controls allow grouping by room."""
+        return self.control_type in self.ROOM_GROUPABLE_TYPES
+
+    @property
+    def inside_material(self):
+        """The the inside material layer of the construction.
+
+        Useful for checking that an asymmetric construction is correctly assigned.
+        """
+        return self.materials[-1]
+
+    @property
+    def outside_material(self):
+        """The the outside material layer of the construction.
+
+        Useful for checking that an asymmetric construction is correctly assigned.
+        """
+        return self.materials[0]
+
     @property
     def _ep_shading_type(self):
         """Text for the Shading Type field that EnergyPlus wants in the IDF."""
@@ -448,6 +560,29 @@ class WindowConstructionShade(object):
             return 'BetweenGlassShade' if self.shade_location == 'Between' \
                 else '{}Shade'.format(self.shade_location)
 
+    @property
+    def user_data(self):
+        """Get or set an optional dictionary for additional meta data for this object.
+
+        This will be None until it has been set. All keys and values of this
+        dictionary should be of a standard Python type to ensure correct
+        serialization of the object to/from JSON (eg. str, float, int, list, dict)
+        """
+        if self._user_data is not None:
+            return self._user_data
+
+    @user_data.setter
+    def user_data(self, value):
+        if value is not None:
+            assert isinstance(value, dict), 'Expected dictionary for honeybee_energy' \
+                'object user_data. Got {}.'.format(type(value))
+        self._user_data = value
+
+    @property
+    def properties(self):
+        """Get properties for extensions."""
+        return self._properties
+
     @classmethod
     def from_dict(cls, data):
         """Create a WindowConstructionShade from a dictionary.
@@ -489,6 +624,10 @@ class WindowConstructionShade(object):
                       control_type, setpoint, schedule)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
         return new_obj
 
     @classmethod
@@ -500,8 +639,7 @@ class WindowConstructionShade(object):
             materials: A dictionary with identifiers of materials as keys and
                 Python material objects as values.
             schedules: A dictionary with schedule identifiers as keys and
-                honeybee schedule objects as values. These will be used to
-                assign the schedule to the AirBoundaryConstruction object.
+                honeybee schedule objects as values.
 
         .. code-block:: python
 
@@ -535,6 +673,10 @@ class WindowConstructionShade(object):
                       control_type, setpoint, schedule)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
         return new_obj
 
     def to_idf(self):
@@ -543,10 +685,11 @@ class WindowConstructionShade(object):
         Note that this method only outputs a single string for the bare window
         construction and, to write the full construction into an IDF, the
         construction's unique_materials must also be written along with the
-        output of to_idf_shaded, which contains the shaded construction.
+        output of to_shaded_idf, which contains the shaded construction.
         Also, for each Aperture to which this construction is assigned, a
         ShadingControl object must also be written, which can be obtained from
-        the to_idf_shading_control.
+        the to_shading_control_idf. If the construction has a frame, the frame
+        definition must also be written.
 
         Returns:
             Text string representation of the bare (unshaded) construction.
@@ -582,7 +725,7 @@ class WindowConstructionShade(object):
         control_name = '{}_ShdControl'.format(aperture_identifier)
         control_type = 'OnIfScheduleAllows' if self.schedule is not None and \
             self.control_type == 'AlwaysOn' else self.control_type
-        sch = self.schedule if self.schedule is not None else ''
+        sch = self.schedule.identifier if self.schedule is not None else ''
         sch_bool = 'Yes' if self.schedule is not None else 'No'
         setpt = self.setpoint if self.setpoint is not None else ''
         values = (control_name, room_identifier, 1, self._ep_shading_type,
@@ -613,7 +756,7 @@ class WindowConstructionShade(object):
             abridged: Boolean to note whether the full dictionary describing the
                 object should be returned (False) or just an abridged version (True),
                 which only specifies the identifiers of material layers and
-                schedules. Default: False.
+                schedules. (Default: False).
         """
         base = {'type': 'WindowConstructionShade'} if not \
             abridged else {'type': 'WindowConstructionShadeAbridged'}
@@ -630,6 +773,9 @@ class WindowConstructionShade(object):
                 else self.schedule.to_dict()
         if self._display_name is not None:
             base['display_name'] = self.display_name
+        if self._user_data is not None:
+            base['user_data'] = self.user_data
+        base['properties'] = self.properties.to_dict() if self._properties else None
         return base
 
     def lock(self):
@@ -664,6 +810,8 @@ class WindowConstructionShade(object):
             self.schedule)
         new_con._between_gap = self._between_gap
         new_con._display_name = self._display_name
+        new_con._user_data = None if self._user_data is None else self._user_data.copy()
+        new_con._properties._duplicate_extension_attr(self._properties)
         return new_con
 
     def __key(self):
