@@ -6,12 +6,13 @@ from .construction.dictutil import dict_to_construction
 from .construction.opaque import OpaqueConstruction
 from .construction.window import WindowConstruction
 from .construction.windowshade import WindowConstructionShade
+from .construction.dynamic import WindowConstructionDynamic
 from .construction.shade import ShadeConstruction
 from .construction.air import AirBoundaryConstruction
 import honeybee_energy.lib.constructions as _lib
 
 from honeybee._lockable import lockable
-from honeybee.typing import valid_ep_string
+from honeybee.typing import valid_ep_string, clean_rad_string
 
 
 @lockable
@@ -38,9 +39,9 @@ class ConstructionSet(object):
         shade_construction: An optional ShadeConstruction to set the reflectance
             properties of all outdoor shades to which this ConstructionSet is
             assigned. If None, it will be the honeybee generic shade construction.
-        air_boundary_construction: An optional AirBoundaryConstruction to set
-            the properties of Faces with an AirBoundary type. If None, it
-            will be the honeybee generic air boundary construction.
+        air_boundary_construction: An optional AirBoundaryConstruction or
+            OpaqueConstruction to set the properties of Faces with an AirBoundary
+            type. If None, it will be the honeybee generic air boundary construction.
 
     Properties:
         * identifier
@@ -58,10 +59,12 @@ class ConstructionSet(object):
         * modified_constructions_unique
         * materials_unique
         * modified_materials_unique
+        * user_data
     """
     __slots__ = ('_identifier', '_display_name', '_wall_set', '_floor_set',
                  '_roof_ceiling_set', '_aperture_set', '_door_set',
-                 '_shade_construction', '_air_boundary_construction', '_locked')
+                 '_shade_construction', '_air_boundary_construction',
+                 '_locked', '_user_data')
 
     def __init__(self, identifier, wall_set=None, floor_set=None, roof_ceiling_set=None,
                  aperture_set=None, door_set=None, shade_construction=None,
@@ -77,6 +80,7 @@ class ConstructionSet(object):
         self.door_set = door_set
         self.shade_construction = shade_construction
         self.air_boundary_construction = air_boundary_construction
+        self._user_data = None
 
     @property
     def identifier(self):
@@ -99,10 +103,12 @@ class ConstructionSet(object):
 
     @display_name.setter
     def display_name(self, value):
-        try:
-            self._display_name = str(value)
-        except UnicodeEncodeError:  # Python 2 machine lacking the character set
-            self._display_name = value  # keep it as unicode
+        if value is not None:
+            try:
+                value = str(value)
+            except UnicodeEncodeError:  # Python 2 machine lacking the character set
+                pass  # keep it as unicode
+        self._display_name = value
 
     @property
     def wall_set(self):
@@ -199,8 +205,9 @@ class ConstructionSet(object):
     @air_boundary_construction.setter
     def air_boundary_construction(self, value):
         if value is not None:
-            assert isinstance(value, AirBoundaryConstruction), \
-                'Expected AirBoundaryConstruction. Got {}'.format(type(value))
+            assert isinstance(value, (AirBoundaryConstruction, OpaqueConstruction)), \
+                'Expected AirBoundaryConstruction or OpaqueConstruction. ' \
+                'Got {}'.format(type(value))
             value.lock()   # lock editing in case construction has multiple references
         self._air_boundary_construction = value
 
@@ -245,6 +252,8 @@ class ConstructionSet(object):
         for constr in self.constructions:
             try:
                 materials.extend(constr.materials)
+                if constr.has_frame:
+                    materials.append(constr.frame)
             except AttributeError:
                 pass  # ShadeConstruction or AirBoundaryConstruction
         return list(set(materials))
@@ -256,10 +265,30 @@ class ConstructionSet(object):
         for constr in self.modified_constructions:
             try:
                 materials.extend(constr.materials)
+                if constr.has_frame:
+                    materials.append(constr.frame)
             except AttributeError:
                 pass  # ShadeConstruction or AirBoundaryConstruction
         return list(set(materials))
 
+    @property
+    def user_data(self):
+        """Get or set an optional dictionary for additional meta data for this object.
+
+        This will be None until it has been set. All keys and values of this
+        dictionary should be of a standard Python type to ensure correct
+        serialization of the object to/from JSON (eg. str, float, int, list, dict)
+        """
+        if self._user_data is not None:
+            return self._user_data
+
+    @user_data.setter
+    def user_data(self, value):
+        if value is not None:
+            assert isinstance(value, dict), 'Expected dictionary for honeybee_energy' \
+                'object user_data. Got {}.'.format(type(value))
+        self._user_data = value
+
     def get_face_construction(self, face_type, boundary_condition):
         """Get a construction object that will be assigned to a given type of face.
 
@@ -369,7 +398,8 @@ class ConstructionSet(object):
             in data and data['wall_set'] is not None else None
         floor_set = FloorConstructionSet.from_dict(data['floor_set']) if 'floor_set' \
             in data and data['floor_set'] is not None else None
-        roof_ceiling_set = RoofCeilingConstructionSet.from_dict(data['roof_ceiling_set']) \
+        roof_ceiling_set = \
+            RoofCeilingConstructionSet.from_dict(data['roof_ceiling_set']) \
             if 'roof_ceiling_set' in data and data['roof_ceiling_set'] \
             is not None else None
         aperture_set = ApertureConstructionSet.from_dict(data['aperture_set']) if \
@@ -379,14 +409,21 @@ class ConstructionSet(object):
         shade_con = ShadeConstruction.from_dict(data['shade_construction']) if \
             'shade_construction' in data and data['shade_construction'] is not None \
             else None
-        air_con = AirBoundaryConstruction.from_dict(data['air_boundary_construction']) \
-            if 'air_boundary_construction' in data and \
-            data['air_boundary_construction'] is not None else None
+        air_con = None
+        if 'air_boundary_construction' in data and \
+                data['air_boundary_construction'] is not None:
+            if data['air_boundary_construction']['type'] == 'AirBoundaryConstruction':
+                air_con = AirBoundaryConstruction.from_dict(
+                    data['air_boundary_construction'])
+            else:
+                air_con = OpaqueConstruction.from_dict(data['air_boundary_construction'])
 
         new_obj = cls(data['identifier'], wall_set, floor_set, roof_ceiling_set,
                       aperture_set, door_set, shade_con, air_con)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
         return new_obj
 
     @classmethod
@@ -416,12 +453,19 @@ class ConstructionSet(object):
         """
         assert data['type'] == 'ConstructionSetAbridged', \
             'Expected ConstructionSetAbridged. Got {}.'.format(data['type'])
-        wall_set, floor_set, roof_ceiling_set, aperture_set, door_set, shade_con, \
-            air_con = cls._get_subsets_from_abridged(data, construction_dict)
+        try:
+            wall_set, floor_set, roof_ceiling_set, aperture_set, door_set, shade_con, \
+                air_con = cls._get_subsets_from_abridged(data, construction_dict)
+        except KeyError as e:
+            raise ValueError(
+                'The following construction is missing from the model: {}'.format(e)
+            )
         new_obj = cls(data['identifier'], wall_set, floor_set, roof_ceiling_set,
                       aperture_set, door_set, shade_con, air_con)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
         return new_obj
 
     def to_dict(self, abridged=False, none_for_defaults=True):
@@ -470,8 +514,48 @@ class ConstructionSet(object):
 
         if self._display_name is not None:
             base['display_name'] = self.display_name
+        if self._user_data is not None:
+            base['user_data'] = self.user_data
         return base
 
+    def to_radiance_solar_interior(self):
+        """Honeybee Radiance modifier set for the interior solar properties."""
+        # convert all interior constructions into modifiers
+        unique_mods = {}
+        for constr in self.constructions_unique:
+            unique_mods[constr.identifier] = constr.to_radiance_solar_interior() \
+                if isinstance(constr, OpaqueConstruction) else constr.to_radiance_solar()
+        return self._create_modifier_set('Solar_Interior', unique_mods)
+
+    def to_radiance_visible_interior(self):
+        """Honeybee Radiance modifier set for the interior visible properties."""
+        # convert all interior constructions into modifiers
+        unique_mods = {}
+        for constr in self.constructions_unique:
+            unique_mods[constr.identifier] = constr.to_radiance_visible_interior() \
+                if isinstance(constr, OpaqueConstruction) \
+                else constr.to_radiance_visible()
+        return self._create_modifier_set('Visible_Interior', unique_mods)
+
+    def to_radiance_solar_exterior(self):
+        """Honeybee Radiance modifier set for the exterior solar properties."""
+        # convert all exterior constructions into modifiers
+        unique_mods = {}
+        for constr in self.constructions_unique:
+            unique_mods[constr.identifier] = constr.to_radiance_solar_exterior() \
+                if isinstance(constr, OpaqueConstruction) else constr.to_radiance_solar()
+        return self._create_modifier_set('Solar_Exterior', unique_mods)
+
+    def to_radiance_visible_exterior(self):
+        """Honeybee Radiance modifier set for the exterior visible properties."""
+        # convert all exterior constructions into modifiers
+        unique_mods = {}
+        for constr in self.constructions_unique:
+            unique_mods[constr.identifier] = constr.to_radiance_visible_exterior() \
+                if isinstance(constr, OpaqueConstruction) \
+                else constr.to_radiance_visible()
+        return self._create_modifier_set('Visible_Exterior', unique_mods)
+
     def duplicate(self):
         """Get a copy of this ConstructionSet."""
         return self.__copy__()
@@ -494,6 +578,53 @@ class ConstructionSet(object):
         self._aperture_set.unlock()
         self._door_set.unlock()
 
+    def _create_modifier_set(self, mod_type, unique_mods):
+        """Create a modifier set from a dictionary of radiance modifiers."""
+        # import the radiance dependency
+        try:
+            from honeybee_radiance.modifierset import ModifierSet
+        except ImportError as e:
+            raise ImportError('honeybee_radiance library must be installed to use '
+                              'to_radiance_solar() method. {}'.format(e))
+        # create the modifier set object
+        mod_set = ModifierSet(
+            '{}_{}'.format(clean_rad_string(self.identifier), mod_type))
+        mod_set.wall_set.exterior_modifier = \
+            unique_mods[self.wall_set.exterior_construction.identifier]
+        mod_set.wall_set.interior_modifier = \
+            unique_mods[self.wall_set.interior_construction.identifier]
+        mod_set.floor_set.exterior_modifier = \
+            unique_mods[self.floor_set.exterior_construction.identifier]
+        mod_set.floor_set.interior_modifier = \
+            unique_mods[self.floor_set.interior_construction.identifier]
+        mod_set.roof_ceiling_set.exterior_modifier = \
+            unique_mods[self.roof_ceiling_set.exterior_construction.identifier]
+        mod_set.roof_ceiling_set.interior_modifier = \
+            unique_mods[self.roof_ceiling_set.interior_construction.identifier]
+        mod_set.aperture_set.window_modifier = \
+            unique_mods[self.aperture_set.window_construction.identifier]
+        mod_set.aperture_set.interior_modifier = \
+            unique_mods[self.aperture_set.interior_construction.identifier]
+        mod_set.aperture_set.skylight_modifier = \
+            unique_mods[self.aperture_set.skylight_construction.identifier]
+        mod_set.aperture_set.operable_modifier = \
+            unique_mods[self.aperture_set.operable_construction.identifier]
+        mod_set.door_set.exterior_modifier = \
+            unique_mods[self.door_set.exterior_construction.identifier]
+        mod_set.door_set.interior_modifier = \
+            unique_mods[self.door_set.interior_construction.identifier]
+        mod_set.door_set.exterior_glass_modifier = \
+            unique_mods[self.door_set.exterior_glass_construction.identifier]
+        mod_set.door_set.interior_glass_modifier = \
+            unique_mods[self.door_set.interior_glass_construction.identifier]
+        mod_set.door_set.overhead_modifier = \
+            unique_mods[self.door_set.overhead_construction.identifier]
+        mod_set.shade_set.exterior_modifier = \
+            unique_mods[self.shade_construction.identifier]
+        mod_set.air_boundary_modifier = \
+            unique_mods[self.air_boundary_construction.identifier]
+        return mod_set
+
     def _get_constr_from_set(self, face_type_set, boundary_condition):
         """Get a specific construction from a face_type_set."""
         if boundary_condition == 'Outdoors':
@@ -607,6 +738,7 @@ class ConstructionSet(object):
                                   self._shade_construction,
                                   self._air_boundary_construction)
         new_obj._display_name = self._display_name
+        new_obj.user_data = None if self._user_data is None else self._user_data.copy()
         return new_obj
 
     def __key(self):
@@ -623,7 +755,7 @@ class ConstructionSet(object):
         return not self.__eq__(other)
 
     def __repr__(self):
-        return 'Energy Construction Set: {}'.format(self.identifier)
+        return 'Energy Construction Set: {}'.format(self.display_name)
 
 
 @lockable
@@ -787,8 +919,10 @@ class _FaceSetBase(object):
         return iter(self.constructions)
 
     def __copy__(self):
-        return self.__class__(self._exterior_construction, self._interior_construction,
-                              self._ground_construction)
+        new_obj = self.__class__(
+            self._exterior_construction, self._interior_construction,
+            self._ground_construction)
+        return(new_obj)
 
     def __repr__(self):
         return 'Base Face Set'
@@ -848,10 +982,10 @@ class WallConstructionSet(_FaceSetBase):
         self._ground_construction = value
 
     def __repr__(self):
-        return 'Wall Construction Set:\n Exterior: {}\n Interior: {}' \
-            '\n Ground: {}'.format(self.exterior_construction.identifier,
-                                   self.interior_construction.identifier,
-                                   self.ground_construction.identifier)
+        return 'Wall Construction Set: [Exterior: {}] [Interior: {}]' \
+            ' [Ground: {}]'.format(self.exterior_construction.display_name,
+                                   self.interior_construction.display_name,
+                                   self.ground_construction.display_name)
 
 
 @lockable
@@ -908,10 +1042,10 @@ class FloorConstructionSet(_FaceSetBase):
         self._ground_construction = value
 
     def __repr__(self):
-        return 'Floor Construction Set:\n Exterior: {}\n Interior: {}' \
-            '\n Ground: {}'.format(self.exterior_construction.identifier,
-                                   self.interior_construction.identifier,
-                                   self.ground_construction.identifier)
+        return 'Floor Construction Set: [Exterior: {}] [Interior: {}]' \
+            ' [Ground: {}]'.format(self.exterior_construction.display_name,
+                                   self.interior_construction.display_name,
+                                   self.ground_construction.display_name)
 
 
 @lockable
@@ -968,10 +1102,10 @@ class RoofCeilingConstructionSet(_FaceSetBase):
         self._ground_construction = value
 
     def __repr__(self):
-        return 'RoofCeiling Construction Set:\n Exterior: {}\n Interior: {}' \
-            '\n Ground: {}'.format(self.exterior_construction.identifier,
-                                   self.interior_construction.identifier,
-                                   self.ground_construction.identifier)
+        return 'RoofCeiling Construction Set: [Exterior: {}] [Interior: {}]' \
+            ' [Ground: {}]'.format(self.exterior_construction.display_name,
+                                   self.interior_construction.display_name,
+                                   self.ground_construction.display_name)
 
 
 @lockable
@@ -1136,9 +1270,11 @@ class ApertureConstructionSet(object):
             if abridged:
                 base['window_construction'] = self._window_construction.identifier if \
                     self._window_construction is not None else None
-                base['interior_construction'] = self._interior_construction.identifier if \
+                base['interior_construction'] = \
+                    self._interior_construction.identifier if \
                     self._interior_construction is not None else None
-                base['skylight_construction'] = self._skylight_construction.identifier if \
+                base['skylight_construction'] = \
+                    self._skylight_construction.identifier if \
                     self._skylight_construction is not None else None
                 base['operable_construction'] = \
                     self._operable_construction.identifier if \
@@ -1170,8 +1306,9 @@ class ApertureConstructionSet(object):
 
     def _check_window_construction(self, value):
         """Check that a construction is valid before assigning it."""
-        assert isinstance(value, (WindowConstruction, WindowConstructionShade)), \
-            'Expected WindowConstruction. Got {}'.format(type(value))
+        val_w = (WindowConstruction, WindowConstructionShade, WindowConstructionDynamic)
+        assert isinstance(value, val_w), \
+            'Expected Window Construction. Got {}'.format(type(value))
         value.lock()   # lock editing in case construction has multiple references
 
     def ToString(self):
@@ -1185,17 +1322,19 @@ class ApertureConstructionSet(object):
         return iter(self.constructions)
 
     def __copy__(self):
-        return self.__class__(
+        new_obj = self.__class__(
             self._window_construction, self._interior_construction,
-            self._skylight_construction, self._operable_construction)
+            self._skylight_construction, self._operable_construction
+        )
+        return(new_obj)
 
     def __repr__(self):
-        return 'Aperture Construction Set:\n Window: {}\n Interior: {}' \
-            '\n Skylight: {}\n Operable: {}'.format(
-                self.window_construction.identifier,
-                self.interior_construction.identifier,
-                self.skylight_construction.identifier,
-                self.operable_construction.identifier)
+        return 'Aperture Construction Set: [Window: {}] [Interior: {}]' \
+            ' [Skylight: {}] [Operable: {}]'.format(
+                self.window_construction.display_name,
+                self.interior_construction.display_name,
+                self.skylight_construction.display_name,
+                self.operable_construction.display_name)
 
 
 @lockable
@@ -1367,6 +1506,7 @@ class DoorConstructionSet(object):
         ohc = OpaqueConstruction.from_dict(data['overhead_construction']) \
             if 'overhead_construction' in data and data['overhead_construction'] \
             is not None else None
+
         return cls(extc, intc, egc, igc, ohc)
 
     def to_dict(self, abridged=False, none_for_defaults=True):
@@ -1436,8 +1576,9 @@ class DoorConstructionSet(object):
 
     def _check_window_construction(self, value):
         """Check that a construction is valid before assigning it."""
-        assert isinstance(value, (WindowConstruction, WindowConstructionShade)), \
-            'Expected WindowConstruction. Got {}'.format(type(value))
+        val_w = (WindowConstruction, WindowConstructionShade, WindowConstructionDynamic)
+        assert isinstance(value, val_w), \
+            'Expected Window Construction. Got {}'.format(type(value))
         value.lock()   # lock editing in case construction has multiple references
 
     def ToString(self):
@@ -1451,16 +1592,17 @@ class DoorConstructionSet(object):
         return iter(self.constructions)
 
     def __copy__(self):
-        return self.__class__(
+        new_obj = self.__class__(
             self._exterior_construction, self._interior_construction,
             self._exterior_glass_construction, self._interior_glass_construction,
             self._overhead_construction)
+        return(new_obj)
 
     def __repr__(self):
-        return 'Door Construction Set:\n Exterior: {}\n Interior: {}' \
-            '\n Exterior Glass: {}\n Interior Glass: {}\n Overhead: {}'.format(
-                self.exterior_construction.identifier,
-                self.interior_construction.identifier,
-                self.exterior_glass_construction.identifier,
-                self.interior_glass_construction.identifier,
-                self.overhead_construction.identifier)
+        return 'Door Construction Set: [Exterior: {}] [Interior: {}]' \
+            ' [Exterior Glass: {}] [Interior Glass: {}] [Overhead: {}]'.format(
+                self.exterior_construction.display_name,
+                self.interior_construction.display_name,
+                self.exterior_glass_construction.display_name,
+                self.interior_glass_construction.display_name,
+                self.overhead_construction.display_name)
