@@ -2,23 +2,20 @@
 """Simple ideal air system object used to condition zones."""
 from __future__ import division
 
-from ._base import _HVACSystem
-from ..reader import parse_idf_string
-from ..writer import generate_idf_string
-
 from honeybee._lockable import lockable
 from honeybee.typing import valid_string, float_positive, float_in_range
 from honeybee.altnumber import autosize, no_limit
 
+from ._base import _HVACSystem
+from ..reader import parse_idf_string
+from ..writer import generate_idf_string
+from ..properties.extension import IdealAirSystemProperties
+
 
 @lockable
 class IdealAirSystem(_HVACSystem):
     """Simple ideal air system object used to condition zones.
 
-    Note that this is the only HVAC system supported by honeybee_energy and, to
-    access more advanced HVAC systems, a honeybee_energy extension should be
-    installed.
-
     Args:
         identifier: Text string for ideal air system identifier. Must be < 100 characters
             and not contain any EnergyPlus special characters. This will be used to
@@ -27,7 +24,7 @@ class IdealAirSystem(_HVACSystem):
             the ideal air system. Economizers will mix in a greater amount of
             outdoor air to cool the zone (rather than running the cooling system)
             when the zone needs cooling and the outdoor air is cooler than the zone.
-            Choose from the options below. Default: DifferentialDryBulb.
+            Choose from the options below. (Default: DifferentialDryBulb).
 
             * NoEconomizer
             * DifferentialDryBulb
@@ -35,32 +32,32 @@ class IdealAirSystem(_HVACSystem):
 
         demand_controlled_ventilation: Boolean to note whether demand controlled
             ventilation should be used on the system, which will vary the amount
-            of ventilation air according to the occupancy schedule of the zone.
-            Default: False.
+            of ventilation air according to the occupancy schedule of the
+            Room. (Default: False).
         sensible_heat_recovery: A number between 0 and 1 for the effectiveness
-            of sensible heat recovery within the system. Default: 0.
+            of sensible heat recovery within the system. (Default: 0).
         latent_heat_recovery: A number between 0 and 1 for the effectiveness
-            of latent heat recovery within the system. Default: 0.
+            of latent heat recovery within the system. (Default: 0).
         heating_air_temperature: A number for the maximum heating supply air
-            temperature [C]. Default: 50, which is typical for many air-based
-            HVAC systems.
+            temperature [C]. (Default: 50, which is typical for many air-based
+            HVAC systems).
         cooling_air_temperature: A number for the minimum cooling supply air
-            temperature [C]. Default: 13, which is typical for many air-based
-            HVAC systems.
+            temperature [C]. (Default: 13, which is typical for many air-based
+            HVAC systems).
         heating_limit: A number for the maximum heating capacity in Watts. This
             can also be an Autosize object to indicate that the capacity should
             be determined during the EnergyPlus sizing calculation. This can also
             be a NoLimit object to indicate no upper limit to the heating
-            capacity. Default: autosize.
+            capacity. (Default: autosize).
         cooling_limit: A number for the maximum cooling capacity in Watts. This
             can also be an Autosize object to indicate that the capacity should
             be determined during the EnergyPlus sizing calculation. This can also
             be a NoLimit object to indicate no upper limit to the cooling
-            capacity.
-        heating_availability: An optional schedule to set the availability of
-            heating over the course of the simulation. Default: None.
-        cooling_availability: An optional schedule to set the availability of
-            cooling over the course of the simulation. Default: None.
+            capacity. (Default: autosize).
+        heating_availability: An optional on/off schedule to set the availability of
+            heating over the course of the simulation. (Default: None).
+        cooling_availability: An optional on/off schedule to set the availability of
+            cooling over the course of the simulation. (Default: None).
 
     Properties:
         * identifier
@@ -75,12 +72,14 @@ class IdealAirSystem(_HVACSystem):
         * cooling_limit
         * heating_availability
         * cooling_availability
+        * schedules
+        * properties
     """
     __slots__ = ('_economizer_type', '_demand_controlled_ventilation',
                  '_sensible_heat_recovery', '_latent_heat_recovery',
                  '_heating_air_temperature', '_cooling_air_temperature',
                  '_heating_limit', '_cooling_limit', '_heating_availability',
-                 '_cooling_availability')
+                 '_cooling_availability', '_properties')
     ECONOMIZER_TYPES = ('NoEconomizer', 'DifferentialDryBulb', 'DifferentialEnthalpy')
 
     def __init__(self, identifier, economizer_type='DifferentialDryBulb',
@@ -91,7 +90,7 @@ class IdealAirSystem(_HVACSystem):
                  heating_availability=None, cooling_availability=None):
         """Initialize IdealAirSystem."""
         # initialize base HVAC system properties
-        _HVACSystem.__init__(self, identifier, is_single_room=True)
+        _HVACSystem.__init__(self, identifier)
 
         # set the main features of the HVAC system
         self.economizer_type = economizer_type
@@ -109,6 +108,9 @@ class IdealAirSystem(_HVACSystem):
         self.heating_availability = heating_availability
         self.cooling_availability = cooling_availability
 
+        # initialize properties for extensions
+        self._properties = IdealAirSystemProperties(self)
+
     @property
     def economizer_type(self):
         """Get or set text to indicate the type of air-side economizer.
@@ -275,7 +277,7 @@ class IdealAirSystem(_HVACSystem):
         ep_strs = parse_idf_string(idf_string, 'HVACTemplate:Zone:IdealLoadsAirSystem,')
 
         # set defaults for anything not included
-        identifier = '{}_IdealAir'.format(ep_strs[0])
+        identifier = '{} Ideal Loads Air System'.format(ep_strs[0])
         econ = 'DifferentialDryBulb'
         dcv = False
         sensible = 0
@@ -337,7 +339,7 @@ class IdealAirSystem(_HVACSystem):
 
             {
             "type": "IdealAirSystem",
-            "identifier": "Classroom1_IdealAir",  # identifier for the HVAC
+            "identifier": "Classroom1 Ideal Loads Air System",  # identifier for the HVAC
             "display_name": "Standard IdealAir",  # name for the HVAC
             "economizer_type": 'DifferentialDryBulb',  # Economizer type
             "demand_controlled_ventilation": True,  # Demand controlled ventilation
@@ -370,6 +372,10 @@ class IdealAirSystem(_HVACSystem):
                       cool_temp, heat_limit, cool_limit, heat_avail, cool_avail)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
         return new_obj
 
     @classmethod
@@ -387,7 +393,7 @@ class IdealAirSystem(_HVACSystem):
 
             {
             "type": 'IdealAirSystemAbridged',
-            "identifier": 'Warehouse1_IdealAir',  # identifier for the HVAC
+            "identifier": 'Warehouse1 Ideal Loads Air System',  # identifier for the HVAC
             "economizer_type": 'DifferentialDryBulb',  # Economizer type
             "demand_controlled_ventilation": True,  # Demand controlled ventilation
             "sensible_heat_recovery": 0.75,  # Sensible heat recovery effectiveness
@@ -396,8 +402,8 @@ class IdealAirSystem(_HVACSystem):
             "cooling_air_temperature": 15,  # Cooling supply air temperature
             "heating_limit": 'autosize',  # Max size of the heating system in Watts
             "cooling_limit": 'autosize',  # Max size of the cooling system in Watts
-            "heating_availability": "Warehouse Heating Control",  # identifier of schedule
-            "cooling_availability": "Warehouse Cooling Control",  # identifier of schedule
+            "heating_availability": "Warehouse Heating Control",  # schedule identifier
+            "cooling_availability": "Warehouse Cooling Control",  # schedule identifier
             }
         """
         assert data['type'] == 'IdealAirSystemAbridged', \
@@ -425,26 +431,31 @@ class IdealAirSystem(_HVACSystem):
                       cool_temp, heat_limit, cool_limit, heat_avail, cool_avail)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
         return new_obj
 
-    def to_idf(self):
+    def to_idf(self, room):
         """IDF string representation of IdealAirSystem object.
 
-        Note that this ideal air system must be assigned to a honeybee Room that
-        has a Setpoint object for this method to work correctly since all setpoints
-        and ventilation requirements are pulled from this assigned Room.
-
-        Also note that this method does not return full definitions of heating/cooling
+        Note that this method does not return full definitions of heating/cooling
         availability schedules and so this objects's schedules must also be translated
         into the final IDF file.
+
+        Args:
+            room: A Honeybee Room for which the specific IDF string will be generated.
+                This Room must have a Setpoint object for this method to work
+                correctly since all setpoints (and any ventilation requirements)
+                are pulled from this Room.
         """
         # check that a setpoint object is assigned
-        assert self._parent is not None and \
-            self._parent.properties.energy.setpoint is not None, \
-            'IdealAirSystem must be assigned to a Room ' \
+        r_set_p = room.properties.energy.setpoint
+        assert r_set_p is not None, 'IdealAirSystem must be assigned to a Room ' \
             'with a setpoint object to use IdealAirSystem.to_idf.'
 
-        # extract all of the fields from this object and its parent
+        # extract all of the fields from this object and its room
         # heating limit
         if self.heating_limit != no_limit:
             h_lim_type = 'LimitCapacity'
@@ -466,24 +477,24 @@ class IdealAirSystem(_HVACSystem):
         cool_avail = self.cooling_availability.identifier if \
             self.cooling_availability is not None else ''
         # humidifying setpoint
-        if self._parent.properties.energy.setpoint.humidifying_setpoint is not None:
+        if r_set_p.humidifying_setpoint is not None:
             humid_type = 'Humidistat'
-            humid_setpt = self._parent.properties.energy.setpoint.humidifying_setpoint
+            humid_setpt = r_set_p.humidifying_setpoint
         else:
             humid_type = 'None'
             humid_setpt = ''
         # dehumidifying setpoint
-        if self._parent.properties.energy.setpoint.dehumidifying_setpoint is not None:
+        if r_set_p.dehumidifying_setpoint is not None:
             dehumid_type = 'Humidistat'
-            dehumid_setpt = self._parent.properties.energy.setpoint.dehumidifying_setpoint
+            dehumid_setpt = r_set_p.dehumidifying_setpoint
         else:
             dehumid_type = 'None'
             dehumid_setpt = ''
         # ventilation requirements
-        if self._parent.properties.energy.ventilation is not None:
+        if room.properties.energy.ventilation is not None:
             oa_method = 'DetailedSpecification'
-            oa_id = '{}..{}'.format(self._parent.properties.energy.ventilation.identifier,
-                                    self._parent.identifier)
+            oa_id = '{}..{}'.format(room.properties.energy.ventilation.identifier,
+                                    room.identifier)
         else:
             oa_method = 'None'
             oa_id = ''
@@ -498,9 +509,9 @@ class IdealAirSystem(_HVACSystem):
             heat_recovery = 'Sensible'
 
         # return a full IDF string
-        thermostat = '{}..{}'.format(self._parent.properties.energy.setpoint.identifier,
-                                     self._parent.identifier)
-        values = (self._parent.identifier, thermostat,
+        thermostat = '{}..{}'.format(r_set_p.identifier, room.identifier) \
+            if r_set_p.setpoint_cutout_difference == 0 else ''
+        values = (room.identifier, thermostat,
                   '', self.heating_air_temperature, self.cooling_air_temperature,
                   '', '', h_lim_type, '', heat_limit, c_lim_type, air_limit, cool_limit,
                   heat_avail, cool_avail, dehumid_type, '', dehumid_setpt,
@@ -552,16 +563,13 @@ class IdealAirSystem(_HVACSystem):
                 abridged else self.cooling_availability.to_dict()
         if self._display_name is not None:
             base['display_name'] = self.display_name
+        if self._user_data is not None:
+            base['user_data'] = self.user_data
+        prop_dict = self.properties.to_dict()
+        if prop_dict is not None:
+            base['properties'] = prop_dict
         return base
 
-    def duplicate(self):
-        """Get a copy of this object."""
-        return self.__copy__()
-
-    def ToString(self):
-        """Overwrite .NET ToString."""
-        return self.__repr__()
-
     def _air_temperature_check(self):
         """Check that heating_air_temperature is greater than cooling_air_temperature."""
         assert self._heating_air_temperature > self._cooling_air_temperature, \
@@ -601,6 +609,11 @@ class IdealAirSystem(_HVACSystem):
 
         return econ, dcv, sensible, latent, heat_temp, cool_temp, heat_limit, cool_limit
 
+    @property
+    def properties(self):
+        """Get properties for extensions."""
+        return self._properties
+
     def __copy__(self):
         new_obj = IdealAirSystem(
             self._identifier, self._economizer_type, self._demand_controlled_ventilation,
@@ -609,6 +622,8 @@ class IdealAirSystem(_HVACSystem):
             self._heating_limit, self._cooling_limit, self._heating_availability,
             self._cooling_availability)
         new_obj._display_name = self._display_name
+        new_obj._user_data = None if self._user_data is None else self._user_data.copy()
+        new_obj._properties._duplicate_extension_attr(self._properties)
         return new_obj
 
     def __key(self):
@@ -630,7 +645,4 @@ class IdealAirSystem(_HVACSystem):
         return not self.__eq__(other)
 
     def __repr__(self):
-        return 'IdealAirSystem: {}\n economizer: {}\n dcv: {}\n sensible recovery: {}' \
-            '\n latent recovery: {}'.format(
-                self.identifier, self.economizer_type, self.demand_controlled_ventilation,
-                self.sensible_heat_recovery, self.latent_heat_recovery)
+        return 'IdealAirSystem: {}'.format(self.display_name)
