@@ -2,14 +2,15 @@
 """Complete definition of equipment in a simulation, including schedule and load."""
 from __future__ import division
 
+from honeybee._lockable import lockable
+from honeybee.typing import float_in_range, float_positive, clean_and_id_ep_string
+
 from ._base import _LoadBase
 from ..schedule.ruleset import ScheduleRuleset
 from ..schedule.fixedinterval import ScheduleFixedInterval
 from ..reader import parse_idf_string
 from ..writer import generate_idf_string
-
-from honeybee._lockable import lockable
-from honeybee.typing import float_in_range, float_positive
+from ..properties.extension import ElectricEquipmentProperties, GasEquipmentProperties
 
 
 @lockable
@@ -27,13 +28,13 @@ class _EquipmentBase(_LoadBase):
             Fractional and the fractional values will get multiplied by the
             watts_per_area to yield a complete equipment profile.
         radiant_fraction: A number between 0 and 1 for the fraction of the total
-            equipment load given off as long wave radiant heat. Default: 0.
+            equipment load given off as long wave radiant heat. (Default: 0).
         latent_fraction: A number between 0 and 1 for the fraction of the total
-            equipment load that is latent (as opposed to sensible). Default: 0.
+            equipment load that is latent (as opposed to sensible). (Default: 0).
         lost_fraction: A number between 0 and 1 for the fraction of the total
             equipment load that is lost outside of the zone and the HVAC system.
             Typically, this is used to represent heat that is exhausted directly
-            out of a zone (as you would for a stove). Default: 0.
+            out of a zone (as you would for a stove). (Default: 0).
 
     Properties:
         * identifier
@@ -44,13 +45,14 @@ class _EquipmentBase(_LoadBase):
         * latent_fraction
         * lost_fraction
         * convected_fraction
+        * user_data
     """
     __slots__ = ('_watts_per_area', '_schedule', '_radiant_fraction',
                  '_latent_fraction', '_lost_fraction')
     _idf_comments = ('name', 'zone name', 'schedule name', 'equipment level method',
                      'equipment power level {W}', 'equipment per floor area {W/m2}',
                      'equipment per person {W/ppl}', 'latent fraction',
-                     'radiant fration', 'lost fraction')
+                     'radiant fraction', 'lost fraction')
 
     def __init__(self, identifier, watts_per_area, schedule, radiant_fraction=0,
                  latent_fraction=0, lost_fraction=0):
@@ -127,6 +129,48 @@ class _EquipmentBase(_LoadBase):
         return 1 - sum((self._radiant_fraction, self._latent_fraction,
                         self._lost_fraction))
 
+    def diversify(self, count, watts_stdev=20, schedule_offset=1, timestep=1,
+                  schedule_indices=None):
+        """Get an array of diversified Equipment derived from this "average" one.
+
+        Approximately 2/3 of the schedules in the output objects will be offset
+        from the mean by the input schedule_offset (1/3 ahead and another 1/3 behind).
+
+        Args:
+            count: An positive integer for the number of diversified objects to
+                generate from this mean object.
+            watts_stdev: A number between 0 and 100 for the percent of the watts_per_area
+                representing one standard deviation of diversification from
+                the mean. (Default 20 percent).
+            schedule_offset: A positive integer for the number of timesteps at which
+                the lighting schedule of the resulting objects will be shifted - roughly
+                1/3 of the objects ahead and another 1/3 behind. (Default: 1).
+            timestep: An integer for the number of timesteps per hour at which the
+                shifting is occurring. This must be a value between 1 and 60, which
+                is evenly divisible by 60. 1 indicates that each step is an hour
+                while 60 indicates that each step is a minute. (Default: 1).
+            schedule_indices: An optional list of integers from 0 to 2 with a length
+                equal to the input count, which will be used to set whether a given
+                schedule is behind (0), ahead (2), or the same (1). This can be
+                used to coordinate schedules across diversified programs. If None
+                a random list of integers will be generated. (Default: None).
+        """
+        # generate shifted schedules and a gaussian distribution of watts_per_area
+        usage_schs = self._shift_schedule(self.schedule, schedule_offset, timestep)
+        stdev = self.watts_per_area * (watts_stdev / 100)
+        new_loads, sch_ints = self._gaussian_values(count, self.watts_per_area, stdev)
+        sch_ints = sch_ints if schedule_indices is None else schedule_indices
+
+        # generate the new objects and return them
+        new_objects = []
+        for load_val, sch_int in zip(new_loads, sch_ints):
+            new_obj = self.duplicate()
+            new_obj.identifier = clean_and_id_ep_string(self.identifier)
+            new_obj.watts_per_area = load_val
+            new_obj.schedule = usage_schs[sch_int]
+            new_objects.append(new_obj)
+        return new_objects
+
     def _check_fractions(self):
         tot = (self._radiant_fraction, self._latent_fraction, self._lost_fraction)
         assert sum(tot) <= 1 + 1e-9, 'Sum of equipment radiant_fraction, ' \
@@ -149,21 +193,30 @@ class _EquipmentBase(_LoadBase):
             abridged else self.schedule.identifier
         if self._display_name is not None:
             base['display_name'] = self.display_name
+        if self._user_data is not None:
+            base['user_data'] = self.user_data
+        prop_dict = self.properties.to_dict()
+        if prop_dict is not None:
+            base['properties'] = prop_dict
         return base
 
     @staticmethod
     def _extract_ep_properties(ep_strs, schedule_dict):
         """Extract relevant EnergyPlus properties from a list of strings."""
         # check the inputs
-        assert ep_strs[3].lower() == 'watts/area', 'Equipment must use ' \
-            'Watts/Area method to be loaded from IDF to honeybee.'
+        if len(ep_strs) > 3:
+            assert ep_strs[3].lower() == 'watts/area', 'Equipment must use ' \
+                'Watts/Area method to be loaded from IDF to honeybee.'
+        else:
+            raise AssertionError('Equipment must use Watts/Area method to be loaded '
+                                 'from IDF to honeybee.')
         # extract the properties from the string
         rad_fract = 0
         lat_fract = 0
         lost_fract = 0
         try:
-            rad_fract = ep_strs[8] if ep_strs[8] != '' else 0
             lat_fract = ep_strs[7] if ep_strs[7] != '' else 0
+            rad_fract = ep_strs[8] if ep_strs[8] != '' else 0
             lost_fract = ep_strs[9] if ep_strs[9] != '' else 0
         except IndexError:
             pass  # shorter equipment definition lacking fractions
@@ -241,11 +294,13 @@ class _EquipmentBase(_LoadBase):
             self.identifier, self.watts_per_area, self.schedule,
             self.radiant_fraction, self.latent_fraction, self.lost_fraction)
         new_obj._display_name = self._display_name
+        new_obj._user_data = None if self._user_data is None else self._user_data.copy()
         return new_obj
 
     def __repr__(self):
-        return 'Equipment:\n name: {}\n watts per area: {}\n schedule: ' \
-            '{}'.format(self.identifier, self.watts_per_area, self.schedule.identifier)
+        return '{}: {} [{} W/m2] [schedule: {}]'.format(
+            self.__class__.__name__, self.display_name, round(self.watts_per_area, 1),
+            self.schedule.display_name)
 
 
 @lockable
@@ -263,13 +318,13 @@ class ElectricEquipment(_EquipmentBase):
             Fractional and the fractional values will get multiplied by the
             watts_per_area to yield a complete equipment profile.
         radiant_fraction: A number between 0 and 1 for the fraction of the total
-            equipment load given off as long wave radiant heat. Default: 0.
+            equipment load given off as long wave radiant heat. (Default: 0).
         latent_fraction: A number between 0 and 1 for the fraction of the total
-            equipment load that is latent (as opposed to sensible). Default: 0.
+            equipment load that is latent (as opposed to sensible). (Default: 0).
         lost_fraction: A number between 0 and 1 for the fraction of the total
             equipment load that is lost outside of the zone and the HVAC system.
             Typically, this is used to represent heat that is exhausted directly
-            out of a zone (as you would for a stove). Default: 0.
+            out of a zone (as you would for a stove). (Default: 0).
 
     Properties:
         * identifier
@@ -280,6 +335,7 @@ class ElectricEquipment(_EquipmentBase):
         * latent_fraction
         * lost_fraction
         * convected_fraction
+        * user_data
     """
     __slots__ = ()
 
@@ -288,6 +344,7 @@ class ElectricEquipment(_EquipmentBase):
         """Initialize Electric Equipment."""
         _EquipmentBase.__init__(self, identifier, watts_per_area, schedule,
                                 radiant_fraction, latent_fraction, lost_fraction)
+        self._properties = ElectricEquipmentProperties(self)
 
     @classmethod
     def from_idf(cls, idf_string, schedule_dict):
@@ -350,6 +407,10 @@ class ElectricEquipment(_EquipmentBase):
                       sched, rad_f, lat_f, lost_f)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
         return new_obj
 
     @classmethod
@@ -382,6 +443,10 @@ class ElectricEquipment(_EquipmentBase):
                       sched, rad_f, lat_f, lost_f)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
         return new_obj
 
     def to_idf(self, zone_identifier):
@@ -453,12 +518,10 @@ class ElectricEquipment(_EquipmentBase):
             self.identifier, self.watts_per_area, self.schedule,
             self.radiant_fraction, self.latent_fraction, self.lost_fraction)
         new_obj._display_name = self._display_name
+        new_obj._user_data = None if self._user_data is None else self._user_data.copy()
+        new_obj._properties._duplicate_extension_attr(self._properties)
         return new_obj
 
-    def __repr__(self):
-        return 'ElectricEquipment:\n name: {}\n watts per area: {}\n schedule: ' \
-            '{}'.format(self.identifier, self.watts_per_area, self.schedule.identifier)
-
 
 @lockable
 class GasEquipment(_EquipmentBase):
@@ -475,13 +538,13 @@ class GasEquipment(_EquipmentBase):
             Fractional and the fractional values will get multiplied by the
             watts_per_area to yield a complete equipment profile.
         radiant_fraction: A number between 0 and 1 for the fraction of the total
-            equipment load given off as long wave radiant heat. Default: 0.
+            equipment load given off as long wave radiant heat. (Default: 0).
         latent_fraction: A number between 0 and 1 for the fraction of the total
-            equipment load that is latent (as opposed to sensible). Default: 0.
+            equipment load that is latent (as opposed to sensible). (Default: 0).
         lost_fraction: A number between 0 and 1 for the fraction of the total
             equipment load that is lost outside of the zone and the HVAC system.
             Typically, this is used to represent heat that is exhausted directly
-            out of a zone (as you would for a stove). Default: 0.
+            out of a zone (as you would for a stove). (Default: 0).
 
     Properties:
         * identifier
@@ -492,6 +555,7 @@ class GasEquipment(_EquipmentBase):
         * latent_fraction
         * lost_fraction
         * convected_fraction
+        * user_data
     """
     __slots__ = ()
 
@@ -500,6 +564,7 @@ class GasEquipment(_EquipmentBase):
         """Initialize Gas Equipment."""
         _EquipmentBase.__init__(self, identifier, watts_per_area, schedule,
                                 radiant_fraction, latent_fraction, lost_fraction)
+        self._properties = GasEquipmentProperties(self)
 
     @classmethod
     def from_idf(cls, idf_string, schedule_dict):
@@ -562,6 +627,10 @@ class GasEquipment(_EquipmentBase):
                       sched, rad_f, lat_f, lost_f)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
         return new_obj
 
     @classmethod
@@ -594,6 +663,10 @@ class GasEquipment(_EquipmentBase):
                       sched, rad_f, lat_f, lost_f)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
         return new_obj
 
     def to_idf(self, zone_identifier):
@@ -664,8 +737,6 @@ class GasEquipment(_EquipmentBase):
             self.identifier, self.watts_per_area, self.schedule,
             self.radiant_fraction, self.latent_fraction, self.lost_fraction)
         new_obj._display_name = self._display_name
+        new_obj._user_data = None if self._user_data is None else self._user_data.copy()
+        new_obj._properties._duplicate_extension_attr(self._properties)
         return new_obj
-
-    def __repr__(self):
-        return 'GasEquipment:\n name: {}\n watts per area: {}\n schedule: ' \
-            '{}'.format(self.identifier, self.watts_per_area, self.schedule.identifier)
