@@ -2,14 +2,15 @@
 """Complete definition of infiltration in a simulation, including schedule and load."""
 from __future__ import division
 
+from honeybee._lockable import lockable
+from honeybee.typing import float_positive, clean_and_id_ep_string
+
 from ._base import _LoadBase
 from ..schedule.ruleset import ScheduleRuleset
 from ..schedule.fixedinterval import ScheduleFixedInterval
 from ..reader import parse_idf_string
 from ..writer import generate_idf_string
-
-from honeybee._lockable import lockable
-from honeybee.typing import float_positive
+from ..properties.extension import InfiltrationProperties
 
 
 @lockable
@@ -57,6 +58,7 @@ class Infiltration(_LoadBase):
         * constant_coefficient
         * temperature_coefficient
         * velocity_coefficient
+        * user_data
     """
     __slots__ = ('_flow_per_exterior_area', '_schedule', '_constant_coefficient',
                  '_temperature_coefficient', '_velocity_coefficient')
@@ -71,6 +73,7 @@ class Infiltration(_LoadBase):
         self.constant_coefficient = constant_coefficient
         self.temperature_coefficient = temperature_coefficient
         self.velocity_coefficient = velocity_coefficient
+        self._properties = InfiltrationProperties(self)
 
     @property
     def flow_per_exterior_area(self):
@@ -125,28 +128,6 @@ class Infiltration(_LoadBase):
         self._temperature_coefficient = float_positive(
             value, 'infiltration temperature coefficient')
 
-    @property
-    def constant_coefficient(self):
-        """Get or set the infiltration fraction remaining constant despite the outdoors.
-        """
-        return self._constant_coefficient
-
-    @constant_coefficient.setter
-    def constant_coefficient(self, value):
-        self._constant_coefficient = float_positive(
-            value, 'infiltration constant coefficient')
-
-    @property
-    def temperature_coefficient(self):
-        """Get or set the coefficient for the interior/exterior temperature difference.
-        """
-        return self._temperature_coefficient
-
-    @temperature_coefficient.setter
-    def temperature_coefficient(self, value):
-        self._temperature_coefficient = float_positive(
-            value, 'infiltration temperature coefficient')
-
     @property
     def velocity_coefficient(self):
         """Get or set the coefficient for the exterior wind speed."""
@@ -157,6 +138,49 @@ class Infiltration(_LoadBase):
         self._velocity_coefficient = float_positive(
             value, 'infiltration velocity coefficient')
 
+    def diversify(self, count, flow_stdev=20, schedule_offset=1, timestep=1,
+                  schedule_indices=None):
+        """Get an array of diversified Infiltration derived from this "average" one.
+
+        Approximately 2/3 of the schedules in the output objects will be offset
+        from the mean by the input schedule_offset (1/3 ahead and another 1/3 behind).
+
+        Args:
+            count: An positive integer for the number of diversified objects to
+                generate from this mean object.
+            flow_stdev: A number between 0 and 100 for the percent of the
+                flow_per_exterior_area representing one standard deviation of
+                diversification from the mean. (Default 20 percent).
+            schedule_offset: A positive integer for the number of timesteps at which
+                the lighting schedule of the resulting objects will be shifted - roughly
+                1/3 of the objects ahead and another 1/3 behind. (Default: 1).
+            timestep: An integer for the number of timesteps per hour at which the
+                shifting is occurring. This must be a value between 1 and 60, which
+                is evenly divisible by 60. 1 indicates that each step is an hour
+                while 60 indicates that each step is a minute. (Default: 1).
+            schedule_indices: An optional list of integers from 0 to 2 with a length
+                equal to the input count, which will be used to set whether a given
+                schedule is behind (0), ahead (2), or the same (1). This can be
+                used to coordinate schedules across diversified programs. If None
+                a random list of integers will be genrated. (Default: None).
+        """
+        # generate shifted schedules and gaussian distribution of flow_per_exterior_area
+        usage_schs = self._shift_schedule(self.schedule, schedule_offset, timestep)
+        stdev = self.flow_per_exterior_area * (flow_stdev / 100)
+        new_loads, sch_ints = self._gaussian_values(
+            count, self.flow_per_exterior_area, stdev)
+        sch_ints = sch_ints if schedule_indices is None else schedule_indices
+
+        # generate the new objects and return them
+        new_objects = []
+        for load_val, sch_int in zip(new_loads, sch_ints):
+            new_obj = self.duplicate()
+            new_obj.identifier = clean_and_id_ep_string(self.identifier)
+            new_obj.flow_per_exterior_area = load_val
+            new_obj.schedule = usage_schs[sch_int]
+            new_objects.append(new_obj)
+        return new_objects
+
     @classmethod
     def from_idf(cls, idf_string, schedule_dict):
         """Create an Infiltration object from an EnergyPlus IDF text string.
@@ -240,6 +264,10 @@ class Infiltration(_LoadBase):
                       sched, const, tem, vel)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
         return new_obj
 
     @classmethod
@@ -277,6 +305,10 @@ class Infiltration(_LoadBase):
                       sched, const, tem, vel)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
         return new_obj
 
     def to_idf(self, zone_identifier):
@@ -323,6 +355,11 @@ class Infiltration(_LoadBase):
             base['velocity_coefficient'] = self.velocity_coefficient
         if self._display_name is not None:
             base['display_name'] = self.display_name
+        if self._user_data is not None:
+            base['user_data'] = self._user_data
+        prop_dict = self.properties.to_dict()
+        if prop_dict is not None:
+            base['properties'] = prop_dict
         return base
 
     @staticmethod
@@ -397,9 +434,11 @@ class Infiltration(_LoadBase):
             self.constant_coefficient, self.temperature_coefficient,
             self.velocity_coefficient)
         new_obj._display_name = self._display_name
+        new_obj._user_data = None if self._user_data is None else self._user_data.copy()
+        new_obj._properties._duplicate_extension_attr(self._properties)
         return new_obj
 
     def __repr__(self):
-        return 'Infiltration:\n name: {}\n flow per exterior area: {}\n schedule: ' \
-            '{}'.format(self.identifier, self.flow_per_exterior_area,
-                        self.schedule.identifier)
+        return 'Infiltration: {} [{} m3/s-m2] [schedule: {}]'.format(
+            self.display_name, round(self.flow_per_exterior_area, 6),
+            self.schedule.display_name)
