@@ -2,14 +2,15 @@
 """Complete definition of lighting in a simulation, including schedule and load."""
 from __future__ import division
 
+from honeybee._lockable import lockable
+from honeybee.typing import float_in_range, float_positive, clean_and_id_ep_string
+
 from ._base import _LoadBase
 from ..schedule.ruleset import ScheduleRuleset
 from ..schedule.fixedinterval import ScheduleFixedInterval
 from ..reader import parse_idf_string
 from ..writer import generate_idf_string
-
-from honeybee._lockable import lockable
-from honeybee.typing import float_in_range, float_positive
+from ..properties.extension import LightingProperties
 
 
 @lockable
@@ -28,13 +29,13 @@ class Lighting(_LoadBase):
             watts_per_area to yield a complete lighting profile.
         return_air_fraction: A number between 0 and 1 for the fraction of the total
             lighting load that goes into the zone return air (into the zone outlet
-            node). Default: 0.0 (representative of pendant lighting).
+            node). (Default: 0.0). (representative of pendant lighting).
         radiant_fraction: A number between 0 and 1 for the fraction of the total
             lighting load given off as long wave radiant heat.
-            Default: 0.32 (representative of pendant lighting).
+            (Default: 0.32). (representative of pendant lighting).
         visible_fraction: A number between 0 and 1 for the fraction of the total
             lighting load given off as short wave visible light.
-            Default: 0.25  (representative of pendant lighting).
+            (Default: 0.25).  (representative of pendant lighting).
 
     Properties:
         * identifier
@@ -45,9 +46,11 @@ class Lighting(_LoadBase):
         * radiant_fraction
         * visible_fraction
         * convected_fraction
+        * baseline_watts_per_area
+        * user_data
     """
     __slots__ = ('_watts_per_area', '_schedule', '_return_air_fraction',
-                 '_radiant_fraction', '_visible_fraction')
+                 '_radiant_fraction', '_visible_fraction', '_baseline_watts_per_area')
 
     def __init__(self, identifier, watts_per_area, schedule, return_air_fraction=0.0,
                  radiant_fraction=0.32, visible_fraction=0.25):
@@ -61,6 +64,8 @@ class Lighting(_LoadBase):
         self.return_air_fraction = return_air_fraction
         self.radiant_fraction = radiant_fraction
         self.visible_fraction = visible_fraction
+        self.baseline_watts_per_area = None  # can be set by the user later
+        self._properties = LightingProperties(self)
 
     @property
     def watts_per_area(self):
@@ -133,9 +138,69 @@ class Lighting(_LoadBase):
         tot = (self._return_air_fraction, self._radiant_fraction, self._visible_fraction)
         return 1 - sum(tot)
 
+    @property
+    def baseline_watts_per_area(self):
+        """Get or set the baseline lighting power density in W/m2 of floor area.
+
+        This baseline is useful to track how much better the installed lights are
+        in comparison to a standard like ASHRAE 90.1. If set to None, it will
+        default to 11.84029 W/m2, which is that ASHRAE 90.1-2004 baseline for
+        an office.
+        """
+        return self._baseline_watts_per_area if self._baseline_watts_per_area \
+            is not None else 11.84029
+
+    @baseline_watts_per_area.setter
+    def baseline_watts_per_area(self, value):
+        if value is not None:
+            value = float_positive(value, 'lighting baseline watts per area')
+        self._baseline_watts_per_area = value
+
+    def diversify(self, count, watts_stdev=20, schedule_offset=1, timestep=1,
+                  schedule_indices=None):
+        """Get an array of diversified Lighting derived from this "average" one.
+
+        Approximately 2/3 of the schedules in the output objects will be offset
+        from the mean by the input schedule_offset (1/3 ahead and another 1/3 behind).
+
+        Args:
+            count: An positive integer for the number of diversified objects to
+                generate from this mean object.
+            watts_stdev: A number between 0 and 100 for the percent of the
+                watts_per_area representing one standard deviation
+                of diversification from the mean. (Default 20 percent).
+            schedule_offset: A positive integer for the number of timesteps at which
+                the lighting schedule of the resulting objects will be shifted - roughly
+                1/3 of the objects ahead and another 1/3 behind. (Default: 1).
+            timestep: An integer for the number of timesteps per hour at which the
+                shifting is occurring. This must be a value between 1 and 60, which
+                is evenly divisible by 60. 1 indicates that each step is an hour
+                while 60 indicates that each step is a minute. (Default: 1).
+            schedule_indices: An optional list of integers from 0 to 2 with a length
+                equal to the input count, which will be used to set whether a given
+                schedule is behind (0), ahead (2), or the same (1). This can be
+                used to coordinate schedules across diversified programs. If None
+                a random list of integers will be generated. (Default: None).
+        """
+        # generate shifted schedules and a gaussian distribution of watts_per_area
+        usage_schs = self._shift_schedule(self.schedule, schedule_offset, timestep)
+        stdev = self.watts_per_area * (watts_stdev / 100)
+        new_loads, sch_ints = self._gaussian_values(count, self.watts_per_area, stdev)
+        sch_ints = sch_ints if schedule_indices is None else schedule_indices
+
+        # generate the new objects and return them
+        new_objects = []
+        for load_val, sch_int in zip(new_loads, sch_ints):
+            new_obj = self.duplicate()
+            new_obj.identifier = clean_and_id_ep_string(self.identifier)
+            new_obj.watts_per_area = load_val
+            new_obj.schedule = usage_schs[sch_int]
+            new_objects.append(new_obj)
+        return new_objects
+
     @classmethod
     def from_idf(cls, idf_string, schedule_dict):
-        """Create an Lighting object from an EnergyPlus IDF text string.
+        """Create a Lighting object from an EnergyPlus IDF text string.
 
         Note that the Lighting idf_string must use the 'watts per zone floor area'
         method in order to be successfully imported.
@@ -212,9 +277,11 @@ class Lighting(_LoadBase):
         ret_fract, rad_fract, vis_fract = cls._optional_dict_keys(data)
         new_obj = cls(data['identifier'], data['watts_per_area'], sched,
                       ret_fract, rad_fract, vis_fract)
-        if 'display_name' in data and data['display_name'] is not None:
-            new_obj.display_name = data['display_name']
-        return new_obj
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
+        return cls._apply_optional_dict_props(new_obj, data)
 
     @classmethod
     def from_dict_abridged(cls, data, schedule_dict):
@@ -249,9 +316,11 @@ class Lighting(_LoadBase):
         ret_fract, rad_fract, vis_fract = cls._optional_dict_keys(data)
         new_obj = cls(data['identifier'], data['watts_per_area'], sched,
                       ret_fract, rad_fract, vis_fract)
-        if 'display_name' in data and data['display_name'] is not None:
-            new_obj.display_name = data['display_name']
-        return new_obj
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
+        return cls._apply_optional_dict_props(new_obj, data)
 
     def to_idf(self, zone_identifier):
         """IDF string representation of Lighting object.
@@ -272,7 +341,7 @@ class Lighting(_LoadBase):
         comments = ('name', 'zone name', 'schedule name', 'lighting level method',
                     'lighting power level {W}', 'lighting per floor area {W/m2}',
                     'lighting per person {W/ppl}', 'return air fraction',
-                    'radiant fration', 'visible fraction')
+                    'radiant fraction', 'visible fraction')
         return generate_idf_string('Lights', values, comments)
 
     def to_dict(self, abridged=False):
@@ -293,6 +362,13 @@ class Lighting(_LoadBase):
             abridged else self.schedule.identifier
         if self._display_name is not None:
             base['display_name'] = self.display_name
+        if self._baseline_watts_per_area is not None:
+            base['baseline_watts_per_area'] = self._baseline_watts_per_area
+        if self._user_data is not None:
+            base['user_data'] = self._user_data
+        prop_dict = self.properties.to_dict()
+        if prop_dict is not None:
+            base['properties'] = prop_dict
         return base
 
     @staticmethod
@@ -321,9 +397,12 @@ class Lighting(_LoadBase):
 
         # calculate the average values
         lpd = sum([li.watts_per_area * w for li, w in zip(lightings, weights)])
-        ret_fract = sum([li.return_air_fraction * w for li, w in zip(lightings, u_weights)])
-        rad_fract = sum([li.radiant_fraction * w for li, w in zip(lightings, u_weights)])
-        vis_fract = sum([li.visible_fraction * w for li, w in zip(lightings, u_weights)])
+        ret_fract = sum(
+            [li.return_air_fraction * w for li, w in zip(lightings, u_weights)])
+        rad_fract = sum(
+            [li.radiant_fraction * w for li, w in zip(lightings, u_weights)])
+        vis_fract = sum(
+            [li.visible_fraction * w for li, w in zip(lightings, u_weights)])
 
         # calculate the average schedules
         sched = Lighting._average_schedule(
@@ -347,6 +426,16 @@ class Lighting(_LoadBase):
         vis_fract = data['visible_fraction'] if 'visible_fraction' in data else 0.25
         return ret_fract, rad_fract, vis_fract
 
+    @staticmethod
+    def _apply_optional_dict_props(new_obj, data):
+        """Apply optional properties like display_name to an object from a dictionary."""
+        if 'display_name' in data and data['display_name'] is not None:
+            new_obj.display_name = data['display_name']
+        if 'baseline_watts_per_area' in data and \
+                data['baseline_watts_per_area'] is not None:
+            new_obj.baseline_watts_per_area = data['baseline_watts_per_area']
+        return new_obj
+
     def __key(self):
         """A tuple based on the object properties, useful for hashing."""
         return (self.identifier, self.watts_per_area, hash(self.schedule),
@@ -366,8 +455,12 @@ class Lighting(_LoadBase):
             self.identifier, self.watts_per_area, self.schedule,
             self.return_air_fraction, self.radiant_fraction, self.visible_fraction)
         new_obj._display_name = self._display_name
+        new_obj._baseline_watts_per_area = self._baseline_watts_per_area
+        new_obj._user_data = None if self._user_data is None else self._user_data.copy()
+        new_obj._properties._duplicate_extension_attr(self._properties)
         return new_obj
 
     def __repr__(self):
-        return 'Lighting:\n name: {}\n watts per area: {}\n schedule: ' \
-            '{}'.format(self.identifier, self.watts_per_area, self.schedule.identifier)
+        return 'Lighting: {} [{} W/m2] [schedule: {}]'.format(
+            self.display_name, round(self.watts_per_area, 1),
+            self.schedule.display_name)
