@@ -2,18 +2,19 @@
 """Complete definition of people in a simulation, including schedule and load."""
 from __future__ import division
 
+from honeybee._lockable import lockable
+from honeybee.typing import float_in_range, float_positive, clean_and_id_ep_string
+from honeybee.altnumber import autocalculate
+
 from ._base import _LoadBase
 from ..schedule.ruleset import ScheduleRuleset
 from ..schedule.fixedinterval import ScheduleFixedInterval
 from ..reader import parse_idf_string
 from ..writer import generate_idf_string
+from ..properties.extension import PeopleProperties
 
 import honeybee_energy.lib.schedules as _sched_lib
 
-from honeybee._lockable import lockable
-from honeybee.typing import float_in_range, float_positive
-from honeybee.altnumber import autocalculate
-
 
 @lockable
 class People(_LoadBase):
@@ -31,18 +32,18 @@ class People(_LoadBase):
             the people_per_area to yield a complete occupancy profile.
         activity_schedule: A ScheduleRuleset or ScheduleFixedInterval for the
             activity of the occupants over the course of the year. The type of
-            this schedule should be Power and the values of the schedule equal
+            this schedule should be ActivityLevel and the values of the schedule equal
             to the number of Watts given off by an individual person in the room.
-            If None, it will a default constant schedule with 120 Watts per person
+            If None, a default constant schedule with 120 Watts per person
             will be used, which is typical of awake, adult humans who are seated.
-        radiant_fraction: A number between 0 and 1 for the fraction of the sensible
-            heat given off by people that is radiant (as opposed to convective).
-            Default: 0.3.
+        radiant_fraction: A number between 0 and 1 for the fraction of the
+            sensible heat given off by people that is radiant (as opposed to
+            convective). (Default: 0.3).
         latent_fraction: A number between 0 and 1 for the fraction of the heat
             given off by people that is latent (as opposed to sensible). This
             input can also be an Autocalculate object, which will automatically
             estimate the latent fraction based on the occupant's activity level.
-            Default: autocalculate.
+            (Default: autocalculate).
 
     Properties:
         * identifier
@@ -53,6 +54,7 @@ class People(_LoadBase):
         * activity_schedule
         * radiant_fraction
         * latent_fraction
+        * user_data
     """
     __slots__ = ('_people_per_area', '_occupancy_schedule', '_activity_schedule',
                  '_radiant_fraction', '_latent_fraction')
@@ -67,6 +69,7 @@ class People(_LoadBase):
         self.activity_schedule = activity_schedule
         self.radiant_fraction = radiant_fraction
         self.latent_fraction = latent_fraction
+        self._properties = PeopleProperties(self)
 
     @property
     def people_per_area(self):
@@ -143,6 +146,49 @@ class People(_LoadBase):
             self._latent_fraction = float_in_range(
                 value, 0.0, 1.0, 'people latent fraction')
 
+    def diversify(self, count, occupancy_stdev=20, schedule_offset=1, timestep=1,
+                  schedule_indices=None):
+        """Get an array of diversified People derived from this "average" one.
+
+        Approximately 2/3 of the schedules in the output objects will be offset
+        from the mean by the input schedule_offset (1/3 ahead and another 1/3 behind).
+
+        Args:
+            count: An positive integer for the number of diversified objects to
+                generate from this mean object.
+            occupancy_stdev: A number between 0 and 100 for the percent of the
+                occupancy people_per_area representing one standard deviation
+                of diversification from the mean. (Default 20 percent).
+            schedule_offset: A positive integer for the number of timesteps at which
+                the occupancy schedule of the resulting objects will be shifted - roughly
+                1/3 of the objects ahead and another 1/3 behind. (Default: 1).
+            timestep: An integer for the number of timesteps per hour at which the
+                shifting is occurring. This must be a value between 1 and 60, which
+                is evenly divisible by 60. 1 indicates that each step is an hour
+                while 60 indicates that each step is a minute. (Default: 1).
+            schedule_indices: An optional list of integers from 0 to 2 with a length
+                equal to the input count, which will be used to set whether a given
+                schedule is behind (0), ahead (2), or the same (1). This can be
+                used to coordinate schedules across diversified programs. If None
+                a random list of integers will be generated. (Default: None).
+        """
+        # generate shifted schedules and a gaussian distribution of people_per_area
+        occ_schs = self._shift_schedule(
+            self.occupancy_schedule, schedule_offset, timestep)
+        stdev = self.people_per_area * (occupancy_stdev / 100)
+        new_loads, sch_ints = self._gaussian_values(count, self.people_per_area, stdev)
+        sch_ints = sch_ints if schedule_indices is None else schedule_indices
+
+        # generate the new objects and return them
+        new_objects = []
+        for load_val, sch_int in zip(new_loads, sch_ints):
+            new_obj = self.duplicate()
+            new_obj.identifier = clean_and_id_ep_string(self.identifier)
+            new_obj.people_per_area = load_val
+            new_obj.occupancy_schedule = occ_schs[sch_int]
+            new_objects.append(new_obj)
+        return new_objects
+
     @classmethod
     def from_idf(cls, idf_string, schedule_dict):
         """Create an People object from an EnergyPlus IDF text string.
@@ -213,12 +259,17 @@ class People(_LoadBase):
             'Expected People dictionary. Got {}.'.format(data['type'])
         occ_sched = cls._get_schedule_from_dict(data['occupancy_schedule'])
         act_sched = cls._get_schedule_from_dict(data['activity_schedule']) if \
-            'activity_schedule' in data and data['activity_schedule'] is not None else None
+            'activity_schedule' in data and data['activity_schedule'] is not None \
+            else None
         rad_fract, lat_fract = cls._optional_dict_keys(data)
         new_obj = cls(data['identifier'], data['people_per_area'], occ_sched, act_sched,
                       rad_fract, lat_fract)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
         return new_obj
 
     @classmethod
@@ -256,6 +307,10 @@ class People(_LoadBase):
                       activity_sched, rad_fract, lat_fract)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
         return new_obj
 
     def to_idf(self, zone_identifier):
@@ -280,7 +335,7 @@ class People(_LoadBase):
                   self.activity_schedule.identifier)
         comments = ('name', 'zone name', 'occupancy schedule name', 'occupancy method',
                     'number of people {ppl}', 'people per floor area {ppl/m2}',
-                    'floor area per person {m2/ppl}', 'radiant fration',
+                    'floor area per person {m2/ppl}', 'radiant fraction',
                     'sensible heat fraction', 'activity schedule name')
         return generate_idf_string('People', values, comments)
 
@@ -306,6 +361,11 @@ class People(_LoadBase):
             base['activity_schedule'] = self.activity_schedule.identifier
         if self._display_name is not None:
             base['display_name'] = self.display_name
+        if self._user_data is not None:
+            base['user_data'] = self.user_data
+        prop_dict = self.properties.to_dict()
+        if prop_dict is not None:
+            base['properties'] = prop_dict
         return base
 
     @staticmethod
@@ -358,16 +418,21 @@ class People(_LoadBase):
     def _check_activity_schedule_type(self, schedule):
         """Check that the type limit of an input schedule is fractional."""
         if schedule.schedule_type_limit is not None:
-            assert schedule.schedule_type_limit.unit == 'W', 'Activity schedule ' \
-                'should be in Watts [ActivityLevel]. Got a schedule of unit_type ' \
-                '[{}].'.format(schedule.schedule_type_limit.unit_type)
+            t_lim = schedule.schedule_type_limit
+            assert t_lim.unit_type == 'ActivityLevel', 'Activity schedule must have a ' \
+                'unit type of ActivityLevel. Got a schedule' \
+                ' of unit type [{}].'.format(t_lim.unit_type)
+            assert t_lim.lower_limit == 0, 'Activity schedule should have either ' \
+                'no type limit or a lower limit of 0. Got a schedule type with ' \
+                'lower limit [{}].'.format(t_lim.lower_limit)
 
     @staticmethod
     def _optional_dict_keys(data):
         """Get the optional keys from a People dictionary."""
         rad_fract = data['radiant_fraction'] if 'radiant_fraction' in data else 0.3
         lat_fract = autocalculate if 'latent_fraction' not in data or \
-            data['latent_fraction'] == autocalculate.to_dict() else data['latent_fraction']
+            data['latent_fraction'] == autocalculate.to_dict() \
+            else data['latent_fraction']
         return rad_fract, lat_fract
 
     @staticmethod
@@ -377,13 +442,15 @@ class People(_LoadBase):
             occ_sched = schedule_dict[occ_sch_id]
         except KeyError as e:
             raise ValueError('Failed to find {} in the schedule_dict.'.format(e))
-        if act_sch_id.lower() == 'seated adult activity':
+        if act_sch_id == '' or act_sch_id.lower() == 'seated adult activity':
             activity_sched = None
         else:
             try:
                 activity_sched = schedule_dict[act_sch_id]
             except KeyError as e:
-                raise ValueError('Failed to find {} in the People schedule_dict.'.format(e))
+                raise ValueError(
+                    'Failed to find {} in the People schedule_dict.'.format(e)
+                )
         return occ_sched, activity_sched
 
     def __key(self):
@@ -406,9 +473,11 @@ class People(_LoadBase):
             self.identifier, self.people_per_area, self.occupancy_schedule,
             self.activity_schedule, self.radiant_fraction, self.latent_fraction)
         new_obj._display_name = self._display_name
+        new_obj._user_data = None if self._user_data is None else self._user_data.copy()
+        new_obj._properties._duplicate_extension_attr(self._properties)
         return new_obj
 
     def __repr__(self):
-        return 'People:\n name: {}\n people per area: {}\n schedule: ' \
-            '{}'.format(self.identifier, self.people_per_area,
-                        self.occupancy_schedule.identifier)
+        return 'People: {} [{} people/m2] [schedule: {}]'.format(
+            self.display_name, round(self.people_per_area, 3),
+            self.occupancy_schedule.display_name)
