@@ -2,17 +2,18 @@
 """Complete definition of ventilation in a simulation, including schedule and load."""
 from __future__ import division
 
+from honeybee._lockable import lockable
+from honeybee.typing import float_positive
+
 from ._base import _LoadBase
 from ..schedule.ruleset import ScheduleRuleset
 from ..schedule.fixedinterval import ScheduleFixedInterval
 from ..reader import parse_idf_string
 from ..writer import generate_idf_string
+from ..properties.extension import VentilationProperties
 
 import honeybee_energy.lib.scheduletypelimits as _type_lib
 
-from honeybee._lockable import lockable
-from honeybee.typing import float_positive
-
 
 @lockable
 class Ventilation(_LoadBase):
@@ -61,10 +62,10 @@ class Ventilation(_LoadBase):
         * flow_per_zone
         * air_changes_per_hour
         * schedule
+        * user_data
     """
     __slots__ = ('_flow_per_person', '_flow_per_area', '_flow_per_zone',
                  '_air_changes_per_hour', '_schedule')
-    CALCULATION_METHODS = ('Sum', 'Maximum')
 
     def __init__(self, identifier, flow_per_person=0, flow_per_area=0, flow_per_zone=0,
                  air_changes_per_hour=0, schedule=None):
@@ -75,6 +76,7 @@ class Ventilation(_LoadBase):
         self.flow_per_zone = flow_per_zone
         self.air_changes_per_hour = air_changes_per_hour
         self.schedule = schedule
+        self._properties = VentilationProperties(self)
 
     @property
     def flow_per_person(self):
@@ -238,6 +240,10 @@ class Ventilation(_LoadBase):
         new_obj = cls(data['identifier'], person, area, zone, ach, sched)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
         return new_obj
 
     @classmethod
@@ -276,6 +282,10 @@ class Ventilation(_LoadBase):
         new_obj = cls(data['identifier'], person, area, zone, ach, sched)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
         return new_obj
 
     def to_idf(self, zone_identifier):
@@ -322,6 +332,11 @@ class Ventilation(_LoadBase):
                 abridged else self.schedule.identifier
         if self._display_name is not None:
             base['display_name'] = self.display_name
+        if self._user_data is not None:
+            base['user_data'] = self.user_data
+        prop_dict = self.properties.to_dict()
+        if prop_dict is not None:
+            base['properties'] = prop_dict
         return base
 
     @staticmethod
@@ -402,10 +417,11 @@ class Ventilation(_LoadBase):
             self.identifier, self.flow_per_person, self.flow_per_area,
             self.flow_per_zone, self.air_changes_per_hour, self.schedule)
         new_obj._display_name = self._display_name
+        new_obj._user_data = None if self._user_data is None else self._user_data.copy()
+        new_obj._properties._duplicate_extension_attr(self._properties)
         return new_obj
 
     def __repr__(self):
-        return 'Ventilation:\n name: {}\n flow per person: {}\n flow per area: ' \
-            '{}\n flow per zone: {}\n ACH: {}'.format(
-                self.identifier, self.flow_per_person, self.flow_per_area,
-                self.flow_per_zone, self.air_changes_per_hour)
+        return 'Ventilation: {} [{} m3/s-person] [{} m3/s-m2] [{} ACH]'.format(
+            self.display_name, round(self.flow_per_person, 4),
+            round(self.flow_per_area, 6), round(self.air_changes_per_hour, 2))
