@@ -332,6 +332,7 @@ class EnergyWindowMaterialGas(_EnergyWindowMaterialGasBase):
         * specific_heat
         * density
         * prandtl
+        * user_data
     """
     __slots__ = ('_gas_type',)
 
@@ -376,6 +377,9 @@ class EnergyWindowMaterialGas(_EnergyWindowMaterialGasBase):
             idf_string: A text string fully describing an EnergyPlus material.
         """
         ep_strs = parse_idf_string(idf_string, 'WindowMaterial:Gas,')
+        assert ep_strs[1].title() != 'Custom', \
+            'Honeybee EnergyWindowMaterialGas cannot use EnergyPlus Custom gas type.\n' \
+            'Use honeybee EnergyWindowMaterialGasCustom instead.'
         return cls(ep_strs[0], ep_strs[2], ep_strs[1])
 
     @classmethod
@@ -402,6 +406,8 @@ class EnergyWindowMaterialGas(_EnergyWindowMaterialGasBase):
         new_obj = cls(data['identifier'], thickness, gas_type)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
         return new_obj
 
     def to_idf(self):
@@ -420,6 +426,8 @@ class EnergyWindowMaterialGas(_EnergyWindowMaterialGasBase):
         }
         if self._display_name is not None:
             base['display_name'] = self.display_name
+        if self._user_data is not None:
+            base['user_data'] = self.user_data
         return base
 
     def _coeff_property(self, dictionary, t_kelvin):
@@ -448,6 +456,7 @@ class EnergyWindowMaterialGas(_EnergyWindowMaterialGasBase):
     def __copy__(self):
         new_obj = EnergyWindowMaterialGas(self.identifier, self.thickness, self.gas_type)
         new_obj._display_name = self._display_name
+        new_obj._user_data = None if self._user_data is None else self._user_data.copy()
         return new_obj
 
 
@@ -589,6 +598,8 @@ class EnergyWindowMaterialGasMixture(_EnergyWindowMaterialGasBase):
                       data['gas_fractions'])
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
         return new_obj
 
     def to_idf(self):
@@ -613,6 +624,8 @@ class EnergyWindowMaterialGasMixture(_EnergyWindowMaterialGasBase):
         }
         if self._display_name is not None:
             base['display_name'] = self.display_name
+        if self._user_data is not None:
+            base['user_data'] = self.user_data
         return base
 
     def _weighted_avg_coeff_property(self, dictionary, t_kelvin):
@@ -644,6 +657,7 @@ class EnergyWindowMaterialGasMixture(_EnergyWindowMaterialGasBase):
         new_obj = EnergyWindowMaterialGasMixture(
             self.identifier, self.thickness, self.gas_types, self.gas_fractions)
         new_obj._display_name = self._display_name
+        new_obj._user_data = None if self._user_data is None else self._user_data.copy()
         return new_obj
 
 
@@ -741,6 +755,7 @@ class EnergyWindowMaterialGasCustom(_EnergyWindowMaterialGasBase):
         self.specific_heat_coeff_c = specific_heat_coeff_c
         self.specific_heat_ratio = specific_heat_ratio
         self.molecular_weight = molecular_weight
+        self._user_data = None
 
     @property
     def conductivity_coeff_a(self):
@@ -866,10 +881,24 @@ class EnergyWindowMaterialGasCustom(_EnergyWindowMaterialGasBase):
         Args:
             idf_string: A text string fully describing an EnergyPlus material.
         """
+        # check that the gas is, in fact custom
         ep_s = parse_idf_string(idf_string, 'WindowMaterial:Gas,')
         assert ep_s[1].title() == 'Custom', 'Exected Custom Gas. Got a specific one.'
+        assert len(ep_s) == 14, 'Not enough fields present for Custom Gas ' \
+            'IDF description. Expected 14 properties. Got {}.'.format(len(ep_s))
         ep_s.pop(1)
-        return cls(*ep_s)
+        # reorder the coefficients
+        start = [ep_s[0], ep_s[1]]
+        a_coef = [ep_s[2], ep_s[5], ep_s[8]]
+        b_coef = [ep_s[3], ep_s[6], ep_s[9]]
+        c_coef = [ep_s[4], ep_s[7], ep_s[10]]
+        end = [ep_s[12], ep_s[11]]
+        eps_cl = start + a_coef + b_coef + c_coef + end
+        # assume that any blank strings are just coefficients of 0
+        for i, val in enumerate(eps_cl[2:11]):
+            clean_val = val if val != '' else 0
+            eps_cl[i + 2] = clean_val
+        return cls(*eps_cl)
 
     @classmethod
     def from_dict(cls, data):
@@ -896,17 +925,23 @@ class EnergyWindowMaterialGasCustom(_EnergyWindowMaterialGasBase):
             'Expected EnergyWindowMaterialGasCustom. Got {}.'.format(data['type'])
         con_b = 0 if 'conductivity_coeff_b' not in data else data['conductivity_coeff_b']
         vis_b = 0 if 'viscosity_coeff_b' not in data else data['viscosity_coeff_b']
-        sph_b = 0 if 'specific_heat_coeff_b' not in data else data['specific_heat_coeff_b']
+        sph_b = 0 if 'specific_heat_coeff_b' not in data \
+            else data['specific_heat_coeff_b']
         con_c = 0 if 'conductivity_coeff_c' not in data else data['conductivity_coeff_c']
         vis_c = 0 if 'viscosity_coeff_c' not in data else data['viscosity_coeff_c']
-        sph_c = 0 if 'specific_heat_coeff_c' not in data else data['specific_heat_coeff_c']
+        sph_c = 0 if 'specific_heat_coeff_c' not in data \
+            else data['specific_heat_coeff_c']
         sphr = 1.0 if 'specific_heat_ratio' not in data else data['specific_heat_ratio']
         mw = 20.0 if 'molecular_weight' not in data else data['molecular_weight']
-        new_obj = cls(data['identifier'], data['thickness'], data['conductivity_coeff_a'],
-                      data['viscosity_coeff_a'], data['specific_heat_coeff_a'],
+        new_obj = cls(data['identifier'], data['thickness'],
+                      data['conductivity_coeff_a'],
+                      data['viscosity_coeff_a'],
+                      data['specific_heat_coeff_a'],
                       con_b, vis_b, sph_b, con_c, vis_c, sph_c, sphr, mw)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
         return new_obj
 
     def to_idf(self):
@@ -944,6 +979,8 @@ class EnergyWindowMaterialGasCustom(_EnergyWindowMaterialGasBase):
         }
         if self._display_name is not None:
             base['display_name'] = self.display_name
+        if self._user_data is not None:
+            base['user_data'] = self.user_data
         return base
 
     def __key(self):
@@ -977,4 +1014,5 @@ class EnergyWindowMaterialGasCustom(_EnergyWindowMaterialGasBase):
             self.viscosity_coeff_c, self.specific_heat_coeff_c,
             self.specific_heat_ratio, self.molecular_weight)
         new_obj._display_name = self._display_name
+        new_obj._user_data = None if self._user_data is None else self._user_data.copy()
         return new_obj
