@@ -8,6 +8,7 @@ represent an entire window assembly (including glazing, gaps, and frame), and
 therefore must be the only material in its parent construction.
 """
 from __future__ import division
+import math
 
 from ._base import _EnergyMaterialWindowBase
 from ..reader import parse_idf_string
@@ -15,6 +16,7 @@ from ..writer import generate_idf_string
 
 from honeybee._lockable import lockable
 from honeybee.typing import float_in_range, float_positive
+from honeybee.altnumber import autocalculate
 
 
 @lockable
@@ -80,6 +82,11 @@ class EnergyWindowMaterialGlazing(_EnergyWindowMaterialGlazingBase):
         * resistivity
         * u_value
         * r_value
+        * solar_absorptance
+        * visible_absorptance
+        * solar_transmissivity
+        * visible_transmissivity
+        * user_data
     """
     __slots__ = ('_thickness', '_solar_transmittance', '_solar_reflectance',
                  '_solar_reflectance_back', '_visible_transmittance',
@@ -95,12 +102,13 @@ class EnergyWindowMaterialGlazing(_EnergyWindowMaterialGlazingBase):
         """Initialize energy window material glazing."""
         _EnergyWindowMaterialGlazingBase.__init__(self, identifier)
 
-        # default for checking transmittance + reflectance < 1
+        # default values for checking transmittance + reflectance < 1
         self._solar_reflectance = 0
         self._solar_reflectance_back = None
         self._visible_reflectance = 0
         self._visible_reflectance_back = None
 
+        # assign the specified properties from __init__
         self.thickness = thickness
         self.solar_transmittance = solar_transmittance
         self.solar_reflectance = solar_reflectance
@@ -121,6 +129,7 @@ class EnergyWindowMaterialGlazing(_EnergyWindowMaterialGlazingBase):
     @thickness.setter
     def thickness(self, thick):
         self._thickness = float_positive(thick, 'glazing material thickness')
+        self._compare_thickness_conductivity()
 
     @property
     def solar_transmittance(self):
@@ -161,8 +170,8 @@ class EnergyWindowMaterialGlazing(_EnergyWindowMaterialGlazingBase):
     def solar_reflectance_back(self, s_ref):
         if s_ref is not None:
             s_ref = float_in_range(s_ref, 0.0, 1.0, 'glazing material solar reflectance')
-            assert s_ref + self._solar_transmittance <= 1, 'Sum of window transmittance ' \
-                'and reflectance ({}) is greater than 1.'.format(
+            assert s_ref + self._solar_transmittance <= 1, 'Sum of window ' \
+                'transmittance and reflectance ({}) is greater than 1.'.format(
                     s_ref + self._solar_transmittance)
         self._solar_reflectance_back = s_ref
 
@@ -252,6 +261,7 @@ class EnergyWindowMaterialGlazing(_EnergyWindowMaterialGlazingBase):
     @conductivity.setter
     def conductivity(self, cond):
         self._conductivity = float_positive(cond, 'glazing material conductivity')
+        self._compare_thickness_conductivity()
 
     @property
     def dirt_correction(self):
@@ -308,6 +318,26 @@ class EnergyWindowMaterialGlazing(_EnergyWindowMaterialGlazingBase):
         self._conductivity = self.thickness / \
             float_positive(r_val, 'glazing material r-value')
 
+    @property
+    def solar_absorptance(self):
+        """Get the solar absorptance of the glass at normal incidence."""
+        return 1 - self.solar_transmittance - self.solar_reflectance
+
+    @property
+    def visible_absorptance(self):
+        """Get the visible absorptance of the glass at normal incidence."""
+        return 1 - self.visible_transmittance - self.visible_reflectance
+
+    @property
+    def solar_transmissivity(self):
+        """Get the solar transmissivity of the glass at normal incidence."""
+        return self._transmissivity_from_transmittance(self._solar_transmittance)
+
+    @property
+    def visible_transmissivity(self):
+        """Get the visible transmissivity of the glass at normal incidence."""
+        return self._transmissivity_from_transmittance(self._visible_transmittance)
+
     @classmethod
     def from_idf(cls, idf_string):
         """Create EnergyWindowMaterialGlazing from an EnergyPlus text string.
@@ -316,6 +346,11 @@ class EnergyWindowMaterialGlazing(_EnergyWindowMaterialGlazingBase):
             idf_string: A text string fully describing an EnergyPlus material.
         """
         ep_s = parse_idf_string(idf_string, 'WindowMaterial:Glazing,')
+        idf_defaults = {10: 0, 11: 0.84, 12: 0.84, 13: 0.9, 14: 1.0, 15: 'No'}
+        for i, ep_str in enumerate(ep_s):  # fill in any default values
+            if ep_str == '' and i in idf_defaults:
+                ep_s[i] = idf_defaults[i]
+
         assert ep_s[1] == 'SpectralAverage', \
             'Expected SpectralAverage glazing type. Got {}.'.format(ep_s[1])
         new_mat = cls(ep_s[0], ep_s[3], ep_s[4], ep_s[5], ep_s[7], ep_s[8],
@@ -354,28 +389,45 @@ class EnergyWindowMaterialGlazing(_EnergyWindowMaterialGlazingBase):
         assert data['type'] == 'EnergyWindowMaterialGlazing', \
             'Expected EnergyWindowMaterialGlazing. Got {}.'.format(data['type'])
 
-        optional_keys = ('thickness', 'solar_transmittance', 'solar_reflectance',
-                         'solar_reflectance_back', 'visible_transmittance',
-                         'visible_reflectance', 'visible_reflectance_back',
-                         'infrared_transmittance', 'emissivity', 'emissivity_back',
-                         'conductivity', 'dirt_correction', 'solar_diffusing')
-        optional_vals = (0.003, 0.85, 0.075, None, 0.9, 0.075, None, 0,
-                         0.84, 0.84, 0.9, 1.0, False)
-        for key, val in zip(optional_keys, optional_vals):
-            if key not in data:
-                data[key] = val
+        thick = data['thickness'] if 'thickness' in data and data['thickness'] \
+            is not None else 0.003
+        t_sol = data['solar_transmittance'] if 'solar_transmittance' in data and \
+            data['solar_transmittance'] is not None else 0.85
+        r_sol = data['solar_reflectance'] if 'solar_reflectance' in data and \
+            data['solar_reflectance'] is not None else 0.075
+        r_sol_b = data['solar_reflectance_back'] if 'solar_reflectance_back' \
+            in data and data['solar_reflectance_back'] != autocalculate.to_dict() \
+            else None
+        t_vis = data['visible_transmittance'] if 'visible_transmittance' in data and \
+            data['visible_transmittance'] is not None else 0.9
+        r_vis = data['visible_reflectance'] if 'visible_reflectance' in data and \
+            data['visible_reflectance'] is not None else 0.075
+        r_vis_b = data['visible_reflectance_back'] if 'visible_reflectance_back' \
+            in data and data['visible_reflectance_back'] != autocalculate.to_dict() \
+            else None
+        t_inf = data['infrared_transmittance'] if 'infrared_transmittance' in data and \
+            data['infrared_transmittance'] is not None else 0.0
+        emis = data['emissivity'] if 'emissivity' in data and \
+            data['emissivity'] is not None else 0.84
+        emis_b = data['emissivity_back'] if 'emissivity_back' in data and \
+            data['emissivity_back'] is not None else 0.84
+        cond = data['conductivity'] if 'conductivity' in data and \
+            data['conductivity'] is not None else 0.9
+        dirt = data['dirt_correction'] if 'dirt_correction' in data and \
+            data['dirt_correction'] is not None else 1.0
+        sol_diff = data['solar_diffusing'] if 'solar_diffusing' in data and \
+            data['solar_diffusing'] is not None else False
 
-        new_mat = cls(data['identifier'], data['thickness'],
-                      data['solar_transmittance'], data['solar_reflectance'],
-                      data['visible_transmittance'], data['visible_reflectance'],
-                      data['infrared_transmittance'], data['emissivity'],
-                      data['emissivity_back'], data['conductivity'])
-        new_mat.solar_reflectance_back = data['solar_reflectance_back']
-        new_mat.visible_reflectance_back = data['visible_reflectance_back']
-        new_mat.dirt_correction = data['dirt_correction']
-        new_mat.solar_diffusing = data['solar_diffusing']
+        new_mat = cls(data['identifier'], thick, t_sol, r_sol, t_vis, r_vis,
+                      t_inf, emis, emis_b, cond)
+        new_mat.solar_reflectance_back = r_sol_b
+        new_mat.visible_reflectance_back = r_vis_b
+        new_mat.dirt_correction = dirt
+        new_mat.solar_diffusing = sol_diff
         if 'display_name' in data and data['display_name'] is not None:
             new_mat.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_mat.user_data = data['user_data']
         return new_mat
 
     def to_idf(self):
@@ -418,8 +470,19 @@ class EnergyWindowMaterialGlazing(_EnergyWindowMaterialGlazingBase):
         }
         if self._display_name is not None:
             base['display_name'] = self.display_name
+        if self._user_data is not None:
+            base['user_data'] = self.user_data
         return base
 
+    @staticmethod
+    def _transmissivity_from_transmittance(transmittance):
+        """Get transmissivity from a transmittance value"""
+        try:
+            return (math.sqrt(0.8402528435 + 0.0072522239 * (transmittance ** 2)) -
+                    0.9166530661) / 0.0036261119 / transmittance
+        except ZeroDivisionError:
+            return 0
+
     def __key(self):
         """A tuple based on the object properties, useful for hashing."""
         return (self.identifier, self.thickness, self.solar_transmittance,
@@ -453,6 +516,8 @@ class EnergyWindowMaterialGlazing(_EnergyWindowMaterialGlazingBase):
         new_material._dirt_correction = self._dirt_correction
         new_material._solar_diffusing = self._solar_diffusing
         new_material._display_name = self._display_name
+        new_material._user_data = None if self._user_data is None \
+            else self._user_data.copy()
         return new_material
 
 
@@ -482,9 +547,18 @@ class EnergyWindowMaterialSimpleGlazSys(_EnergyWindowMaterialGlazingBase):
         * r_factor
         * u_value
         * r_value
+        * solar_transmittance
+        * solar_reflectance
+        * solar_reflectance_back
+        * solar_absorptance
+        * visible_transmittance
+        * visible_reflectance
+        * visible_reflectance_back
+        * visible_absorptance
+        * thickness
+        * user_data
     """
     __slots__ = ('_u_factor', '_shgc', '_vt')
-    _film_resistance = (1 / 23) + (1 / 7)  # interior + exterior films resistance
 
     def __init__(self, identifier, u_factor, shgc, vt=0.6):
         """Initialize energy window material simple glazing system."""
@@ -500,7 +574,7 @@ class EnergyWindowMaterialSimpleGlazSys(_EnergyWindowMaterialGlazingBase):
 
     @u_factor.setter
     def u_factor(self, u_fac):
-        # NOTE: u-values above 5.8 are not usually realistic but 12 is used as a hard limit
+        # NOTE: u-values above 5.8 are not realistic but 12 is the absolute hard limit
         self._u_factor = float_in_range(u_fac, 0.0, 12, 'glazing material u-factor')
 
     @property
@@ -532,13 +606,113 @@ class EnergyWindowMaterialSimpleGlazSys(_EnergyWindowMaterialGlazingBase):
 
     @property
     def u_value(self):
-        """U-value of the material layer [W/m2-K] (excluding air film resistance)."""
+        """U-value of the material layer [W/m2-K] (excluding air film resistance).
+
+        This is the U-value of glazing system material layer as understood by EnergyPlus.
+        """
         return 1 / self.r_value
 
     @property
     def r_value(self):
-        """R-value of the material layer [m2-K/W] (excluding air film resistance)."""
-        return (1 / self.u_factor) - self._film_resistance
+        """R-value of the material layer [m2-K/W] (excluding air film resistance).
+
+        This is the R-value of glazing system as understood by EnergyPlus.
+        """
+        out_r = 1 / ((0.025342 * self._u_factor) + 29.163853)
+        in_r = 1 / ((0.359073 * math.log(self._u_factor)) + 6.949915) \
+            if self._u_factor < 5.85 else \
+            1 / ((1.788041 * self._u_factor) - 2.886625)
+        return (1 / self.u_factor) - out_r - in_r
+
+    @property
+    def solar_transmittance(self):
+        """Get the solar transmittance of the glazing system at normal incidence."""
+        if self.u_factor > 3.4:
+            term_1 = (0.939998 * (self.shgc ** 2)) + (0.20332 * self.shgc) \
+                if self.shgc < 0.7206 else (1.30415 * self.shgc) - 0.30515
+        if self.u_factor < 4.5:
+            term_2 = (0.085775 * (self.shgc ** 2)) + (0.963954 * self.shgc) - 0.084958 \
+                if self.shgc > 0.15 else (0.4104 * self.shgc)
+        if self.u_factor >= 4.5:
+            return term_1
+        elif self.u_factor <= 3.4:
+            return term_2
+        else:
+            weight = (self.u_factor - 3.4) / 1.1
+            return (term_1 * weight) + (term_2 * (1 - weight))
+
+    @property
+    def solar_reflectance(self):
+        """Get the solar reflectance of the glazing system at normal incidence."""
+        # get values needed to compute the reflectance
+        t_sol, r_val = self.solar_transmittance, self.r_value
+        # determine the resistance of inside and outside air films
+        s_t = self.shgc - t_sol
+        if self.u_factor > 3.4:
+            term_1_is = 1 / ((29.436546 * (s_t ** 3)) - (21.943415 * (s_t ** 2)) +
+                             (9.945872 * s_t) + 7.426151)
+            term_1_os = 1 / ((2.225824 * s_t) + 20.57708)
+        if self.u_factor < 4.5:
+            term_2_is = 1 / ((199.8208128 * (s_t ** 3)) - (90.639733 * (s_t ** 2)) +
+                             (19.737055 * s_t) + 6.766575)
+            term_2_os = 1 / ((5.763355 * s_t) + 20.541528)
+        if self.u_factor >= 4.5:
+            r_is, r_os = term_1_is, term_1_os
+        elif self.u_factor <= 3.4:
+            r_is, r_os = term_2_is, term_2_os
+        else:
+            weight = (self.u_factor - 3.4) / 1.1
+            r_is = (term_1_is * weight) + (term_2_is * (1 - weight))
+            r_os = (term_1_os * weight) + (term_2_os * (1 - weight))
+        # compute the inward flowing fraction of solar heat and the reflectance
+        f_in = (r_os + (0.5 * r_val)) / (r_os + r_val + r_is)
+        return 1 - t_sol - ((self.shgc - t_sol) / f_in)
+
+    @property
+    def solar_reflectance_back(self):
+        """Get the solar reflectance of the glazing system at normal incidence."""
+        return self.solar_reflectance
+
+    @property
+    def solar_absorptance(self):
+        """Get the solar absorptance of the glazing system at normal incidence.
+
+        This is the solar absorptance as understood by EnergyPlus.
+        """
+        return 1 - self.solar_transmittance - self.solar_reflectance
+
+    @property
+    def visible_transmittance(self):
+        """Get the visible transmittance of the glazing system at normal incidence."""
+        return self.vt
+
+    @property
+    def visible_reflectance(self):
+        """Get the visible reflectance of the glazing system at normal incidence.
+
+        This is the visible reflectance as understood by EnergyPlus.
+        """
+        return (-0.0622 * (self.vt ** 3)) + (0.4277 * self.vt ** 2) - \
+            (0.4169 * self.vt) + 0.2399
+
+    @property
+    def visible_reflectance_back(self):
+        """Get the visible reflectance of the glazing system at normal incidence."""
+        return self.visible_reflectance
+
+    @property
+    def visible_absorptance(self):
+        """Get the visible absorptance of the glazing system at normal incidence.
+
+        This is the visible absorptance as understood by EnergyPlus.
+        """
+        return 1 - self.vt - self.visible_reflectance
+
+    @property
+    def thickness(self):
+        """Get the thickess of the glazing system as interpreted by EnergyPlus [m]."""
+        u_val = self.u_value
+        return 0.002 if u_val > 7 else (0.05914 - (0.00714 * u_val))
 
     @classmethod
     def from_idf(cls, idf_string):
@@ -548,6 +722,11 @@ class EnergyWindowMaterialSimpleGlazSys(_EnergyWindowMaterialGlazingBase):
             idf_string: A text string fully describing an EnergyPlus material.
         """
         ep_strs = parse_idf_string(idf_string, 'WindowMaterial:SimpleGlazingSystem,')
+        if len(ep_strs) == 4 and ep_strs[3] == '':
+            ep_strs.pop(3)
+            glz_mat = cls(*ep_strs)
+            glz_mat.vt = glz_mat.solar_transmittance
+            return glz_mat
         return cls(*ep_strs)
 
     @classmethod
@@ -570,11 +749,12 @@ class EnergyWindowMaterialSimpleGlazSys(_EnergyWindowMaterialGlazingBase):
         """
         assert data['type'] == 'EnergyWindowMaterialSimpleGlazSys', \
             'Expected EnergyWindowMaterialSimpleGlazSys. Got {}.'.format(data['type'])
-        if 'vt' not in data:
-            data['vt'] = 0.6
-        new_obj = cls(data['identifier'], data['u_factor'], data['shgc'], data['vt'])
+        vt = 0.6 if 'vt' not in data else data['vt']
+        new_obj = cls(data['identifier'], data['u_factor'], data['shgc'], vt)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
         return new_obj
 
     def to_idf(self):
@@ -595,6 +775,8 @@ class EnergyWindowMaterialSimpleGlazSys(_EnergyWindowMaterialGlazingBase):
         }
         if self._display_name is not None:
             base['display_name'] = self.display_name
+        if self._user_data is not None:
+            base['user_data'] = self.user_data
         return base
 
     def __key(self):
@@ -618,4 +800,6 @@ class EnergyWindowMaterialSimpleGlazSys(_EnergyWindowMaterialGlazingBase):
         new_material = EnergyWindowMaterialSimpleGlazSys(
             self.identifier, self.u_factor, self.shgc, self.vt)
         new_material._display_name = self._display_name
+        new_material._user_data = None if self._user_data is None \
+            else self._user_data.copy()
         return new_material
