@@ -238,7 +238,7 @@ class _ColorObject(object):
         return self.__repr__()
 
     def __repr__(self):
-        return 'Color Object'
+        return 'Color Object:'
 
 
 class ColorRoom(_ColorObject):
@@ -272,6 +272,10 @@ class ColorRoom(_ColorObject):
             This will be used to ensure the legend units display correctly when
             data is floor-normalized. Examples include 'm', 'mm', 'ft'.
             (Default: 'm' for meters).
+        space_based: Boolean to note whether the result is reported on the EnergyPlus
+            Space level instead of the Zone level. In this case, the matching to
+            the Room will account for the fact that the Space name is the Room
+            name with _Space added to it. (Default: False).
 
     Properties:
         * data_collections
@@ -280,7 +284,9 @@ class ColorRoom(_ColorObject):
         * simulation_step
         * normalize_by_floor
         * geo_unit
+        * space_based
         * matched_rooms
+        * matched_data
         * matched_values
         * matched_floor_faces
         * matched_floor_areas
@@ -293,10 +299,11 @@ class ColorRoom(_ColorObject):
         * min_point
         * max_point
     """
-    __slots__ = ('_rooms',)
+    __slots__ = ('_rooms', '_space_based')
 
     def __init__(self, data_collections, rooms, legend_parameters=None,
-                 simulation_step=None, normalize_by_floor=True, geo_unit='m'):
+                 simulation_step=None, normalize_by_floor=True, geo_unit='m',
+                 space_based=False):
         """Initialize ColorRoom."""
         # initialize the base object
         _ColorObject.__init__(self, data_collections, legend_parameters,
@@ -317,7 +324,9 @@ class ColorRoom(_ColorObject):
         self._calculate_min_max(self._rooms)
 
         # match the rooms with the data collections
-        self._matched_objects = match_rooms_to_data(data_collections, rooms)
+        self._space_based = bool(space_based)
+        self._matched_objects = match_rooms_to_data(
+            data_collections, rooms, space_based=self._space_based)
         if len(self._matched_objects) == 0:
             raise ValueError('None of the ColorRoom data collections could be '
                              'matched to the input rooms')
@@ -340,11 +349,21 @@ class ColorRoom(_ColorObject):
     def normalize_by_floor(self, value):
         self._normalize = bool(value)
 
+    @property
+    def space_based(self):
+        """Get a boolean for whether results are set to be space-based."""
+        return self._space_based
+
     @property
     def matched_rooms(self):
         """Get a tuple of honeybee Rooms that have been matched to the data."""
         return tuple(obj[0] for obj in self._matched_objects)
 
+    @property
+    def matched_data(self):
+        """Get a tuple of data collections that have been matched to the rooms."""
+        return tuple(obj[1] for obj in self._matched_objects)
+
     @property
     def matched_values(self):
         """Get an array of numbers that correspond to the matched_rooms.
@@ -373,14 +392,15 @@ class ColorRoom(_ColorObject):
                     return tuple(obj[1].average for obj in self._matched_objects)
             else:  # normalize the data by floor area
                 vals = []
+                zip_obj = zip(self._matched_objects, self.matched_floor_areas)
                 if self._base_type.cumulative:  # divide total values by floor area
-                    for obj, f_area in zip(self._matched_objects, self.matched_floor_areas):
+                    for obj, f_area in zip_obj:
                         try:
                             vals.append(obj[1].total / (f_area * obj[2]))
                         except ZeroDivisionError:  # no floor faces in the Room
                             vals.append(0)
                 else:  # divide average values by floor area
-                    for obj, f_area in zip(self._matched_objects, self.matched_floor_areas):
+                    for obj, f_area in zip_obj:
                         try:
                             vals.append(obj[1].average / f_area)
                         except ZeroDivisionError:  # no floor faces in the Room
@@ -422,12 +442,12 @@ class ColorRoom(_ColorObject):
         upper_title_location, etc.
         """
         return GraphicContainer(
-            self.matched_values, self.min_point, self._max_point,
+            self.matched_values, self.min_point, self.max_point,
             self.legend_parameters, self.data_type, str(self.unit))
 
     def __repr__(self):
         """Color Room representation."""
-        return 'Color Room:\n{} Rooms\n{}'.format(
+        return 'Color Room: [{} Rooms] [{}]'.format(
             len(self._matched_objects), self._base_collection.header)
 
 
@@ -469,6 +489,7 @@ class ColorFace(_ColorObject):
         * normalize
         * geo_unit
         * matched_flat_faces
+        * matched_data
         * matched_values
         * matched_flat_geometry
         * matched_flat_areas
@@ -531,6 +552,11 @@ class ColorFace(_ColorObject):
         """Get a tuple of honeybee objects that have been matched to the data."""
         return tuple(obj[0] for obj in self._matched_objects)
 
+    @property
+    def matched_data(self):
+        """Get a tuple of data collections that have been matched to the flat_faces."""
+        return tuple(obj[1] for obj in self._matched_objects)
+
     @property
     def matched_values(self):
         """Get an array of numbers that correspond to the matched_flat_faces.
@@ -556,11 +582,12 @@ class ColorFace(_ColorObject):
                     return tuple(obj[1].average for obj in self._matched_objects)
             else:  # normalize the data by face area
                 vals = []
+                zip_obj = zip(self._matched_objects, self.matched_flat_areas)
                 if self._base_type.cumulative:  # divide total values by face area
-                    for obj, f_area in zip(self._matched_objects, self.matched_flat_areas):
+                    for obj, f_area in zip_obj:
                         vals.append(obj[1].total / f_area)
                 else:  # divide average values by face area
-                    for obj, f_area in zip(self._matched_objects, self.matched_flat_areas):
+                    for obj, f_area in zip_obj:
                         vals.append(obj[1].average / f_area)
                 return vals
 
@@ -599,10 +626,10 @@ class ColorFace(_ColorObject):
         upper_title_location, etc.
         """
         return GraphicContainer(
-            self.matched_values, self.min_point, self._max_point,
+            self.matched_values, self.min_point, self.max_point,
             self.legend_parameters, self.data_type, str(self.unit))
 
     def __repr__(self):
         """Color Face representation."""
-        return 'Color Face:\n{} Objects\n{}'.format(
+        return 'Color Face: [{} Objects] [{}]'.format(
             len(self._matched_objects), self._base_collection.header)
