@@ -2,21 +2,6 @@
 """Annual schedule defined by a list of values at a fixed interval or timestep."""
 from __future__ import division
 
-from .typelimit import ScheduleTypeLimit
-from ..reader import parse_idf_string
-from ..writer import generate_idf_string
-
-from honeybee._lockable import lockable
-from honeybee.typing import valid_ep_string, float_in_range, int_in_range, \
-    tuple_with_length
-
-from ladybug.datacollection import HourlyContinuousCollection
-from ladybug.header import Header
-from ladybug.analysisperiod import AnalysisPeriod
-from ladybug.dt import Date, DateTime
-from ladybug.datatype.generic import GenericType
-from ladybug.futil import write_to_file, csv_to_matrix
-
 import os
 import re
 try:
@@ -28,6 +13,22 @@ try:
 except ImportError:
     xrange = range  # python 3
 
+from ladybug.datacollection import HourlyContinuousCollection
+from ladybug.header import Header
+from ladybug.analysisperiod import AnalysisPeriod
+from ladybug.dt import Date, DateTime
+from ladybug.datatype.generic import GenericType
+from ladybug.futil import write_to_file, csv_to_matrix
+
+from honeybee._lockable import lockable
+from honeybee.typing import valid_ep_string, float_in_range, int_in_range, \
+    tuple_with_length
+
+from .typelimit import ScheduleTypeLimit
+from ..reader import parse_idf_string, clean_idf_file_contents
+from ..writer import generate_idf_string
+from ..properties.extension import ScheduleFixedIntervalProperties
+
 
 @lockable
 class ScheduleFixedInterval(object):
@@ -37,7 +38,7 @@ class ScheduleFixedInterval(object):
         identifier: Text string for a unique Schedule ID. Must be < 100 characters
             and not contain any EnergyPlus special characters. This will be used to
             identify the object across a model and in the exported IDF.
-        values: A list of values occuring at a fixed interval over the simulation.
+        values: A list of values occurring at a fixed interval over the simulation.
             Typically, this should be a list of 8760 values for each hour of the
             year but it can be a shorter list if you don't plan on using it in
             an annual simulation. In this case, the start_date should probably be
@@ -77,11 +78,13 @@ class ScheduleFixedInterval(object):
         * interpolate
         * end_date_time
         * is_leap_year
+        * is_constant
         * data_collection
+        * user_data
     """
     __slots__ = ('_identifier', '_display_name', '_values', '_schedule_type_limit',
                  '_start_date', '_placeholder_value', '_timestep', '_interpolate',
-                 '_locked')
+                 '_locked', '_properties', '_user_data')
     _schedule_file_comments = \
         ('schedule name', 'schedule type limits', 'file name', 'column number',
          'rows to skip', 'number of hours of data', 'column separator',
@@ -111,6 +114,10 @@ class ScheduleFixedInterval(object):
         self.placeholder_value = placeholder_value
         self.interpolate = interpolate
 
+        # initialize properties for extensions and user data
+        self._properties = ScheduleFixedIntervalProperties(self)
+        self._user_data = None
+
     @property
     def identifier(self):
         """Get or set the text string for unique schedule identifier."""
@@ -133,10 +140,12 @@ class ScheduleFixedInterval(object):
 
     @display_name.setter
     def display_name(self, value):
-        try:
-            self._display_name = str(value)
-        except UnicodeEncodeError:  # Python 2 machine lacking the character set
-            self._display_name = value  # keep it as unicode
+        if value is not None:
+            try:
+                value = str(value)
+            except UnicodeEncodeError:  # Python 2 machine lacking the character set
+                pass  # keep it as unicode
+        self._display_name = value
 
     @property
     def values(self):
@@ -214,6 +223,12 @@ class ScheduleFixedInterval(object):
         """
         return self._start_date.leap_year
 
+    @property
+    def is_constant(self):
+        """Boolean noting whether the schedule is representable with a single value."""
+        val_1 = self._values[0]
+        return all(element == val_1 for element in self._values)
+
     @property
     def data_collection(self):
         """DataCollection of schedule values at this schedule's start_date and timestep.
@@ -223,9 +238,32 @@ class ScheduleFixedInterval(object):
                                   end_dt.month, end_dt.day, end_dt.hour, self.timestep,
                                   self.is_leap_year)
         data_type, unit = self._get_lb_data_type_and_unit()
-        header = Header(data_type, unit, a_period, metadata={'schedule': self.identifier})
+        header = Header(
+            data_type, unit, a_period, metadata={'schedule': self.identifier})
         return HourlyContinuousCollection(header, self._values)
 
+    @property
+    def user_data(self):
+        """Get or set an optional dictionary for additional meta data for this object.
+
+        This will be None until it has been set. All keys and values of this
+        dictionary should be of a standard Python type to ensure correct
+        serialization of the object to/from JSON (eg. str, float, int, list, dict)
+        """
+        return self._user_data
+
+    @user_data.setter
+    def user_data(self, value):
+        if value is not None:
+            assert isinstance(value, dict), 'Expected dictionary for honeybee_energy' \
+                'object user_data. Got {}.'.format(type(value))
+        self._user_data = value
+
+    @property
+    def properties(self):
+        """Get properties for extensions."""
+        return self._properties
+
     def values_at_timestep(
             self, timestep=1, start_date=None, end_date=None):
         """Get a list of sequential schedule values over the year at a given timestep.
@@ -279,7 +317,7 @@ class ScheduleFixedInterval(object):
             vals_at_step = []
             ind = 0
             step_ratio = self.timestep / timestep
-            for i in xrange(int(len(self._values) / step_ratio)):
+            for _ in xrange(int(len(self._values) / step_ratio)):
                 vals_at_step.append(self._values[int(ind)])
                 ind += step_ratio
         else:
@@ -295,7 +333,7 @@ class ScheduleFixedInterval(object):
             else:
                 n_step = int(timestep / self.timestep)
                 for val in self._values:
-                    for stp in xrange(n_step):
+                    for _ in xrange(n_step):
                         vals_at_step.append(val)
 
         # build up the full list of values accounting for start and end dates
@@ -350,10 +388,43 @@ class ScheduleFixedInterval(object):
                                   end_date.month, end_date.day, 23, timestep,
                                   self.is_leap_year)
         data_type, unit = self._get_lb_data_type_and_unit()
-        header = Header(data_type, unit, a_period, metadata={'schedule': self.identifier})
+        header = Header(
+            data_type, unit, a_period, metadata={'schedule': self.identifier})
         values = self.values_at_timestep(timestep, start_date, end_date)
         return HourlyContinuousCollection(header, values)
 
+    def shift_by_step(self, step_count=1, timestep=1):
+        """Get a version of this object where the values are shifted.
+
+        This is useful when attempting to derive a set of diversified schedules
+        from a single average schedule.
+
+        Args:
+            step_count: An integer for the number of timesteps at which the schedule
+                will be shifted. Positive values indicate a shift of values forward
+                in time while negative values indicate a shift backwards in
+                time. (Default: 1).
+            timestep: An integer for the number of timesteps per hour at which the
+                shifting is occurring. This must be a value between 1 and 60, which
+                is evenly divisible by 60. 1 indicates that each step is an hour
+                while 60 indicates that each step is a minute. (Default: 1).
+        """
+        # figure out the number of values to be moved
+        val_count = step_count * int(self.timestep / timestep)
+        # shift by the number of values
+        if val_count >= 0:
+            new_values = self.values[-val_count:] + self.values[:-val_count]
+        else:
+            new_values = list(self.values)
+            for _ in range(abs(val_count)):
+                new_values.append(new_values.pop(0))
+        # return the shifted schedule
+        new_id = '{}_Shift_{}mins'.format(
+            self.identifier, int((60 / timestep) * step_count))
+        return ScheduleFixedInterval(
+            new_id, new_values, self.schedule_type_limit, self.timestep,
+            self.start_date, self.placeholder_value, self.interpolate)
+
     @classmethod
     def from_idf(cls, idf_string, type_idf_string=None):
         """Create a ScheduleFixedInterval from an EnergyPlus IDF text strings.
@@ -430,6 +501,10 @@ class ScheduleFixedInterval(object):
                       start_date, placeholder_value, interpolate)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
         return new_obj
 
     @classmethod
@@ -469,6 +544,10 @@ class ScheduleFixedInterval(object):
             typ_lim is not None else None
         if 'display_name' in data and data['display_name'] is not None:
             schedule.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            schedule.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            schedule.properties._load_extension_attr_from_dict(data['properties'])
         return schedule
 
     def to_idf(self, schedule_directory, include_datetimes=False):
@@ -582,6 +661,11 @@ class ScheduleFixedInterval(object):
                 base['schedule_type_limit'] = self._schedule_type_limit.identifier
         if self._display_name is not None:
             base['display_name'] = self.display_name
+        if self._user_data is not None:
+            base['user_data'] = self.user_data
+        prop_dict = self.properties.to_dict()
+        if prop_dict is not None:
+            base['properties'] = prop_dict
         return base
 
     def duplicate(self):
@@ -667,11 +751,8 @@ class ScheduleFixedInterval(object):
             A list of all Schedule:File objects in the IDF file as honeybee_energy
             ScheduleFixedInterval objects.
         """
-        # check the file
-        assert os.path.isfile(idf_file), 'Cannot find an idf file at {}'.format(idf_file)
-        with open(idf_file, 'r') as ep_file:
-            file_contents = ep_file.read()
-
+        # read the file and remove lines of comments
+        file_contents = clean_idf_file_contents(idf_file)
         # extract all of the ScheduleTypeLimit objects
         type_pattern = re.compile(r"(?i)(ScheduleTypeLimits,[\s\S]*?;)")
         sch_type_str = type_pattern.findall(file_contents)
@@ -750,8 +831,10 @@ class ScheduleFixedInterval(object):
                     for values in zip(*all_values)]
 
         # return the final schedule
-        return ScheduleFixedInterval(identifier, sch_vals, schedules[0].schedule_type_limit,
-                                     timestep, start_date=Date(1, 1, lp_yr))
+        return ScheduleFixedInterval(
+            identifier, sch_vals, schedules[0].schedule_type_limit,
+            timestep, start_date=Date(1, 1, lp_yr)
+        )
 
     def _check_values(self, values):
         """Check values whenever they come through the values setter."""
@@ -826,6 +909,8 @@ class ScheduleFixedInterval(object):
             self.identifier, self._values, self._schedule_type_limit, self._timestep,
             self._start_date, self._placeholder_value, self._interpolate)
         new_obj._display_name = self._display_name
+        new_obj._user_data = None if self._user_data is None else self._user_data.copy()
+        new_obj._properties._duplicate_extension_attr(self._properties)
         return new_obj
 
     def ToString(self):
@@ -833,6 +918,6 @@ class ScheduleFixedInterval(object):
         return self.__repr__()
 
     def __repr__(self):
-        return 'ScheduleFixedInterval:\n name: {}\n period: {} - {}\n timestep: ' \
-            '{}'.format(self.identifier, self.start_date, self.end_date_time.strftime('%d %b'),
-                        self.timestep)
+        return 'ScheduleFixedInterval: {} [{} - {}] [timestep: {}]'.format(
+            self.display_name, self.start_date,
+            self.end_date_time.strftime('%d %b'), self.timestep)
