@@ -2,23 +2,22 @@
 """Complete annual schedule object built from ScheduleDay and rules for applying them."""
 from __future__ import division
 
+import re
+
+from honeybee._lockable import lockable
+from honeybee.typing import tuple_with_length, valid_ep_string
+from ladybug.analysisperiod import AnalysisPeriod
+from ladybug.datacollection import HourlyContinuousCollection
+from ladybug.datatype.generic import GenericType
+from ladybug.dt import Date, Time
+from ladybug.header import Header
+
+from ..reader import parse_idf_string, clean_idf_file_contents
+from ..writer import generate_idf_string
+from ..properties.extension import ScheduleRulesetProperties
 from .day import ScheduleDay
 from .rule import ScheduleRule
 from .typelimit import ScheduleTypeLimit
-from ..reader import parse_idf_string
-from ..writer import generate_idf_string
-
-from honeybee._lockable import lockable
-from honeybee.typing import valid_ep_string, tuple_with_length
-
-from ladybug.datacollection import HourlyContinuousCollection
-from ladybug.header import Header
-from ladybug.analysisperiod import AnalysisPeriod
-from ladybug.dt import Date
-from ladybug.datatype.generic import GenericType
-
-import re
-import os
 
 
 @lockable
@@ -53,12 +52,15 @@ class ScheduleRuleset(object):
         * winter_designday_schedule
         * holiday_schedule
         * day_schedules
+        * typical_day_schedules
         * is_constant
         * is_single_week
+        * user_data
     """
     __slots__ = ('_identifier', '_display_name', '_default_day_schedule',
                  '_schedule_rules', '_holiday_schedule', '_summer_designday_schedule',
-                 '_winter_designday_schedule', '_schedule_type_limit', '_locked')
+                 '_winter_designday_schedule', '_schedule_type_limit',
+                 '_locked', '_properties', '_user_data')
     _dow_text_to_int = {'sunday': 1, 'monday': 2, 'tuesday': 3, 'wednesday': 4,
                         'thursday': 2, 'friday': 3, 'saturday': 7}
     _schedule_week_comments = (
@@ -80,6 +82,10 @@ class ScheduleRuleset(object):
         self.summer_designday_schedule = summer_designday_schedule
         self.winter_designday_schedule = winter_designday_schedule
 
+        # initialize properties for extensions and user data
+        self._properties = ScheduleRulesetProperties(self)
+        self._user_data = None
+
     @property
     def identifier(self):
         """Get or set the text string for schedule unique identifier."""
@@ -101,10 +107,12 @@ class ScheduleRuleset(object):
 
     @display_name.setter
     def display_name(self, value):
-        try:
-            self._display_name = str(value)
-        except UnicodeEncodeError:  # Python 2 machine lacking the character set
-            self._display_name = value  # keep it as unicode
+        if value is not None:
+            try:
+                value = str(value)
+            except UnicodeEncodeError:  # Python 2 machine lacking the character set
+                pass  # keep it as unicode
+        self._display_name = value
 
     @property
     def default_day_schedule(self):
@@ -221,6 +229,19 @@ class ScheduleRuleset(object):
                 day_scheds.append(rule._schedule_day)
         return day_scheds
 
+    @property
+    def typical_day_schedules(self):
+        """Get a list of all unique ScheduleDay objects used over the annual run period.
+
+        This excludes the schedules of the design days and the holiday schedule,
+        which gives a sense of the typical schedule over the year.
+        """
+        day_scheds = [self.default_day_schedule]
+        for rule in self.schedule_rules:
+            if not self._instance_in_array(rule._schedule_day, day_scheds):
+                day_scheds.append(rule._schedule_day)
+        return day_scheds
+
     @property
     def is_constant(self):
         """Boolean noting whether the schedule is representable with a single value."""
@@ -239,6 +260,28 @@ class ScheduleRuleset(object):
             return True
         return False
 
+    @property
+    def user_data(self):
+        """Get or set an optional dictionary for additional meta data for this object.
+
+        This will be None until it has been set. All keys and values of this
+        dictionary should be of a standard Python type to ensure correct
+        serialization of the object to/from JSON (eg. str, float, int, list, dict)
+        """
+        return self._user_data
+
+    @user_data.setter
+    def user_data(self, value):
+        if value is not None:
+            assert isinstance(value, dict), 'Expected dictionary for honeybee_energy' \
+                'object user_data. Got {}.'.format(type(value))
+        self._user_data = value
+
+    @property
+    def properties(self):
+        """Get properties for extensions."""
+        return self._properties
+
     def add_rule(self, rule):
         """Add a ScheduleRule to this ScheduleRuleset.
 
@@ -365,11 +408,52 @@ class ScheduleRuleset(object):
         else:
             unit = 'unknown'
             data_type = GenericType('Unknown Data Type', unit)
-        header = Header(data_type, unit, a_period, metadata={'schedule': self.identifier})
+        header = Header(data_type, unit, a_period,
+                        metadata={'schedule': self.identifier})
         values = self.values(timestep, start_date, end_date, start_dow,
                              holidays, leap_year)
         return HourlyContinuousCollection(header, values)
 
+    def shift_by_step(self, step_count=1, timestep=1):
+        """Get a version of this object where the day_schedule values are shifted.
+
+        This is useful when attempting to derive a set of diversified schedules
+        from a single average schedule.
+
+        Args:
+            step_count: An integer for the number of timesteps at which the schedule
+                will be shifted. Positive values indicate a shift of values forward
+                in time while negative values indicate a shift backwards in
+                time. (Default: 1).
+            timestep: An integer for the number of timesteps per hour at which the
+                shifting is occurring. This must be a value between 1 and 60, which
+                is evenly divisible by 60. 1 indicates that each step is an hour
+                while 60 indicates that each step is a minute. (Default: 1).
+        """
+        # shift all of the day schedules according to the inputs
+        day_scheds = self.day_schedules
+        shift_dict = {sch.identifier: sch.shift_by_step(step_count, timestep)
+                      for sch in day_scheds}
+        # figure out where each of the shifted schedules belong
+        new_default = shift_dict[self.default_day_schedule.identifier]
+        new_summer = shift_dict[self._summer_designday_schedule.identifier] \
+            if self._summer_designday_schedule is not None else None
+        new_winter = shift_dict[self._winter_designday_schedule.identifier] \
+            if self._winter_designday_schedule is not None else None
+        new_holiday = shift_dict[self._holiday_schedule.identifier] \
+            if self._holiday_schedule is not None else None
+        new_rules = []
+        for rule in self.schedule_rules:
+            new_rule = rule.duplicate()
+            new_rule.schedule_day = shift_dict[rule.schedule_day.identifier]
+            new_rules.append(new_rule)
+        # return the shifted schedule
+        new_id = '{}_Shift_{}mins'.format(
+            self.identifier, int((60 / timestep) * step_count))
+        return ScheduleRuleset(
+            new_id, new_default, new_rules, self.schedule_type_limit, new_holiday,
+            new_summer, new_winter)
+
     @classmethod
     def from_constant_value(cls, identifier, value, schedule_type_limit=None):
         """Create a ScheduleRuleset fromm a single constant value.
@@ -436,13 +520,17 @@ class ScheduleRuleset(object):
                 following: (1, 2, 3, 4, 5, 6, 10, 12, 15, 20, 30, 60).
             schedule_type_limit: A ScheduleTypeLimit object that will be used to
                 validate schedule values against upper/lower limits and assign
-                units to the schedule values.
+                units to the schedule values. (Default: None).
             summer_designday_values: A list of [24 * timestep] numerical values for
                 the summer design day. If None, the daily schedule with the highest
-                average value will be used.
+                average value will be used unless the schedule_type_limit has a
+                Temperature unit_type, in which case it will be the daily schedule
+                with the lowest average value. (Default: None).
             winter_designday_values: A list of [24 * timestep] numerical values for
                 the winter design day. If None, the daily schedule with the lowest
-                average value will be used.
+                average value will be used unless the schedule_type_limit has a
+                Temperature unit_type, in which case it will be the daily schedule
+                with the highest average value. (Default: None).
         """
         # process the rules for the days of the week
         schedule_rules = []
@@ -471,15 +559,19 @@ class ScheduleRuleset(object):
 
         # get ScheduleDay for summer and winter design days
         avg_day_vals = [sum(vals) / len(vals) for vals in applied_day_values]
+        temp_type = True if schedule_type_limit is not None and \
+            schedule_type_limit.unit_type == 'Temperature' else False
         if summer_designday_values is None:
-            sch_i = avg_day_vals.index(max(avg_day_vals))
+            sch_i = avg_day_vals.index(min(avg_day_vals)) if temp_type \
+                else avg_day_vals.index(max(avg_day_vals))
             summer = schedule_rules[sch_i]._schedule_day.duplicate()
             summer.identifier = '{}_SmrDsn'.format(summer.identifier)
         else:
             summer = ScheduleDay.from_values_at_timestep(
                 '{}_SmrDsn'.format(identifier), summer_designday_values, timestep)
         if winter_designday_values is None:
-            sch_i = avg_day_vals.index(min(avg_day_vals))
+            sch_i = avg_day_vals.index(max(avg_day_vals)) if temp_type \
+                else avg_day_vals.index(min(avg_day_vals))
             winter = schedule_rules[sch_i]._schedule_day.duplicate()
             winter.identifier = '{}_WntrDsn'.format(summer.identifier)
         else:
@@ -551,7 +643,7 @@ class ScheduleRuleset(object):
 
     @classmethod
     def from_idf(cls, year_idf_string, week_idf_strings, day_idf_strings,
-                 type_idf_string=None):
+                 type_idf_string=None): 
         """Create a ScheduleRuleset from an EnergyPlus IDF text strings.
 
         Args:
@@ -585,9 +677,26 @@ class ScheduleRuleset(object):
         default_day_schedule = all_rules[0].schedule_day
         holiday_sch, summer_dd_sch, winter_dd_sch = week_dd_dict[year_sch[2]]
         sched = cls(year_sch[0], default_day_schedule, all_rules[1:], schedule_type)
-        cls._apply_designdays_with_check(sched, holiday_sch, summer_dd_sch, winter_dd_sch)
+        cls._apply_designdays_with_check(
+            sched, holiday_sch, summer_dd_sch, winter_dd_sch)
         return sched
 
+    @classmethod
+    def from_idf_constant(cls, idf_string, type_idf_string=None):
+        """Create a ScheduleRuleset from an EnergyPlus Schedule:Constant string.
+
+        Args:
+            idf_string: A text string fully describing an EnergyPlus Schedule:Constant.
+            type_idf_string: An optional text string for the ScheduleTypeLimits.
+                If None, the resulting schedule will have no ScheduleTypeLimit.
+        """
+        const_sch = parse_idf_string(idf_string)
+        sched_val = float(const_sch[2]) if const_sch[2] != '' else 0
+        schedule_type = ScheduleTypeLimit.from_idf(type_idf_string) if type_idf_string \
+            is not None else None
+        return ScheduleRuleset.from_constant_value(
+            const_sch[0], sched_val, schedule_type)
+
     @classmethod
     def from_dict(cls, data):
         """Create a ScheduleRuleset from a dictionary.
@@ -647,6 +756,10 @@ class ScheduleRuleset(object):
                       holiday_sched, summer_sched, winter_sched)
         if 'display_name' in data and data['display_name'] is not None:
             new_obj.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            new_obj.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            new_obj.properties._load_extension_attr_from_dict(data['properties'])
         return new_obj
 
     @classmethod
@@ -687,6 +800,10 @@ class ScheduleRuleset(object):
             typ_lim is not None else None
         if 'display_name' in data and data['display_name'] is not None:
             schedule.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            schedule.user_data = data['user_data']
+        if 'properties' in data and data['properties'] is not None:
+            schedule.properties._load_extension_attr_from_dict(data['properties'])
         return schedule
 
     def to_rules(self, start_date, end_date):
@@ -706,20 +823,32 @@ class ScheduleRuleset(object):
         ScheduleRule._check_date(end_date)
         st_doy = ScheduleRule._doy_non_leap_year(start_date)
         end_doy = ScheduleRule._doy_non_leap_year(end_date)
+        assert start_date <= end_date, \
+            'Start date must be before end date for ScheduleRuleset.to_rules().'
 
         # assemble all of the rules already applied to this ScheduleRuleset
         rules = []
         for rule in self._schedule_rules:
-            if (rule._start_doy < st_doy and rule._end_doy < st_doy) or \
-                    (rule._start_doy > st_doy and rule._end_doy > end_doy):
-                pass  # no overlap with input period
+            if not rule.is_reversed:
+                if (rule._start_doy < st_doy and rule._end_doy < st_doy) or \
+                        (rule._start_doy > st_doy and rule._end_doy > end_doy):
+                    pass  # no overlap with input period
+                else:
+                    new_rule = rule.duplicate()
+                    if rule._start_doy < st_doy:
+                        new_rule.start_date = start_date
+                    if rule._end_doy > end_doy:
+                        new_rule.end_date = end_date
+                    rules.append(new_rule)
             else:
-                new_rule = rule.duplicate()
-                if rule._start_doy < st_doy:
-                    new_rule.start_date = start_date
-                if rule._end_doy > end_doy:
-                    new_rule.end_date = end_date
-                rules.append(new_rule)
+                if rule._start_doy <= end_doy:
+                    new_rule1 = rule.duplicate()
+                    new_rule1.end_date = end_doy
+                    rules.append(new_rule1)
+                if rule._end_doy >= st_doy:
+                    new_rule2 = rule.duplicate()
+                    new_rule2.start_date = st_doy
+                    rules.append(new_rule2)
 
         # add the default_day_schedule for all days not covered by rules
         default_rule = ScheduleRule(self.default_day_schedule.duplicate(),
@@ -790,7 +919,7 @@ class ScheduleRuleset(object):
             rules_each_day = []
             for doy in range(1, 366):
                 rules_on_doy = tuple(i for i, rule in enumerate(self._schedule_rules)
-                                     if rule._start_doy <= doy <= rule._end_doy)
+                                     if rule.does_rule_apply_doy(doy))
                 rules_each_day.append(rules_on_doy)
             unique_rule_sets = set(rules_each_day)
             # check if any combination yield the same week schedule and remove duplicates
@@ -856,13 +985,16 @@ class ScheduleRuleset(object):
 
         # optional properties
         if len(self._schedule_rules) != 0:
-            base['schedule_rules'] = [rule.to_dict(True) for rule in self._schedule_rules]
+            base['schedule_rules'] = \
+                [rule.to_dict(True) for rule in self._schedule_rules]
         if self._holiday_schedule is not None:
             base['holiday_schedule'] = self._holiday_schedule.identifier
         if self._summer_designday_schedule is not None:
-            base['summer_designday_schedule'] = self._summer_designday_schedule.identifier
+            base['summer_designday_schedule'] = \
+                self._summer_designday_schedule.identifier
         if self._winter_designday_schedule is not None:
-            base['winter_designday_schedule'] = self._winter_designday_schedule.identifier
+            base['winter_designday_schedule'] = \
+                self._winter_designday_schedule.identifier
 
         # optional properties that can be abridged
         if self._schedule_type_limit is not None:
@@ -872,6 +1004,11 @@ class ScheduleRuleset(object):
                 base['schedule_type_limit'] = self._schedule_type_limit.identifier
         if self._display_name is not None:
             base['display_name'] = self.display_name
+        if self._user_data is not None:
+            base['user_data'] = self.user_data
+        prop_dict = self.properties.to_dict()
+        if prop_dict is not None:
+            base['properties'] = prop_dict
         return base
 
     def duplicate(self):
@@ -905,23 +1042,24 @@ class ScheduleRuleset(object):
             rule.unlock()
 
     @staticmethod
-    def extract_all_from_idf_file(idf_file):
+    def extract_all_from_idf_file(idf_file, import_compact=False):
         """Extract all ScheduleRuleset objects from an EnergyPlus IDF file.
 
         Args:
             idf_file: A path to an IDF file containing objects for Schedule:Year and
                 corresponding Schedule:Week and Schedule:Day objects. The Schedule:Year
                 will be used to assemble all of these into a ScheduleRuleset.
+            import_compact: Boolean to note whether to parse Schedule:Compact (True)
+                or not (False).
+                Default: False.
 
         Returns:
             schedules --
             A list of all Schedule:Year objects in the IDF file as honeybee_energy
             ScheduleRuleset objects.
         """
-        # check the file
-        assert os.path.isfile(idf_file), 'Cannot find an idf file at {}'.format(idf_file)
-        with open(idf_file, 'r') as ep_file:
-            file_contents = ep_file.read()
+        # read the file and remove lines of comments
+        file_contents = clean_idf_file_contents(idf_file)
         # extract all of the ScheduleDay objects
         day_pattern1 = re.compile(r"(?i)(Schedule:Day:Interval,[\s\S]*?;)")
         day_pattern2 = re.compile(r"(?i)(Schedule:Day:Hourly,[\s\S]*?;)")
@@ -932,7 +1070,8 @@ class ScheduleRuleset(object):
         # extract all of the Schedule:Week objects
         week_pattern_1 = re.compile(r"(?i)(Schedule:Week:Daily,[\s\S]*?;)")
         week_pattern_2 = re.compile(r"(?i)(Schedule:Week:Compact,[\s\S]*?;)")
-        week_sch_str = week_pattern_1.findall(file_contents) + week_pattern_2.findall(file_contents)
+        week_sch_str = week_pattern_1.findall(file_contents) + \
+            week_pattern_2.findall(file_contents)
         week_sch_dict, week_dd_dict = ScheduleRuleset._idf_week_schedule_dictionary(
             week_sch_str, day_schedule_dict)
         # extract all of the ScheduleTypeLimit objects
@@ -947,6 +1086,17 @@ class ScheduleRuleset(object):
         constant_pattern = re.compile(r"(?i)(Schedule:Constant,[\s\S]*?;)")
         constant_props = tuple(parse_idf_string(idf_string) for
                                idf_string in constant_pattern.findall(file_contents))
+        # extract all of the Schedule:Compact objects and convert to ScheduleRuleset
+        conpact_pattern = re.compile(r"(?i)(Schedule:Compact,[\s\S]*?;)")
+        compact_props = (
+            tuple(
+                parse_idf_string(idf_string)
+                for idf_string in conpact_pattern.findall(file_contents)
+            )
+            if import_compact  # only if user chooses so.
+            else []
+        )
+
         # compile all of the ScheduleRuleset objects from extracted properties
         schedules = []
         for year_sch in year_props:
@@ -957,8 +1107,8 @@ class ScheduleRuleset(object):
                 st_date = Date(int(year_sch[i + 1]), int(year_sch[i + 2]))
                 end_date = Date(int(year_sch[i + 3]), int(year_sch[i + 4]))
                 for rule in rules:
-                    rule.start_date = st_date
                     rule.end_date = end_date
+                    rule.start_date = st_date
                 all_rules.extend(rules)
             # gather the other day schedules
             holiday_sch, summer_dd_sch, winter_dd_sch = week_dd_dict[year_sch[2]]
@@ -986,11 +1136,161 @@ class ScheduleRuleset(object):
             sch_ruleset = ScheduleRuleset.from_constant_value(
                 const_sch[0], sched_val, schedule_type)
             schedules.append(sch_ruleset)
+        for compact_sch in compact_props:
+            schedule_type = (
+                sch_type_dict[compact_sch[1]] if compact_sch[1] != "" else None
+            )
+            schedule_rules = []
+            holiday_schedule = None
+            winter_designday_schedule = None
+            summer_designday_schedule = None
+            start_date = Date.from_doy(1)
+            end_date = Date.from_doy(365)
+            untils = [Time(0, 0)]  # initialize list of until times.
+            n = -1  # initialize the n-th until time
+            rules = []  # initialize list of rules.
+            for field in compact_sch[2:]:
+                field = field.lower()
+                if "through" in field:
+                    # Each `through` field generates a new ScheduleRule
+                    # initialize rule with ScheduleDay as placeholder.
+                    rule = ScheduleRule(ScheduleDay(compact_sch[0], [0], [Time(0, 0)]))
+
+                    # start_date is either Jan 1st or end_date from previous
+                    # `through` field
+                    start_date = (
+                        start_date if end_date == Date.from_doy(365) else end_date
+                    )
+
+                    _, date = field.split(":")  # get end_date from field
+                    month, day = date.split("/")
+                    end_date = Date.from_dict({"month": int(month), "day": int(day)})
+                elif "for" in field:
+                    # reset values for new set; each `for` is a new rule
+                    n = -1  # reset the n-th until time
+                    untils = [Time(0, 0)]  # reset list of until times.
+                    rules = []  # reset list of rules for this `for` field.
+
+                    # Create a rule; all different `if` statements because we want to
+                    # catch more than one case,
+                    # eg. `For: Sunday Holidays AllOtherDays, !- Field 54`
+                    def create_rule_for(apply_to):
+                        """Return ScheduleRule with the `apply_to` rule set to True."""
+                        apply_to_attr_map = {
+                            "alldays": "apply_all",
+                            "weekdays": "apply_weekday",
+                            "weekends": "apply_weekend",
+                            "sunday": "apply_sunday",
+                            "monday": "apply_monday",
+                            "tuesday": "apply_tuesday",
+                            "wednesday": "apply_wednesday",
+                            "thursday": "apply_thursday",
+                            "friday": "apply_friday",
+                            "saturday": "apply_saturday",
+                        }
+                        rule = ScheduleRule(ScheduleDay(apply_to, [0], [Time(0, 0)]))
+                        setattr(rule, apply_to_attr_map[apply_to], True)
+                        return rule
+
+                    # Create rules for regular days
+                    for rule_name in [
+                        "alldays",
+                        "weekdays",
+                        "weekends",
+                        "sunday",
+                        "monday",
+                        "tuesday",
+                        "wednesday",
+                        "thursday",
+                        "friday",
+                        "saturday",
+                    ]:
+                        if rule_name in field:
+                            rule = create_rule_for(rule_name)
+                            rules.append(rule)
+
+                    # Create rules for holidays and designdays
+                    if "holiday" in field:
+                        rule = ScheduleRule(ScheduleDay("holiday", [0], [Time(0, 0)]))
+                        holiday_schedule = rule.schedule_day
+                        rules.append(rule)
+                    if "summerdesignday" in field:
+                        rule = ScheduleRule(
+                            ScheduleDay("summerdesignday", [0], [Time(0, 0)])
+                        )
+                        summer_designday_schedule = rule.schedule_day
+                        rules.append(rule)
+                    if "winterdesignday" in field:
+                        rule = ScheduleRule(
+                            ScheduleDay("winterdesignday", [0], [Time(0, 0)])
+                        )
+                        winter_designday_schedule = rule.schedule_day
+                        rules.append(rule)
+                    if "allotherdays" in field:
+                        rule = ScheduleRule(
+                            ScheduleDay("allotherdays", [0], [Time(0, 0)])
+                        )
+                        apply_mtx = [rul.week_apply_tuple for rul in schedule_rules]
+                        if not apply_mtx:  # situation if allotherdays is the only rule.
+                            rule.apply_all = True
+                        else:
+                            for j, dow in enumerate(zip(*apply_mtx)):
+                                if not any(dow):
+                                    rule.apply_day_by_dow(j + 1)
+                        rules.append(rule)
+
+                    for rule in rules:
+                        # for each created rule in this `for` field, add rules to
+                        # ScheduleRuleset list of rules and set start_date and end_date.
+                        if len(rule.days_applied) != 0:
+                            schedule_rules.append(rule)
+
+                        # set range for rule (from previous `through` field)
+                        rule.start_date = start_date
+                        rule.end_date = end_date
+                elif "until" in field:
+                    _, hour, min = field.split(":")  # get hour and minutes
+
+                    # value is applied `until` a certain Time, but `ScheduleDay` is
+                    # applied `from` a certain Time. Also, Time is 0:23 Hours while IDF
+                    # is 1:24 Hours.
+                    until = Time(int(hour) - 1, int(min))  # to 0:23 Hours repr
+                    untils.append(until)
+
+                    # increment n
+                    n += 1
+                elif "interpolate" in field:
+                    # Set interpolate on all rules for this `for` field
+                    _, interpolate = field.split(":")
+                    for rule in rules:
+                        rule.schedule_day.interpolate = interpolate != "no"
+                else:
+                    begin = untils[n]  # index list of `until` times
+                    for rule in rules:
+                        # apply field value for each created rules; try to replace the
+                        # placeholder value first, else add the value.
+                        try:
+                            rule.schedule_day.replace_value_by_time(begin, float(field))
+                        except ValueError:
+                            rule.schedule_day.add_value(float(field), begin)
+            default_day_schedule = schedule_rules[0].schedule_day
+            sch_ruleset = ScheduleRuleset(
+                default_day_schedule=default_day_schedule,
+                identifier=compact_sch[0],
+                schedule_type_limit=schedule_type,
+                schedule_rules=schedule_rules[1:]
+            )
+            ScheduleRuleset._apply_designdays_with_check(
+                sch_ruleset,
+                holiday_schedule,
+                summer_designday_schedule,
+                winter_designday_schedule)
+            schedules.append(sch_ruleset)
         return schedules
 
     @staticmethod
     def average_schedules(identifier, schedules, weights=None, timestep_resolution=1):
-        """Create a ScheduleRuleset that is a weighted average between other ScheduleRulesets.
+        """Create a ScheduleRuleset that is a weighted average between ScheduleRulesets.
 
         Args:
             identifier: Text string for a unique ID for the new unique ScheduleRuleset.
@@ -1039,10 +1339,12 @@ class ScheduleRuleset(object):
             week_schedules = []
             for i, rule_indices in enumerate(unique_rule_sets):
                 week_identifier = '{}_{}'.format(identifier, i)
-                week_sched = ScheduleRuleset._get_avg_week(week_identifier, schedules, weights,
-                                                           timestep_resolution, rule_indices)
+                week_sched = ScheduleRuleset._get_avg_week(
+                    week_identifier, schedules, weights,
+                    timestep_resolution, rule_indices
+                )
                 week_schedules.append(week_sched)
-            # create a dictionary mapping unique rule index lists to average week schedules
+            # create a dictionary mapping unique rule indices to average week schedules
             rule_set_map = {}
             for rule_i, week_sched in zip(unique_rule_sets, week_schedules):
                 rule_set_map[rule_i] = week_sched
@@ -1149,7 +1451,7 @@ class ScheduleRuleset(object):
             week_fields.append(self._winter_designday_schedule.identifier)
         else:
             week_fields.append(self._default_day_schedule.identifier)
-        for i in range(2):  # add the extra 2 custom days that are rarely used in E+
+        for _ in range(2):  # add the extra 2 custom days that are rarely used in E+
             week_fields.append(self.default_day_schedule.identifier)
         return week_fields
 
@@ -1339,6 +1641,12 @@ class ScheduleRuleset(object):
                 return True
         return False
 
+    def __bool__(self):
+        return True
+
+    def __nonzero__(self):
+        return True
+
     def __len__(self):
         return len(self._schedule_rules)
 
@@ -1376,6 +1684,8 @@ class ScheduleRuleset(object):
             [rule.duplicate() for rule in self._schedule_rules],
             self._schedule_type_limit, holiday, summer, winter)
         new_obj._display_name = self._display_name
+        new_obj._user_data = None if self._user_data is None else self._user_data.copy()
+        new_obj._properties._duplicate_extension_attr(self._properties)
         return new_obj
 
     def ToString(self):
@@ -1383,7 +1693,6 @@ class ScheduleRuleset(object):
         return self.__repr__()
 
     def __repr__(self):
-        return 'ScheduleRuleset:\n name: {}\n default_day: {}\n' \
-            ' schedule_rules:\n  {}'.format(
-                self.identifier, self.default_day_schedule.identifier,
-                '\n  '.join([rule.schedule_day.identifier for rule in self._schedule_rules]))
+        return 'ScheduleRuleset: {} [default day: {}] [{} rules]'.format(
+            self.display_name, self.default_day_schedule.display_name,
+            len(self._schedule_rules))
