@@ -81,8 +81,8 @@ class SimulationParameter(object):
     @output.setter
     def output(self, value):
         if value is not None:
-            assert isinstance(value, SimulationOutput), 'Expected SimulationOutput for ' \
-                'SimulationParameter output. Got {}.'.format(type(value))
+            assert isinstance(value, SimulationOutput), 'Expected SimulationOutput ' \
+                'for SimulationParameter output. Got {}.'.format(type(value))
             self._output = value
         else:
             self._output = SimulationOutput()
@@ -103,8 +103,7 @@ class SimulationParameter(object):
 
     @property
     def timestep(self):
-        """Get or set a integer for the number of days with unique shadow calculations.
-        """
+        """Get or set a integer for the number of simulation timesteps per hour."""
         return self._timestep
 
     @timestep.setter
@@ -238,7 +237,7 @@ class SimulationParameter(object):
             identifier: Text string for to be used as a unique identifier for the
                 building object.
         """
-        values = (identifier, -self.north_angle, self.terrain_type, '', '',
+        values = (identifier, self.north_angle, self.terrain_type, '', '',
                   self.shadow_calculation.solar_distribution)
         comments = ('name',
                     'clockwise north axis',
@@ -248,6 +247,21 @@ class SimulationParameter(object):
                     'solar distribution')
         return generate_idf_string('Building', values, comments)
 
+    def water_mains_idf(self):
+        """Get an IDF string for the water mains object."""
+        # TODO: Remove generation of mains temps from des days if bug is fixed in E+ 9.7
+        if len(self.sizing_parameter.design_days) > 0:
+            db_temps = [dday.dry_bulb_condition.dry_bulb_max
+                        for dday in self.sizing_parameter.design_days]
+            avg_temp = (max(db_temps) + min(db_temps)) / 2
+            return generate_idf_string(
+                'Site:WaterMainsTemperature', ('Correlation', '', str(avg_temp), '4'),
+                ('calculation method', 'schedule', 'average temp', 'delta temp'))
+        else:
+            return generate_idf_string(
+                'Site:WaterMainsTemperature', ('CorrelationFromWeatherFile',),
+                ('calculation method',))
+
     @classmethod
     def from_idf(cls, idf_string):
         """Create a SimulationParameter object from an EnergyPlus IDF text string.
@@ -263,6 +277,7 @@ class SimulationParameter(object):
         """
         # Regex patterns for the various objects comprising the SimulationParameter
         out_style_pattern = re.compile(r"(?i)(OutputControl:Table:Style,[\s\S]*?;)")
+        unmet_pattern = re.compile(r"(?i)(OutputControl:ReportingTolerances,[\s\S]*?;)")
         out_var_pattern = re.compile(r"(?i)(Output:Variable,[\s\S]*?;)")
         out_report_pattern = re.compile(r"(?i)(Output:Table:SummaryReports,[\s\S]*?;)")
         sqlite_pattern = re.compile(r"(?i)(Output:SQLite,[\s\S]*?;)")
@@ -282,13 +297,18 @@ class SimulationParameter(object):
             out_style_str = out_style_pattern.findall(idf_string)[0]
         except IndexError:  # No Table:Style in the file.
             out_style_str = None
+        try:
+            out_unmet_str = unmet_pattern.findall(idf_string)[0]
+        except IndexError:  # No ReportingTolerances in the file.
+            out_unmet_str = None
         try:
             out_report_str = out_report_pattern.findall(idf_string)[0]
         except IndexError:  # No SummaryReports in the file. Default to None.
             out_report_str = None
         sqlite = True if len(sqlite_pattern.findall(idf_string)) != 0 else False
         output = SimulationOutput.from_idf(
-            out_style_str, out_var_pattern.findall(idf_string), out_report_str, sqlite)
+            out_style_str, out_var_pattern.findall(idf_string), out_report_str,
+            out_unmet_str, sqlite)
 
         # process the RunPeriod within the idf_string
         try:
@@ -324,7 +344,7 @@ class SimulationParameter(object):
         try:
             bldg_str = bldg_pattern.findall(idf_string)[0]
             bldg_prop = parse_idf_string(bldg_str)
-            north_angle = -float(bldg_prop[1]) if bldg_prop[1] != '' else 0
+            north_angle = float(bldg_prop[1]) if bldg_prop[1] != '' else 0
             terrain = bldg_prop[2].title() if bldg_prop[2] != '' else 'Suburbs'
             solar_dist = bldg_prop[5] if bldg_prop[5] != '' else 'FullExterior'
         except IndexError:  # No Building in the file. Use honeybee default.
@@ -350,8 +370,8 @@ class SimulationParameter(object):
             location = loc_pattern.findall(idf_string)[0]
         except IndexError:  # No Site:Location in the file.
             location = None
-        sizing_par = SizingParameter.from_idf([dy[0] for dy in ddy_p.findall(idf_string)],
-                                              sizing_str, location)
+        sizing_par = SizingParameter.from_idf(
+            [dy[0] for dy in ddy_p.findall(idf_string)], sizing_str, location)
 
         return cls(output, run_period, timestep, sim_control, shadow_calc,
                    sizing_par, north_angle, terrain)
@@ -419,7 +439,7 @@ class SimulationParameter(object):
                          '!-   ==========================================\n']
 
         # add the outputs requested
-        table_style, output_vars, reports, sqlite, rdd, surfaces = self.output.to_idf()
+        table_style, output_vars, reports, sqlite, rdd, unmet_tol = self.output.to_idf()
         sim_param_str.append(table_style)
         if output_vars is not None:
             sim_param_str.append('\n\n'.join(output_vars))
@@ -428,7 +448,7 @@ class SimulationParameter(object):
         if sqlite is not None:
             sim_param_str.append(sqlite)
         sim_param_str.append(rdd)
-        sim_param_str.append(surfaces)
+        sim_param_str.append(unmet_tol)
 
         # add simulation settings
         sim_param_str.append(self.simulation_control.to_idf())
@@ -453,8 +473,9 @@ class SimulationParameter(object):
         # write the global geometry rules
         sim_param_str.append(self.global_geometry_rules)
 
-        # write the Building object
+        # write the Building and water mains object
         sim_param_str.append(self.building_idf(identifier))
+        sim_param_str.append(self.water_mains_idf())
 
         return '\n\n'.join(sim_param_str)
 
