@@ -2,26 +2,21 @@
 """EnergyPlus Simulation Run Period."""
 from __future__ import division
 
+from ladybug.dt import Date
+from ladybug.analysisperiod import AnalysisPeriod
+from honeybee.typing import valid_string
+
 from .daylightsaving import DaylightSavingTime
 from ..reader import parse_idf_string
 from ..writer import generate_idf_string
 
-from honeybee.typing import valid_string
-
-from ladybug.analysisperiod import AnalysisPeriod
-from ladybug.dt import Date
-
 
 class RunPeriod(object):
     """EnergyPlus Simulation Run Period.
 
     Args:
-        start_date: A ladybug Date object for the start of the run period.
-            Must be before the end date and have a leap_year property matching the
-            end_date. Default: 1 Jan
-        end_date: A ladybug Date object for the end of the run period.
-            Must be after the start date and have a leap_year property matching the
-            start_date. Default: 31 Dec
+        start_date: A ladybug Date for the start of the run period. (Default: 1 Jan)
+        end_date: A ladybug Date for the end of the run period. (Default: 31 Dec)
         start_day_of_week: Text for the day of the week on which the simulation
             starts. Default: 'Sunday'. Choose from the following:
 
@@ -79,7 +74,6 @@ class RunPeriod(object):
             self._start_date = value
         else:
             self._start_date = Date(1, 1)
-        self._check_start_before_end()
 
     @property
     def end_date(self):
@@ -93,7 +87,6 @@ class RunPeriod(object):
             self._end_date = value
         else:
             self._end_date = Date(12, 31)
-        self._check_start_before_end()
 
     @property
     def start_day_of_week(self):
@@ -304,6 +297,23 @@ class RunPeriod(object):
 
         return cls(start_date, end_date, start_day_of_week, holidays, daylight_saving)
 
+    @classmethod
+    def from_string(cls, run_period_string):
+        """Create an RunPeriod object from an RunPeriod string."""
+        # split the various objects that make us the run period
+        run_per_objs = run_period_string.split('\n\n')
+        holidays, dl_saving = None, None
+        if len(run_per_objs) > 1:
+            for obj in run_per_objs:
+                if obj.startswith('RunPeriodControl:DaylightSavingTime'):
+                    dl_saving = obj
+                elif obj.startswith('RunPeriodControl:SpecialDays'):
+                    holidays = []
+                    lines = obj.split('\n')
+                    for i in range(0, len(lines), 3):
+                        holidays.append('\n'.join(lines[i:i+3]))
+        return cls.from_idf(run_per_objs[0], holidays, dl_saving)
+
     def to_idf(self):
         """Get an EnergyPlus string representation of the RunPeriod.
 
@@ -312,9 +322,9 @@ class RunPeriod(object):
 
             -   run_period: An IDF string representation of the RunPeriod object.
 
-            -   holidays: A list of IDF RunPeriodControl:SpecialDays strings that represent
-                the holidays applied to the simulation. Will be None if no holidays
-                are applied to this RunPeriod.
+            -   holidays: A list of IDF RunPeriodControl:SpecialDays strings that
+                represent the holidays applied to the simulation. Will be None
+                if no holidays are applied to this RunPeriod.
 
             -   daylight_saving_time: An IDF RunPeriodControl:DaylightSavingTime string
                 that notes the start and ends dates of Daylight Savings time. Will be
@@ -323,10 +333,11 @@ class RunPeriod(object):
         year = 2016 if self.is_leap_year else 2017
         values = ('CustomRunPeriod', self.start_date.month, self.start_date.day, year,
                   self.end_date.month, self.end_date.day, year,
-                  self.start_day_of_week, 'Yes', 'Yes')
+                  self.start_day_of_week, 'No', 'No', 'No', 'Yes', 'Yes')
         comments = ('name', 'start month', 'start day', 'start year',
                     'end month', 'end day', 'end year', 'start day of week',
-                    'use weather file holidays', 'use weather file daylight savings')
+                    'use weather file holidays', 'use weather file daylight savings',
+                    'apply weekend holiday', 'use epw rain', 'use epw snow')
         run_period = generate_idf_string('RunPeriod', values, comments)
 
         holidays = [self._holiday_to_idf(hol, i) for i, hol in
@@ -356,13 +367,6 @@ class RunPeriod(object):
         """Get a copy of this object."""
         return self.__copy__()
 
-    def _check_start_before_end(self):
-        """Check that the start_date is before the end_date."""
-        assert self.start_date.leap_year is self.end_date.leap_year, \
-            'RunPeriod start_date.leap_year must match the end_date.leap_year'
-        assert self._start_date <= self._end_date, 'RunPeriod start_date must come ' \
-            'before end_date. {} comes after {}.'.format(self.start_date, self.end_date)
-
     @staticmethod
     def _check_date(date, date_name='date'):
         assert isinstance(date, Date), 'Expected ladybug Date for ' \
@@ -402,4 +406,10 @@ class RunPeriod(object):
         return not self.__eq__(other)
 
     def __repr__(self):
-        return self.to_idf()[0]
+        """Represent run period."""
+        run_per, holidays, dl_saving = self.to_idf()
+        if holidays is not None:
+            run_per = run_per + '\n\n' + '\n'.join(holidays)
+        if dl_saving is not None:
+            run_per = run_per + '\n\n' + dl_saving
+        return run_per
