@@ -5,7 +5,7 @@ from __future__ import division
 from ..reader import parse_idf_string
 from ..writer import generate_idf_string
 
-from honeybee.typing import float_positive
+from honeybee.typing import float_positive, valid_ep_string
 
 from ladybug.ddy import DDY
 from ladybug.designday import DesignDay
@@ -17,26 +17,104 @@ class SizingParameter(object):
 
     Args:
         design_days: An array of Ladybug DesignDay objects that represent the
-            criteria for which the HVAC systems will be sized. Default: None.
+            criteria for which the HVAC systems will be sized. (Default: None).
         heating_factor: A number that will get multiplied by the peak heating load
             for each zone in the model in order to size the heating system for
-            the model. Must be greater than 0. Default: 1.25.
+            the model. Must be greater than 0. (Default: 1.25).
         cooling_factor: A number that will get multiplied by the peak cooling load
             for each zone in the model in order to size the cooling system for
-            the model. Must be greater than 0. Default: 1.15.
+            the model. Must be greater than 0. (Default: 1.15).
+        efficiency_standard: Text to specify the efficiency standard, which will
+            automatically set the efficiencies of all HVAC equipment when provided.
+            Note that providing a standard here will cause the OpenStudio translation
+            process to perform an additional sizing calculation with EnergyPlus,
+            which is needed since the default efficiencies of equipment vary depending
+            on their size. THIS WILL SIGNIFICANTLY INCREASE TRANSLATION TIME.
+            However, it is often worthwhile when the goal is to match the
+            HVAC specification with a particular standard. (Default: None).
+            Choose from the following.
+
+            * DOE_Ref_Pre_1980
+            * DOE_Ref_1980_2004
+            * ASHRAE_2004
+            * ASHRAE_2007
+            * ASHRAE_2010
+            * ASHRAE_2013
+            * ASHRAE_2016
+            * ASHRAE_2019
+
+        climate_zone: Text indicating the ASHRAE climate zone to be used with the
+            efficiency_standard. When unspecified, the climate zone will be
+            inferred from the design days. This input can be a single
+            integer (in which case it is interpreted as A) or it can include the
+            A, B, or C qualifier (eg. 3C).
+        building_type: Text for the building type to be used in the efficiency_standard.
+            If the type is not recognized or is None, it will be assumed that the
+            building is a generic NonResidential. The following have meaning
+            for the standard.
+
+            * NonResidential
+            * Residential
+            * MidriseApartment
+            * HighriseApartment
+            * LargeOffice
+            * MediumOffice
+            * SmallOffice
+            * Retail
+            * StripMall
+            * PrimarySchool
+            * SecondarySchool
+            * SmallHotel
+            * LargeHotel
+            * Hospital
+            * Outpatient
+            * Warehouse
+            * SuperMarket
+            * FullServiceRestaurant
+            * QuickServiceRestaurant
+            * Laboratory
+            * Courthouse
+        
+        bypass_efficiency_sizing: A boolean to indicate whether the efficiency
+            standard should trigger an sizing run that sets the efficiencies
+            of all HVAC equipment in the Model (False) or the standard should
+            only be written into the OSM and the sizing run should be
+            bypassed (True). Bypassing the sizing run is useful when you only
+            want to check that the overall HVAC system architecture is correct
+            and you do not want to wait the extra time that it takes to run the
+            sizing calculation. (Default: False).
 
     Properties:
         * design_days
         * heating_factor
         * cooling_factor
+        * efficiency_standard
+        * climate_zone
+        * building_type
+        * bypass_efficiency_sizing
     """
-    __slots__ = ('_design_days', '_heating_factor', '_cooling_factor')
+    __slots__ = ('_design_days', '_heating_factor', '_cooling_factor',
+                 '_efficiency_standard', '_climate_zone', '_building_type',
+                 '_bypass_efficiency_sizing')
+    STANDARDS = ('DOE_Ref_Pre_1980', 'DOE_Ref_1980_2004', 'ASHRAE_2004', 'ASHRAE_2007',
+                 'ASHRAE_2010', 'ASHRAE_2013', 'ASHRAE_2016', 'ASHRAE_2019')
+    CLIMATE_ZONES = (
+        '0A', '1A', '2A', '3A', '4A', '5A', '6A',
+        '0B', '1B', '2B', '3B', '4B', '5B', '6B',
+        '3C', '4C', '5C', '7', '8'
+    )
 
-    def __init__(self, design_days=None, heating_factor=1.25, cooling_factor=1.15):
+    def __init__(self, design_days=None, heating_factor=1.25, cooling_factor=1.15,
+                 efficiency_standard=None, climate_zone=None, building_type=None,
+                 bypass_efficiency_sizing=False):
         """Initialize SizingParameter."""
         self.design_days = design_days
         self.heating_factor = heating_factor
         self.cooling_factor = cooling_factor
+        self.efficiency_standard = efficiency_standard
+        self.climate_zone = climate_zone
+        self.building_type = building_type
+        self.bypass_efficiency_sizing = bypass_efficiency_sizing
 
     @property
     def design_days(self):
@@ -79,6 +157,95 @@ class SizingParameter(object):
         self._cooling_factor = float_positive(value, 'sizing parameter cooling factor')
         assert self._cooling_factor != 0, 'SizingParameter cooling factor cannot be 0.'
 
+    @property
+    def efficiency_standard(self):
+        """Get or set text for the efficiency standard.
+
+        When specified, this will automatically set the efficiencies of all HVAC
+        equipment. Note that setting this variable will cause the OpenStudio
+        translation process to perform an additional sizing calculation with EnergyPlus,
+        which is needed since the default efficiencies of equipment vary depending
+        on their size. So THIS WILL SIGNIFICANTLY INCREASE TRANSLATION TIME.
+        However, it is often worthwhile when the goal is to match the
+        HVAC specification with a particular standard. Choose from the following.
+
+        * DOE_Ref_Pre_1980
+        * DOE_Ref_1980_2004
+        * ASHRAE_2004
+        * ASHRAE_2007
+        * ASHRAE_2010
+        * ASHRAE_2013
+        * ASHRAE_2016
+        * ASHRAE_2019
+        """
+        return self._efficiency_standard
+
+    @efficiency_standard.setter
+    def efficiency_standard(self, value):
+        if value:
+            clean_input = valid_ep_string(value, 'efficiency_standard').lower()
+            for key in self.STANDARDS:
+                if key.lower() == clean_input:
+                    value = key
+                    break
+            else:
+                raise ValueError(
+                    'Efficiency standard "{}" is not recognized.\nChoose from the '
+                    'following:\n{}'.format(value, self.STANDARDS))
+        else:
+            value = None
+        self._efficiency_standard = value
+
+    @property
+    def climate_zone(self):
+        """Get or set text for the climate zone associated with the efficiency standard.
+
+        When unspecified, the climate zone will be inferred from the design days.
+        This input can be a single integer (in which case it is interpreted as A)
+        or it can include the A, B, or C qualifier (eg. 3C).
+        """
+        return self._climate_zone
+
+    @climate_zone.setter
+    def climate_zone(self, value):
+        if value:
+            value = valid_ep_string(value, 'climate_zone').upper()
+            if len(value) == 1 and value not in ('7', '8'):
+                value = '{}A'.format(value)
+            if value not in self.CLIMATE_ZONES:
+                raise ValueError(
+                    'Efficiency climate zone "{}" is not recognized.\nChoose from the '
+                    'following:\n{}'.format(value, self.CLIMATE_ZONES))
+        else:
+            value = None
+        self._climate_zone = value
+
+    @property
+    def building_type(self):
+        """Get or set text for the building type associated with the efficiency standard.
+
+        If the type is not recognized or is None, it will be assumed that the
+        building is a generic NonResidential.
+        """
+        return self._building_type
+
+    @building_type.setter
+    def building_type(self, value):
+        if value:
+            value = valid_ep_string(value, 'building_type')
+        else:
+            value = None
+        self._building_type = value
+
+    @property
+    def bypass_efficiency_sizing(self):
+        """Get or set a boolean for whether efficiency standard triggers a sizing run."""
+        return self._bypass_efficiency_sizing
+
+    @bypass_efficiency_sizing.setter
+    def bypass_efficiency_sizing(self, value):
+        self._bypass_efficiency_sizing = bool(value)
+
     def add_design_day(self, design_day):
         """Add a ladybug DesignDay to this object's design_days.
 
@@ -146,7 +313,7 @@ class SizingParameter(object):
         """Remove a single DesignDay from this object using an index.
 
         Args:
-            design_day_index: An interger for the index of the DesignDay to remove.
+            design_day_index: An integer for the index of the DesignDay to remove.
         """
         del self._design_days[design_day_index]
 
@@ -230,7 +397,11 @@ class SizingParameter(object):
             "type": "SizingParameter",
             "design_days": [],  # Array of Ladybug DesignDay dictionaries
             "heating_factor": 1.25,
-            "cooling_factor": 1.15
+            "cooling_factor": 1.15,
+            "efficiency_standard": "ASHRAE_2004",  # standard for HVAC efficiency
+            "climate_zone": "5A",  # climate zone for HVAC efficiency
+            "building_type": "LargeOffice",  # building type for HVAC efficiency
+            "bypass_efficiency_sizing": False  # bypass the efficiency sizing run
             }
         """
         assert data['type'] == 'SizingParameter', \
@@ -240,7 +411,12 @@ class SizingParameter(object):
             design_days = [DesignDay.from_dict(dday) for dday in data['design_days']]
         heating_factor = data['heating_factor'] if 'heating_factor' in data else 1.25
         cooling_factor = data['cooling_factor'] if 'cooling_factor' in data else 1.15
-        return cls(design_days, heating_factor, cooling_factor)
+        es = data['efficiency_standard'] if 'efficiency_standard' in data else None
+        cz = data['climate_zone'] if 'climate_zone' in data else None
+        bt = data['building_type'] if 'building_type' in data else None
+        bes = data['bypass_efficiency_sizing'] \
+            if 'bypass_efficiency_sizing' in data else False
+        return cls(design_days, heating_factor, cooling_factor, es, cz, bt, bes)
 
     def to_idf(self):
         """Get an EnergyPlus string representation of the SizingParameters.
@@ -268,8 +444,16 @@ class SizingParameter(object):
             'heating_factor': self.heating_factor,
             'cooling_factor': self.cooling_factor
         }
+        if self.efficiency_standard is not None:
+            siz_par['efficiency_standard'] = self.efficiency_standard
+        if self.climate_zone is not None:
+            siz_par['climate_zone'] = self.climate_zone
+        if self.building_type is not None:
+            siz_par['building_type'] = self.building_type
         if len(self._design_days) != 0:
             siz_par['design_days'] = [dday.to_dict(False) for dday in self.design_days]
+        if self.bypass_efficiency_sizing:
+            siz_par['bypass_efficiency_sizing'] = self.bypass_efficiency_sizing
         return siz_par
 
     def to_ddy(self):
@@ -290,13 +474,16 @@ class SizingParameter(object):
         return self.__repr__()
 
     def __copy__(self):
-        return SizingParameter([dday.duplicate() for dday in self._design_days],
-                               self.heating_factor, self.cooling_factor)
+        return SizingParameter(
+            [dday.duplicate() for dday in self._design_days],
+            self.heating_factor, self.cooling_factor, self.efficiency_standard,
+            self.climate_zone, self.building_type, self.bypass_efficiency_sizing)
 
     def __key(self):
         """A tuple based on the object properties, useful for hashing."""
         return tuple(hash(dday) for dday in self._design_days) + \
-            (self.heating_factor, self.cooling_factor)
+            (self.heating_factor, self.cooling_factor, self.efficiency_standard,
+             self.climate_zone, self.building_type, self.bypass_efficiency_sizing)
 
     def __hash__(self):
         return hash(self.__key())
@@ -325,5 +512,5 @@ class SizingParameter(object):
         return item in self._design_days
 
     def __repr__(self):
-        return 'SizingParameter:\n {} design days\n heating: {}\n cooling: ' \
-            '{}'.format(len(self._design_days), self.heating_factor, self.cooling_factor)
+        return 'SizingParameter: [{} design days] [heating: {}] [cooling: {}]'.format(
+            len(self._design_days), self.heating_factor, self.cooling_factor)
